@@ -75,6 +75,12 @@ function configure_default_backends {
         iniset $MANILA_CONF $group_name service_instance_user $MANILA_SERVICE_INSTANCE_USER
         iniset $MANILA_CONF $group_name driver_handles_share_servers True
 
+        if [ "$SHARE_DRIVER" == $MANILA_CONTAINER_DRIVER ]; then
+            iniset $MANILA_CONF $group_name network_api_class $MANILA_NETWORK_API_CLASS
+            iniset $MANILA_CONF $group_name neutron_host_id $(hostname)
+            iniset $MANILA_CONF $group_name neutron_vnic_type $MANILA_NEUTRON_VNIC_TYPE
+        fi
+
         if [ $(trueorfalse False MANILA_USE_SERVICE_INSTANCE_PASSWORD) == True ]; then
             iniset $MANILA_CONF $group_name service_instance_password $MANILA_SERVICE_INSTANCE_PASSWORD
         fi
@@ -118,9 +124,6 @@ function set_cinder_quotas {
 
 # configure_manila - Set config files, create data dirs, etc
 function configure_manila {
-    setup_develop $MANILA_DIR
-    setup_develop $MANILACLIENT_DIR
-
     if [[ ! -d $MANILA_CONF_DIR ]]; then
         sudo mkdir -p $MANILA_CONF_DIR
     fi
@@ -178,6 +181,16 @@ function configure_manila {
     iniset $MANILA_CONF DEFAULT osapi_share_extension manila.api.contrib.standard_extensions
     iniset $MANILA_CONF DEFAULT state_path $MANILA_STATE_PATH
     iniset $MANILA_CONF DEFAULT default_share_type $MANILA_DEFAULT_SHARE_TYPE
+    # TODO(vponomaryov): revert following back in Pike
+    # iniset $MANILA_CONF DEFAULT default_share_group_type $MANILA_DEFAULT_SHARE_GROUP_TYPE
+
+    if ! [[ -z $MANILA_SHARE_MIGRATION_PERIOD_TASK_INTERVAL ]]; then
+        iniset $MANILA_CONF DEFAULT migration_driver_continue_update_interval $MANILA_SHARE_MIGRATION_PERIOD_TASK_INTERVAL
+    fi
+
+    if ! [[ -z $MANILA_DATA_COPY_CHECK_HASH ]]; then
+        iniset $MANILA_CONF DEFAULT check_hash $MANILA_DATA_COPY_CHECK_HASH
+    fi
 
     iniset $MANILA_CONF DEFAULT enabled_share_protocols $MANILA_ENABLED_SHARE_PROTOCOLS
 
@@ -248,31 +261,31 @@ function configure_manila {
 }
 
 
-function configure_manila_ui {
-    if is_service_enabled horizon && [ "$MANILA_UI_ENABLED" = "True" ]; then
-        # NOTE(vponomaryov): workaround for devstack bug: 1540328
-        # where devstack install 'test-requirements' but should not do it
-        # for manila-ui project as it installs Horizon from url.
-        # Remove following two 'mv' commands when mentioned bug is fixed.
-        mv $MANILA_UI_DIR/test-requirements.txt $MANILA_UI_DIR/_test-requirements.txt
-
-        setup_develop $MANILA_UI_DIR
-        cp $MANILA_UI_DIR/manila_ui/enabled/_90_manila_*.py $HORIZON_DIR/openstack_dashboard/local/enabled
-
-        mv $MANILA_UI_DIR/_test-requirements.txt $MANILA_UI_DIR/test-requirements.txt
+function create_manila_service_keypair {
+    if is_service_enabled nova; then
+        local keypair_exists=$( openstack keypair list | grep " $MANILA_SERVICE_KEYPAIR_NAME " )
+        if [[ -z $keypair_exists ]]; then
+            openstack keypair create $MANILA_SERVICE_KEYPAIR_NAME --public-key $MANILA_PATH_TO_PUBLIC_KEY
+        fi
     fi
 }
 
 
-function create_manila_service_keypair {
-    nova keypair-add $MANILA_SERVICE_KEYPAIR_NAME --pub-key $MANILA_PATH_TO_PUBLIC_KEY
+function is_driver_enabled {
+    driver_name=$1
+    for BE in ${MANILA_ENABLED_BACKENDS//,/ }; do
+        share_driver=$(iniget $MANILA_CONF $BE share_driver)
+        if [ "$share_driver" == "$driver_name" ]; then
+            return 0
+        fi
+    done
+    return 1
 }
 
 
 # create_service_share_servers - creates service Nova VMs, one per generic
 # driver, and only if it is configured to mode without handling of share servers.
 function create_service_share_servers {
-    private_net_id=$(nova net-list | grep ' private ' | get_field 1)
     created_admin_network=false
     for BE in ${MANILA_ENABLED_BACKENDS//,/ }; do
         driver_handles_share_servers=$(iniget $MANILA_CONF $BE driver_handles_share_servers)
@@ -281,27 +294,47 @@ function create_service_share_servers {
         if [[ $share_driver == $generic_driver ]]; then
             if [[ $(trueorfalse False driver_handles_share_servers) == False ]]; then
                 vm_name='manila_service_share_server_'$BE
-                nova boot $vm_name \
-                    --flavor $MANILA_SERVICE_VM_FLAVOR_NAME \
-                    --image $MANILA_SERVICE_IMAGE_NAME \
-                    --nic net-id=$private_net_id \
-                    --security-groups $MANILA_SERVICE_SECGROUP \
-                    --key-name $MANILA_SERVICE_KEYPAIR_NAME
+                local vm_exists=$( openstack server list --all-projects | grep " $vm_name " )
+                if [[ -z $vm_exists ]]; then
+                    private_net_id=$(openstack network show $PRIVATE_NETWORK_NAME -f value -c id)
+                    vm_id=$(openstack server create $vm_name \
+                        --flavor $MANILA_SERVICE_VM_FLAVOR_NAME \
+                        --image $MANILA_SERVICE_IMAGE_NAME \
+                        --nic net-id=$private_net_id \
+                        --security-group $MANILA_SERVICE_SECGROUP \
+                        --key-name $MANILA_SERVICE_KEYPAIR_NAME \
+                        | grep ' id ' | get_field 2)
+                else
+                    vm_id=$(openstack server show $vm_name -f value -c id)
+                fi
 
-                vm_id=$(nova show $vm_name | grep ' id ' | get_field 2)
+                floating_ip=$(openstack floating ip create $PUBLIC_NETWORK_NAME --subnet $PUBLIC_SUBNET_NAME | grep 'floating_ip_address' | get_field 2)
+                openstack server add floating ip $vm_id $floating_ip
 
                 iniset $MANILA_CONF $BE service_instance_name_or_id $vm_id
-                iniset $MANILA_CONF $BE service_net_name_or_ip private
-                iniset $MANILA_CONF $BE tenant_net_name_or_ip private
+                iniset $MANILA_CONF $BE service_net_name_or_ip $floating_ip
+                iniset $MANILA_CONF $BE tenant_net_name_or_ip $PRIVATE_NETWORK_NAME
             else
                 if is_service_enabled neutron; then
-                    if [ $created_admin_network == false ]; then
-                        admin_net_id=$(neutron net-create --tenant-id $TENANT_ID admin_net | grep ' id ' | get_field 2)
-                        admin_subnet_id=$(neutron subnet-create --tenant-id $TENANT_ID --ip_version 4 --no-gateway --name admin_subnet --subnetpool None $admin_net_id $FIXED_RANGE | grep ' id ' | get_field 2)
-                        created_admin_network=true
+                    if ! [[ -z $MANILA_ADMIN_NET_RANGE ]]; then
+                        if [ $created_admin_network == false ]; then
+                            project_id=$(openstack project show $SERVICE_PROJECT_NAME -c id -f value)
+                            local admin_net_id=$( openstack network show admin_net -f value -c id )
+                            if [[ -z $admin_net_id ]]; then
+                                openstack network create admin_net --project $project_id
+                                admin_net_id=$(openstack network show admin_net -f value -c id)
+                            fi
+
+                            local admin_subnet_id=$( openstack subnet show admin_subnet -f value -c id )
+                            if [[ -z $admin_subnet_id ]]; then
+                                openstack subnet create admin_subnet --project $project_id --ip-version 4 --network $admin_net_id --gateway None --subnet-range $MANILA_ADMIN_NET_RANGE
+                                admin_subnet_id=$(openstack subnet show admin_subnet -f value -c id)
+                            fi
+                            created_admin_network=true
+                        fi
+                        iniset $MANILA_CONF $BE admin_network_id $admin_net_id
+                        iniset $MANILA_CONF $BE admin_subnet_id $admin_subnet_id
                     fi
-                    iniset $MANILA_CONF $BE admin_network_id $admin_net_id
-                    iniset $MANILA_CONF $BE admin_subnet_id $admin_subnet_id
                 fi
             fi
         fi
@@ -318,30 +351,41 @@ function configure_data_service_generic_driver {
         if [[ $(trueorfalse False driver_handles_share_servers) == False ]]; then
             iniset $MANILA_CONF DEFAULT data_node_access_ip $PUBLIC_NETWORK_GATEWAY
         else
-            iniset $MANILA_CONF DEFAULT data_node_access_ip $FIXED_RANGE
+            if ! [[ -z $MANILA_DATA_NODE_IP ]]; then
+                iniset $MANILA_CONF DEFAULT data_node_access_ip $MANILA_DATA_NODE_IP
+            fi
         fi
     fi
 }
 # create_manila_service_flavor - creates flavor, that will be used by backends
 # with configured generic driver to boot Nova VMs with.
 function create_manila_service_flavor {
-    # Create flavor for Manila's service VM
-    nova flavor-create \
-        $MANILA_SERVICE_VM_FLAVOR_NAME \
-        $MANILA_SERVICE_VM_FLAVOR_REF \
-        $MANILA_SERVICE_VM_FLAVOR_RAM \
-        $MANILA_SERVICE_VM_FLAVOR_DISK \
-        $MANILA_SERVICE_VM_FLAVOR_VCPUS
+    if is_service_enabled nova; then
+        local flavor_exists=$( openstack flavor list | grep " $MANILA_SERVICE_VM_FLAVOR_NAME " )
+        if [[ -z $flavor_exists ]]; then
+            # Create flavor for Manila's service VM
+            openstack flavor create \
+                $MANILA_SERVICE_VM_FLAVOR_NAME \
+                --id $MANILA_SERVICE_VM_FLAVOR_REF \
+                --ram $MANILA_SERVICE_VM_FLAVOR_RAM \
+                --disk $MANILA_SERVICE_VM_FLAVOR_DISK \
+                --vcpus $MANILA_SERVICE_VM_FLAVOR_VCPUS
+        fi
+    fi
 }
 
 # create_manila_service_image - creates image, that will be used by backends
 # with configured generic driver to boot Nova VMs from.
 function create_manila_service_image {
-    TOKEN=$(openstack token issue -c id -f value)
-
-    # Download Manila's image
-    if is_service_enabled g-reg; then
-        upload_image $MANILA_SERVICE_IMAGE_URL $TOKEN
+    if is_service_enabled nova; then
+        TOKEN=$(openstack token issue -c id -f value)
+        local image_exists=$( openstack image list | grep " $MANILA_SERVICE_IMAGE_NAME " )
+        if [[ -z $image_exists ]]; then
+            # Download Manila's image
+            if is_service_enabled g-reg; then
+                upload_image $MANILA_SERVICE_IMAGE_URL $TOKEN
+            fi
+        fi
     fi
 }
 
@@ -349,42 +393,42 @@ function create_manila_service_image {
 # Nova VMs when generic driver is configured.
 function create_manila_service_secgroup {
     # Create a secgroup
-    if ! nova secgroup-list | grep -q $MANILA_SERVICE_SECGROUP; then
-        nova secgroup-create $MANILA_SERVICE_SECGROUP "$MANILA_SERVICE_SECGROUP description"
-        if ! timeout 30 sh -c "while ! nova secgroup-list | grep -q $MANILA_SERVICE_SECGROUP; do sleep 1; done"; then
+    if ! openstack security group list | grep -q $MANILA_SERVICE_SECGROUP; then
+        openstack security group create $MANILA_SERVICE_SECGROUP --description "$MANILA_SERVICE_SECGROUP description"
+        if ! timeout 30 sh -c "while ! openstack security group list | grep -q $MANILA_SERVICE_SECGROUP; do sleep 1; done"; then
             echo "Security group not created"
             exit 1
         fi
     fi
 
     # Configure Security Group Rules
-    if ! nova secgroup-list-rules $MANILA_SERVICE_SECGROUP | grep -q icmp; then
-        nova secgroup-add-rule $MANILA_SERVICE_SECGROUP icmp -1 -1 0.0.0.0/0
+    if ! openstack security group rule list $MANILA_SERVICE_SECGROUP | grep -q icmp; then
+        openstack security group rule create $MANILA_SERVICE_SECGROUP --protocol icmp
     fi
-    if ! nova secgroup-list-rules $MANILA_SERVICE_SECGROUP | grep -q " tcp .* 22 "; then
-        nova secgroup-add-rule $MANILA_SERVICE_SECGROUP tcp 22 22 0.0.0.0/0
+    if ! openstack security group rule list $MANILA_SERVICE_SECGROUP | grep -q " tcp .* 22 "; then
+        openstack security group rule create $MANILA_SERVICE_SECGROUP --protocol tcp --dst-port 22
     fi
-    if ! nova secgroup-list-rules $MANILA_SERVICE_SECGROUP | grep -q " tcp .* 2049 "; then
-        nova secgroup-add-rule $MANILA_SERVICE_SECGROUP tcp 2049 2049 0.0.0.0/0
+    if ! openstack security group rule list $MANILA_SERVICE_SECGROUP | grep -q " tcp .* 2049 "; then
+        openstack security group rule create $MANILA_SERVICE_SECGROUP --protocol tcp --dst-port 2049
     fi
-    if ! nova secgroup-list-rules $MANILA_SERVICE_SECGROUP | grep -q " udp .* 2049 "; then
-        nova secgroup-add-rule $MANILA_SERVICE_SECGROUP udp 2049 2049 0.0.0.0/0
+    if ! openstack security group rule list $MANILA_SERVICE_SECGROUP | grep -q " udp .* 2049 "; then
+        openstack security group rule create $MANILA_SERVICE_SECGROUP --protocol udp --dst-port 2049
     fi
-    if ! nova secgroup-list-rules $MANILA_SERVICE_SECGROUP | grep -q " udp .* 445 "; then
-        nova secgroup-add-rule $MANILA_SERVICE_SECGROUP udp 445 445 0.0.0.0/0
+    if ! openstack security group rule list $MANILA_SERVICE_SECGROUP | grep -q " udp .* 445 "; then
+        openstack security group rule create $MANILA_SERVICE_SECGROUP --protocol udp --dst-port 445
     fi
-    if ! nova secgroup-list-rules $MANILA_SERVICE_SECGROUP | grep -q " tcp .* 445 "; then
-        nova secgroup-add-rule $MANILA_SERVICE_SECGROUP tcp 445 445 0.0.0.0/0
+    if ! openstack security group rule list $MANILA_SERVICE_SECGROUP | grep -q " tcp .* 445 "; then
+        openstack security group rule create $MANILA_SERVICE_SECGROUP --protocol tcp --dst-port 445
     fi
-    if ! nova secgroup-list-rules $MANILA_SERVICE_SECGROUP | grep -q " tcp .* 139 "; then
-        nova secgroup-add-rule $MANILA_SERVICE_SECGROUP tcp 137 139 0.0.0.0/0
+    if ! openstack security group rule list $MANILA_SERVICE_SECGROUP | grep -q " tcp .* 139 "; then
+        openstack security group rule create $MANILA_SERVICE_SECGROUP --protocol tcp --dst-port 137:139
     fi
-    if ! nova secgroup-list-rules $MANILA_SERVICE_SECGROUP | grep -q " udp .* 139 "; then
-        nova secgroup-add-rule $MANILA_SERVICE_SECGROUP udp 137 139 0.0.0.0/0
+    if ! openstack security group rule list $MANILA_SERVICE_SECGROUP | grep -q " udp .* 139 "; then
+        openstack security group rule create $MANILA_SERVICE_SECGROUP --protocol udp --dst-port 137:139
     fi
 
     # List secgroup rules
-    nova secgroup-list-rules $MANILA_SERVICE_SECGROUP
+    openstack security group rule list $MANILA_SERVICE_SECGROUP
 }
 
 # create_manila_accounts - Set up common required manila accounts
@@ -392,20 +436,29 @@ function create_manila_accounts {
 
     create_service_user "manila"
 
-    if [[ "$KEYSTONE_CATALOG_BACKEND" = 'sql' ]]; then
-        # Set up Manila v1 service and endpoint
-        get_or_create_service "manila" "share" "Manila Shared Filesystem Service"
-        get_or_create_endpoint "share" "$REGION_NAME" \
-            "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v1/\$(tenant_id)s" \
-            "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v1/\$(tenant_id)s" \
-            "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v1/\$(tenant_id)s"
+    # Set up Manila v1 service and endpoint
+    get_or_create_service "manila" "share" "Manila Shared Filesystem Service"
+    get_or_create_endpoint "share" "$REGION_NAME" \
+        "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v1/\$(tenant_id)s" \
+        "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v1/\$(tenant_id)s" \
+        "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v1/\$(tenant_id)s"
 
-        # Set up Manila v2 service and endpoint
-        get_or_create_service "manilav2" "sharev2" "Manila Shared Filesystem Service V2"
-        get_or_create_endpoint "sharev2" "$REGION_NAME" \
-            "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v2/\$(tenant_id)s" \
-            "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v2/\$(tenant_id)s" \
-            "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v2/\$(tenant_id)s"
+    # Set up Manila v2 service and endpoint
+    get_or_create_service "manilav2" "sharev2" "Manila Shared Filesystem Service V2"
+    get_or_create_endpoint "sharev2" "$REGION_NAME" \
+        "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v2/\$(tenant_id)s" \
+        "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v2/\$(tenant_id)s" \
+        "$MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT/v2/\$(tenant_id)s"
+}
+
+# create_default_share_group_type - create share group type that will be set as default.
+function create_default_share_group_type {
+    local type_exists=$( manila share-group-type-list | grep " $MANILA_DEFAULT_SHARE_GROUP_TYPE " )
+    if [[ -z $type_exists ]]; then
+        manila share-group-type-create $MANILA_DEFAULT_SHARE_GROUP_TYPE $MANILA_DEFAULT_SHARE_TYPE
+    fi
+    if [[ $MANILA_DEFAULT_SHARE_GROUP_TYPE_SPECS ]]; then
+        manila share-group-type-key $MANILA_DEFAULT_SHARE_GROUP_TYPE set $MANILA_DEFAULT_SHARE_GROUP_TYPE_SPECS
     fi
 }
 
@@ -419,12 +472,53 @@ function create_default_share_type {
     enabled_backends=(${MANILA_ENABLED_BACKENDS//,/ })
     driver_handles_share_servers=$(iniget $MANILA_CONF ${enabled_backends[0]} driver_handles_share_servers)
 
-    manila type-create $MANILA_DEFAULT_SHARE_TYPE $driver_handles_share_servers
+    local type_exists=$( manila type-list | grep " $MANILA_DEFAULT_SHARE_TYPE " )
+    if [[ -z $type_exists ]]; then
+        local command_args="$MANILA_DEFAULT_SHARE_TYPE $driver_handles_share_servers"
+        #if is_driver_enabled $MANILA_CONTAINER_DRIVER; then
+        #    # TODO(aovchinnikov): Remove this condition when Container driver supports
+        #    # snapshots
+        #    command_args="$command_args --snapshot_support false"
+        #fi
+        manila type-create $command_args
+    fi
     if [[ $MANILA_DEFAULT_SHARE_TYPE_EXTRA_SPECS ]]; then
         manila type-key $MANILA_DEFAULT_SHARE_TYPE set $MANILA_DEFAULT_SHARE_TYPE_EXTRA_SPECS
     fi
 }
 
+# create_custom_share_types - create share types suitable for both possible
+# driver modes with names "dhss_true" and "dhss_false".
+function create_custom_share_types {
+    manila type-create dhss_true True
+    if [[ $MANILA_DHSS_TRUE_SHARE_TYPE_EXTRA_SPECS ]]; then
+        manila type-key dhss_true set $MANILA_DHSS_TRUE_SHARE_TYPE_EXTRA_SPECS
+    fi
+
+    manila type-create dhss_false False
+    if [[ $MANILA_DHSS_FALSE_SHARE_TYPE_EXTRA_SPECS ]]; then
+        manila type-key dhss_false set $MANILA_DHSS_FALSE_SHARE_TYPE_EXTRA_SPECS
+    fi
+}
+
+# configure_backing_file - Set up backing file for LVM
+function configure_backing_file {
+    if ! sudo vgs $SHARE_GROUP; then
+        if [ "$CONFIGURE_BACKING_FILE" = "True" ]; then
+            SHARE_BACKING_FILE=${SHARE_BACKING_FILE:-$DATA_DIR/${SHARE_GROUP}-backing-file}
+            # Only create if the file doesn't already exists
+            [[ -f $SHARE_BACKING_FILE ]] || truncate -s $SHARE_BACKING_FILE_SIZE $SHARE_BACKING_FILE
+            DEV=`sudo losetup -f --show $SHARE_BACKING_FILE`
+        else
+            DEV=$SHARE_BACKING_FILE
+        fi
+        # Only create if the loopback device doesn't contain $SHARE_GROUP
+        if ! sudo vgs $SHARE_GROUP; then sudo vgcreate $SHARE_GROUP $DEV; fi
+    fi
+
+    mkdir -p $MANILA_STATE_PATH/shares
+    mkdir -p /tmp/shares
+}
 
 # init_manila - Initializes database and creates manila dir if absent
 function init_manila {
@@ -455,25 +549,19 @@ function init_manila {
             #
             # By default, the backing file is 8G in size, and is stored in ``/opt/stack/data``.
 
-            if ! sudo vgs $SHARE_GROUP; then
-                if [ "$CONFIGURE_BACKING_FILE" = "True" ]; then
-                    SHARE_BACKING_FILE=${SHARE_BACKING_FILE:-$DATA_DIR/${SHARE_GROUP}-backing-file}
-                    # Only create if the file doesn't already exists
-                    [[ -f $SHARE_BACKING_FILE ]] || truncate -s $SHARE_BACKING_FILE_SIZE $SHARE_BACKING_FILE
-                    DEV=`sudo losetup -f --show $SHARE_BACKING_FILE`
-                else
-                    DEV=$SHARE_BACKING_FILE
-                fi
-                # Only create if the loopback device doesn't contain $SHARE_GROUP
-                if ! sudo vgs $SHARE_GROUP; then sudo vgcreate $SHARE_GROUP $DEV; fi
-            fi
-
-            mkdir -p $MANILA_STATE_PATH/shares
+            configure_backing_file
         fi
+    elif [ "$SHARE_DRIVER" == $MANILA_CONTAINER_DRIVER ]; then
+        if is_service_enabled m-shr; then
+            SHARE_GROUP=$MANILA_CONTAINER_VOLUME_GROUP_NAME
+            configure_backing_file
+        fi
+
     elif [ "$SHARE_DRIVER" == "manila.share.drivers.zfsonlinux.driver.ZFSonLinuxShareDriver" ]; then
         if is_service_enabled m-shr; then
             mkdir -p $MANILA_ZFSONLINUX_BACKEND_FILES_CONTAINER_DIR
             file_counter=0
+            MANILA_ZFSONLINUX_SERVICE_IP=${MANILA_ZFSONLINUX_SERVICE_IP:-"127.0.0.1"}
             for BE in ${MANILA_ENABLED_BACKENDS//,/ }; do
                 if [[ $file_counter == 0 ]]; then
                     # NOTE(vponomaryov): create two pools for first ZFS backend
@@ -507,6 +595,7 @@ function init_manila {
                 iniset $MANILA_CONF $BE zfs_use_ssh $MANILA_ZFSONLINUX_USE_SSH
                 iniset $MANILA_CONF $BE zfs_ssh_username $MANILA_ZFSONLINUX_SSH_USERNAME
                 iniset $MANILA_CONF $BE replication_domain $MANILA_ZFSONLINUX_REPLICATION_DOMAIN
+                iniset $MANILA_CONF $BE driver_handles_share_servers False
                 let "file_counter=file_counter+1"
             done
             # Install the server's SSH key in our known_hosts file
@@ -519,6 +608,7 @@ function init_manila {
                 cat $STACK_HOME/.ssh/*.pub >> $SSH_USER_HOME/.ssh/authorized_keys
                 # Give ssh user sudo access
                 echo "$MANILA_ZFSONLINUX_SSH_USERNAME ALL=(ALL) NOPASSWD: ALL" | sudo tee -a /etc/sudoers > /dev/null
+                iniset $MANILA_CONF DEFAULT data_node_access_ip $MANILA_ZFSONLINUX_SERVICE_IP
             fi
         fi
     fi
@@ -529,44 +619,97 @@ function init_manila {
     rm -f $MANILA_AUTH_CACHE_DIR/*
 }
 
+# check_nfs_kernel_service_state_ubuntu- Make sure nfsd is running
+function check_nfs_kernel_service_state_ubuntu {
+    # (aovchinnikov): Workaround for nfs-utils bug 1052264
+    if [[ $(sudo service nfs-kernel-server status &> /dev/null || echo 'fail') == 'fail' ]]; then
+        echo "Apparently nfsd is not running. Trying to fix that."
+        sudo mkdir -p "/media/nfsdonubuntuhelper"
+        # (aovchinnikov): shell wrapping is needed for cases when a file to be written
+        # is owned by root.
+        sudo sh -c "echo '/media/nfsdonubuntuhelper 127.0.0.1(ro)' >> /etc/exports"
+        sudo service nfs-kernel-server start
+    fi
+    if [[ $(sudo service nfs-kernel-server status &> /dev/null || echo 'fail') == 'fail' ]]; then
+        echo "Failed to start nfsd. Exiting."
+        exit 1
+    fi
+}
+
+function _install_nfs_and_samba {
+    if is_ubuntu; then
+        install_package nfs-kernel-server nfs-common samba
+        check_nfs_kernel_service_state_ubuntu
+    elif is_fedora; then
+        install_package nfs-utils samba
+    elif is_suse; then
+        install_package nfs-kernel-server nfs-utils samba
+    else:
+        echo "This distro is not supported. Skipping step of NFS and Samba installation."
+    fi
+}
+
 # install_manila - Collect source and prepare
 function install_manila {
     git_clone $MANILACLIENT_REPO $MANILACLIENT_DIR $MANILACLIENT_BRANCH
+    setup_develop $MANILACLIENT_DIR
+    setup_develop $MANILA_DIR
 
-    if [ "$SHARE_DRIVER" == "manila.share.drivers.lvm.LVMShareDriver" ]; then
-        if is_service_enabled m-shr; then
-            if is_ubuntu; then
-                sudo apt-get install -y nfs-kernel-server nfs-common samba
-            elif is_fedora; then
-                sudo yum install -y nfs-utils nfs-utils-lib samba
-            fi
-        fi
-    elif [ "$SHARE_DRIVER" == "manila.share.drivers.zfsonlinux.driver.ZFSonLinuxShareDriver" ]; then
-        if is_service_enabled m-shr; then
-            if is_ubuntu; then
-                sudo apt-get install -y nfs-kernel-server nfs-common samba
-                # NOTE(vponomaryov): following installation is valid for Ubuntu 'trusty'.
-                sudo apt-get install -y software-properties-common
-                sudo apt-add-repository --yes ppa:zfs-native/stable
-                sudo apt-get -y -q update && sudo apt-get -y -q upgrade
-                sudo apt-get install -y linux-headers-generic
-                sudo apt-get install -y build-essential
-                sudo apt-get install -y ubuntu-zfs
+    if is_service_enabled m-shr; then
+        _install_nfs_and_samba
+
+        if [ "$SHARE_DRIVER" == "manila.share.drivers.zfsonlinux.driver.ZFSonLinuxShareDriver" ]; then
+            if [[ $(sudo zfs list &> /dev/null && sudo zpool list &> /dev/null || echo 'absent') == 'absent' ]]; then
+                # ZFS not found, try to install it
+                if is_ubuntu; then
+                    if [[ $(lsb_release -s -d) == *"14.04"* ]]; then
+                        # Trusty
+                        sudo apt-get install -y software-properties-common
+                        sudo apt-add-repository --yes ppa:zfs-native/stable
+
+                        # Workaround for bug #1609696
+                        sudo apt-mark hold grub*
+
+                        sudo apt-get -y -q update && sudo apt-get -y -q upgrade
+
+                        # Workaround for bug #1609696
+                        sudo apt-mark unhold grub*
+
+                        sudo apt-get install -y linux-headers-generic
+                        sudo apt-get install -y build-essential
+                        sudo apt-get install -y ubuntu-zfs
+                    elif [[ $(lsb_release -s -d) == *"16.04"* ]]; then
+                        # Xenial
+                        sudo apt-get install -y zfsutils-linux
+                    else
+                        echo "Only 'Trusty' and 'Xenial' releases of Ubuntu are supported."
+                        exit 1
+                    fi
+                else
+                    echo "Manila Devstack plugin supports installation "\
+                        "of ZFS packages only for 'Ubuntu' distros. "\
+                        "Please, install it first by other means or add its support "\
+                        "for your distro."
+                    exit 1
+                fi
                 sudo modprobe zfs
+                sudo modprobe zpool
+            fi
+            check_nfs_kernel_service_state_ubuntu
+        elif [ "$SHARE_DRIVER" == $MANILA_CONTAINER_DRIVER ]; then
+            if is_ubuntu; then
+                echo "Installing docker...."
+                install_docker_ubuntu
+                echo "Importing docker image"
+                import_docker_service_image_ubuntu
             else
-                echo "Manila Devstack plugin does not support installation "\
-                    "of ZFS packages for non-'Ubuntu-trusty' distros. "\
-                    "Please, install it first by other means or add its support "\
-                    "for your distro."
+                echo "Manila Devstack plugin does not support Container Driver on  "\
+                     " distros other than Ubuntu."
                 exit 1
             fi
         fi
     fi
 
-    # install manila-ui if horizon is enabled
-    if is_service_enabled horizon && [ "$MANILA_UI_ENABLED" = "True" ]; then
-        git_clone $MANILA_UI_REPO $MANILA_UI_DIR $MANILA_UI_BRANCH
-    fi
 }
 
 #configure_samba - Configure node as Samba server
@@ -582,7 +725,9 @@ function configure_samba {
             sudo service $samba_daemon_name restart || echo "Couldn't restart '$samba_daemon_name' service"
         fi
 
-        sudo cp /usr/share/samba/smb.conf $SMB_CONF
+        if [[ -e /usr/share/samba/smb.conf ]]; then
+            sudo cp /usr/share/samba/smb.conf $SMB_CONF
+        fi
         sudo chown $STACK_USER -R /etc/samba
         iniset $SMB_CONF global include registry
         iniset $SMB_CONF global security user
@@ -593,24 +738,20 @@ function configure_samba {
 
         for backend_name in ${MANILA_ENABLED_BACKENDS//,/ }; do
             iniset $MANILA_CONF $backend_name driver_handles_share_servers False
-            iniset $MANILA_CONF $backend_name lvm_share_export_ip $HOST_IP
+            iniset $MANILA_CONF $backend_name lvm_share_export_ip $MANILA_LVM_SHARE_EXPORT_IP
         done
         iniset $MANILA_CONF DEFAULT data_node_access_ip $HOST_IP
     fi
 }
 
-# start_manila - Start running processes, including screen
-function start_manila {
-    # restart apache to reload running horizon if manila-ui is enabled
-    if is_service_enabled horizon && [ "$MANILA_UI_ENABLED" = "True" ]; then
-        restart_apache_server
-        sleep 3 # Wait for 3 sec to ensure that apache is running
-    fi
+# start_manila_api - starts manila API services and checks its availability
+function start_manila_api {
+    run_process m-api "$MANILA_BIN_DIR/manila-api --config-file $MANILA_CONF"
 
-    screen_it m-api "cd $MANILA_DIR && $MANILA_BIN_DIR/manila-api --config-file $MANILA_CONF"
-    screen_it m-shr "cd $MANILA_DIR && $MANILA_BIN_DIR/manila-share --config-file $MANILA_CONF"
-    screen_it m-sch "cd $MANILA_DIR && $MANILA_BIN_DIR/manila-scheduler --config-file $MANILA_CONF"
-    screen_it m-dat "cd $MANILA_DIR && $MANILA_BIN_DIR/manila-data --config-file $MANILA_CONF"
+    echo "Waiting for Manila API to start..."
+    if ! wait_for_service $SERVICE_TIMEOUT $MANILA_SERVICE_PROTOCOL://$MANILA_SERVICE_HOST:$MANILA_SERVICE_PORT; then
+        die $LINENO "Manila API did not start"
+    fi
 
     # Start proxies if enabled
     if is_service_enabled tls-proxy; then
@@ -618,27 +759,110 @@ function start_manila {
     fi
 }
 
+# start_rest_of_manila - starts non-api manila services
+function start_rest_of_manila {
+    run_process m-shr "$MANILA_BIN_DIR/manila-share --config-file $MANILA_CONF"
+    run_process m-sch "$MANILA_BIN_DIR/manila-scheduler --config-file $MANILA_CONF"
+    run_process m-dat "$MANILA_BIN_DIR/manila-data --config-file $MANILA_CONF"
+}
+
+# start_manila - start all manila services. This function is kept for compatibility
+# reasons with old approach.
+function start_manila {
+    start_manila_api
+    start_rest_of_manila
+}
+
 # stop_manila - Stop running processes
 function stop_manila {
-    # Kill the manila screen windows
+    # Kill the manila processes
     for serv in m-api m-sch m-shr m-dat; do
-        screen -S $SCREEN_NAME -p $serv -X kill
+        stop_process $serv
     done
 }
 
 # update_tempest - Function used for updating Tempest config if Tempest service enabled
 function update_tempest {
     if is_service_enabled tempest; then
+        TEMPEST_CONFIG=${TEMPEST_CONFIG:-$TEMPEST_DIR/etc/tempest.conf}
+        ADMIN_TENANT_NAME=${ADMIN_TENANT_NAME:-"admin"}
+        ADMIN_DOMAIN_NAME=${ADMIN_DOMAIN_NAME:-"Default"}
+        ADMIN_PASSWORD=${ADMIN_PASSWORD:-"secretadmin"}
+
         if [ $(trueorfalse False MANILA_USE_SERVICE_INSTANCE_PASSWORD) == True ]; then
-            iniset $TEMPEST_DIR/etc/tempest.conf share image_password $MANILA_SERVICE_INSTANCE_PASSWORD
+            iniset $TEMPEST_CONFIG share image_password $MANILA_SERVICE_INSTANCE_PASSWORD
         fi
-        iniset $TEMPEST_DIR/etc/tempest.conf share image_with_share_tools $MANILA_SERVICE_IMAGE_NAME
+        iniset $TEMPEST_CONFIG share image_with_share_tools $MANILA_SERVICE_IMAGE_NAME
+        iniset $TEMPEST_CONFIG auth admin_username ${ADMIN_USERNAME:-"admin"}
+        iniset $TEMPEST_CONFIG auth admin_password ${ADMIN_PASSWORD:-"secretadmin"}
+        iniset $TEMPEST_CONFIG auth admin_tenant_name $ADMIN_TENANT_NAME
+        iniset $TEMPEST_CONFIG auth admin_domain_name $ADMIN_DOMAIN_NAME
+        iniset $TEMPEST_CONFIG identity username ${TEMPEST_USERNAME:-"demo"}
+        iniset $TEMPEST_CONFIG identity password $ADMIN_PASSWORD
+        iniset $TEMPEST_CONFIG identity tenant_name ${TEMPEST_TENANT_NAME:-"demo"}
+        iniset $TEMPEST_CONFIG identity domain_name $ADMIN_DOMAIN_NAME
+        iniset $TEMPEST_CONFIG identity alt_username ${ALT_USERNAME:-"alt_demo"}
+        iniset $TEMPEST_CONFIG identity alt_password $ADMIN_PASSWORD
+        iniset $TEMPEST_CONFIG identity alt_tenant_name ${ALT_TENANT_NAME:-"alt_demo"}
+        iniset $TEMPEST_CONFIG identity alt_domain_name $ADMIN_DOMAIN_NAME
     fi
 }
 
+function install_docker_ubuntu {
+    sudo apt-get update
+    install_package apparmor
+    install_package docker.io
+}
+
+function download_image {
+    local image_url=$1
+
+    local image image_fname
+
+    image_fname=`basename "$image_url"`
+    if [[ $image_url != file* ]]; then
+        # Downloads the image (uec ami+akistyle), then extracts it.
+        if [[ ! -f $FILES/$image_fname || "$(stat -c "%s" $FILES/$image_fname)" = "0" ]]; then
+            wget --progress=dot:giga -c $image_url -O $FILES/$image_fname
+            if [[ $? -ne 0 ]]; then
+                echo "Not found: $image_url"
+                return
+            fi
+        fi
+        image="$FILES/${image_fname}"
+    else
+        # File based URL (RFC 1738): ``file://host/path``
+        # Remote files are not considered here.
+        # unix: ``file:///home/user/path/file``
+        # windows: ``file:///C:/Documents%20and%20Settings/user/path/file``
+        image=$(echo $image_url | sed "s/^file:\/\///g")
+        if [[ ! -f $image || "$(stat -c "%s" $image)" == "0" ]]; then
+            echo "Not found: $image_url"
+            return
+        fi
+    fi
+}
+
+function import_docker_service_image_ubuntu {
+    GZIPPED_IMG_NAME=`basename "$MANILA_DOCKER_IMAGE_URL"`
+    IMG_NAME_LOAD=${GZIPPED_IMG_NAME%.*}
+    LOCAL_IMG_NAME=${IMG_NAME_LOAD%.*}
+    if [[ "$(sudo docker images -q $LOCAL_IMG_NAME)" == "" ]]; then
+        download_image $MANILA_DOCKER_IMAGE_URL
+        # Import image in Docker
+        gzip -d $FILES/$GZIPPED_IMG_NAME
+        sudo docker load --input $FILES/$IMG_NAME_LOAD
+    fi
+}
+
+function remove_docker_service_image {
+    sudo docker rmi $MANILA_DOCKER_IMAGE_ALIAS
+}
+
+
 function install_libraries {
     if [ $(trueorfalse False MANILA_MULTI_BACKEND) == True ]; then
-        if [ $(trueorfalse True RUN_MANILA_MIGRATION_TESTS) == True ]; then
+        if [ $(trueorfalse True RUN_MANILA_HOST_ASSISTED_MIGRATION_TESTS) == True ]; then
             if is_ubuntu; then
                 install_package nfs-common
             else
@@ -661,11 +885,6 @@ elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
     echo_summary "Installing extra libraries"
     install_libraries
 
-    # Neutron config update
-    if is_service_enabled neutron; then
-        iniset $Q_DHCP_CONF_FILE DEFAULT dnsmasq_local_resolv False
-    fi
-
     # Cinder config update
     if is_service_enabled cinder && [[ -n "$CINDER_OVERSUBSCRIPTION_RATIO" ]]; then
         CINDER_CONF=${CINDER_CONF:-/etc/cinder/cinder.conf}
@@ -700,26 +919,54 @@ elif [[ "$1" == "stack" && "$2" == "extra" ]]; then
     if is_service_enabled nova; then
         echo_summary "Creating Manila service keypair"
         create_manila_service_keypair
-
-        echo_summary "Creating Manila service VMs for generic driver \
-            backends for which handlng of share servers is disabled."
-        create_service_share_servers
     fi
 
     echo_summary "Configure Samba server"
     configure_samba
 
-    echo_summary "Starting Manila"
-    start_manila
+    echo_summary "Starting Manila API"
+    start_manila_api
+
+    # Workaround for bug #1660304
+    if [ "$SHARE_DRIVER" != "manila.share.drivers.generic.GenericShareDriver" ]; then
+        echo_summary "Starting rest of Manila services - scheduler, share and data"
+        start_rest_of_manila
+    fi
 
     echo_summary "Creating Manila default share type"
     create_default_share_type
 
+    # TODO(vponomaryov): revert following back in Pike
+    # echo_summary "Creating Manila default share group type"
+    # create_default_share_group_type
+
+    echo_summary "Creating Manila custom share types"
+    create_custom_share_types
+
+    echo_summary "Manila UI is no longer enabled by default. \
+        Add enable_plugin manila-ui github.com/openstack/manila-ui \
+        to your local.conf file to enable Manila UI"
+
+elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
+    ###########################################################################
+    # NOTE(vponomaryov): Workaround for bug #1660304
+    # We are able to create Nova VMs now only when last Nova step is performed
+    # which is registration of cell0. It is registered as last action in
+    # "post-extra" section.
+    if is_service_enabled nova; then
+        echo_summary "Creating Manila service VMs for generic driver \
+            backends for which handlng of share servers is disabled."
+        create_service_share_servers
+    fi
+
+    if [ "$SHARE_DRIVER" == "manila.share.drivers.generic.GenericShareDriver" ]; then
+        echo_summary "Starting rest of Manila services - scheduler, share and data"
+        start_rest_of_manila
+    fi
+    ###########################################################################
+
     echo_summary "Update Tempest config"
     update_tempest
-
-    echo_summary "Configuring Manila UI"
-    configure_manila_ui
 fi
 
 if [[ "$1" == "unstack" ]]; then
