@@ -78,13 +78,33 @@ REST_API_VERSION_HISTORY = """
             'force_host_copy' to 'force_host_assisted_migration', removed
             'notify' parameter and removed previous migrate_share API support.
             Updated reset_task_state API to accept 'None' value.
+    * 2.23 - Added share_type to filter results of scheduler-stats/pools API.
+    * 2.24 - Added optional create_share_from_snapshot_support extra spec,
+             which was previously inferred from the 'snapshot_support' extra
+             spec. Also made the 'snapshot_support' extra spec optional.
+    * 2.25 - Added quota-show detail API.
+    * 2.26 - Removed 'nova_net_id' parameter from share_network API.
+    * 2.27 - Added share revert to snapshot API.
+    * 2.28 - Added transitional states to access rules and replaced all
+             transitional access_rules_status values of
+             shares (share_instances) with 'syncing'. Share action API
+             'access_allow' now accepts rules even when a share or any of
+             its instances may have an access_rules_status set to 'error'.
+    * 2.29 - Updated migration_start API adding mandatory parameter
+             'preserve_snapshots' and changed 'preserve_metadata', 'writable',
+             'nondisruptive' to be mandatory as well. All previous
+             migration_start APIs prior to this microversion are now
+             unsupported.
+    * 2.30 - Added cast_rules_to_readonly field to share_instances.
+    * 2.31 - Convert consistency groups to share groups.
+    * 2.32 - Added mountable snapshots APIs.
 """
 
 # The minimum and maximum versions of the API supported
 # The default api version request is defined to be the
-# the minimum version of the API supported.
+# minimum version of the API supported.
 _MIN_API_VERSION = "2.0"
-_MAX_API_VERSION = "2.22"
+_MAX_API_VERSION = "2.32"
 DEFAULT_API_VERSION = _MIN_API_VERSION
 
 
@@ -124,8 +144,13 @@ class APIVersionRequest(utils.ComparableMixin):
 
     def __str__(self):
         """Debug/Logging representation of object."""
-        return ("API Version Request Major: %(major)s, Minor: %(minor)s"
-                % {'major': self._ver_major, 'minor': self._ver_minor})
+        params = {
+            'major': self._ver_major,
+            'minor': self._ver_minor,
+            'experimental': self._experimental,
+        }
+        return ("API Version Request Major: %(major)s, Minor: %(minor)s, "
+                "Experimental: %(experimental)s" % params)
 
     def is_null(self):
         return self._ver_major is None and self._ver_minor is None
