@@ -528,10 +528,10 @@ class ResourceExceptionHandler(object):
                       ex_value, exc_info=exc_info)
             raise Fault(webob.exc.HTTPBadRequest())
         elif isinstance(ex_value, Fault):
-            LOG.info(_LI("Fault thrown: %s"), six.text_type(ex_value))
+            LOG.info(_LI("Fault thrown: %s"), ex_value)
             raise ex_value
         elif isinstance(ex_value, webob.exc.HTTPException):
-            LOG.info(_LI("HTTP exception thrown: %s"), six.text_type(ex_value))
+            LOG.info(_LI("HTTP exception thrown: %s"), ex_value)
             raise Fault(ex_value)
 
         # We didn't handle the exception
@@ -1185,6 +1185,8 @@ class AdminActionsMixin(object):
             constants.STATUS_DELETING,
             constants.STATUS_ERROR,
             constants.STATUS_ERROR_DELETING,
+            constants.STATUS_MIGRATING,
+            constants.STATUS_MIGRATING_TO,
         ]),
         'replica_state': set([
             constants.REPLICA_STATE_ACTIVE,
