@@ -16,8 +16,6 @@
 """The shares api."""
 
 import ast
-import re
-import string
 
 from oslo_log import log
 from oslo_utils import strutils
@@ -30,6 +28,7 @@ from manila.api import common
 from manila.api.openstack import wsgi
 from manila.api.views import share_accesses as share_access_views
 from manila.api.views import shares as share_views
+from manila.common import constants
 from manila import db
 from manila import exception
 from manila.i18n import _, _LI
@@ -74,19 +73,19 @@ class ShareMixin(object):
         try:
             share = self.share_api.get(context, id)
 
-            # NOTE(ameade): If the share is in a consistency group, we require
-            # it's id be specified as a param.
-            if share.get('consistency_group_id'):
-                consistency_group_id = req.params.get('consistency_group_id')
-                if (share.get('consistency_group_id') and
-                        not consistency_group_id):
-                    msg = _("Must provide 'consistency_group_id' as a request "
-                            "parameter when deleting a share in a consistency "
-                            "group.")
+            # NOTE(ameade): If the share is in a share group, we require its
+            # id be specified as a param.
+            sg_id_key = 'share_group_id'
+            if share.get(sg_id_key):
+                share_group_id = req.params.get(sg_id_key)
+                if not share_group_id:
+                    msg = _("Must provide '%s' as a request "
+                            "parameter when deleting a share in a share "
+                            "group.") % sg_id_key
                     raise exc.HTTPBadRequest(explanation=msg)
-                elif consistency_group_id != share.get('consistency_group_id'):
-                    msg = _("The specified 'consistency_group_id' does not "
-                            "match the consistency group id of the share.")
+                elif share_group_id != share.get(sg_id_key):
+                    msg = _("The specified '%s' does not match "
+                            "the share group id of the share.") % sg_id_key
                     raise exc.HTTPBadRequest(explanation=msg)
 
             self.share_api.delete(context, share)
@@ -160,7 +159,7 @@ class ShareMixin(object):
             'display_name', 'status', 'share_server_id', 'volume_type_id',
             'share_type_id', 'snapshot_id', 'host', 'share_network_id',
             'is_public', 'metadata', 'extra_specs', 'sort_key', 'sort_dir',
-            'consistency_group_id', 'cgsnapshot_id'
+            'share_group_id', 'share_group_snapshot_id'
         )
 
     def update(self, req, id, body):
@@ -191,12 +190,13 @@ class ShareMixin(object):
         return self._view_builder.detail(req, share)
 
     def create(self, req, body):
-        # Remove consistency group attributes
-        body.get('share', {}).pop('consistency_group_id', None)
+        # Remove share group attributes
+        body.get('share', {}).pop('share_group_id', None)
         share = self._create(req, body)
         return share
 
-    def _create(self, req, body):
+    def _create(self, req, body,
+                check_create_share_from_snapshot_support=False):
         """Creates a new share."""
         context = req.environ['manila.context']
 
@@ -235,7 +235,7 @@ class ShareMixin(object):
             'availability_zone': availability_zone,
             'metadata': share.get('metadata'),
             'is_public': share.get('is_public', False),
-            'consistency_group_id': share.get('consistency_group_id')
+            'share_group_id': share.get('share_group_id')
         }
 
         snapshot_id = share.get('snapshot_id')
@@ -251,7 +251,7 @@ class ShareMixin(object):
         if snapshot:
             # Need to check that share_network_id from snapshot's
             # parents share equals to share_network_id from args.
-            # If share_network_id is empty than update it with
+            # If share_network_id is empty then update it with
             # share_network_id of parent share.
             parent_share = self.share_api.get(context, snapshot['share_id'])
             parent_share_net_id = parent_share.instance['share_network_id']
@@ -263,6 +263,16 @@ class ShareMixin(object):
             elif parent_share_net_id:
                 share_network_id = parent_share_net_id
 
+            # Verify that share can be created from a snapshot
+            if (check_create_share_from_snapshot_support and
+                    not parent_share['create_share_from_snapshot_support']):
+                msg = (_("A new share may not be created from snapshot '%s', "
+                         "because the snapshot's parent share does not have "
+                         "that capability.")
+                       % snapshot_id)
+                LOG.error(msg)
+                raise exc.HTTPBadRequest(explanation=msg)
+
         if share_network_id:
             try:
                 self.share_api.get_share_network(
@@ -298,10 +308,10 @@ class ShareMixin(object):
                 share_type = def_share_type
 
         # Only use in create share feature. Create share from snapshot
-        # and create share with consistency group features not
+        # and create share with share group features not
         # need this check.
         if (not share_network_id and not snapshot
-                and not share.get('consistency_group_id')
+                and not share.get('share_group_id')
                 and share_type and share_type.get('extra_specs')
                 and (strutils.bool_from_string(share_type.get('extra_specs').
                      get('driver_handles_share_servers')))):
@@ -321,100 +331,38 @@ class ShareMixin(object):
         return self._view_builder.detail(req, new_share)
 
     @staticmethod
-    def _validate_common_name(access):
-        """Validate common name passed by user.
+    def _any_instance_has_errored_rules(share):
+        for instance in share['instances']:
+            access_rules_status = instance['access_rules_status']
+            if access_rules_status == constants.SHARE_INSTANCE_RULES_ERROR:
+                return True
+        return False
 
-        'access' is used as the certificate's CN (common name)
-        to which access is allowed or denied by the backend.
-        The standard allows for just about any string in the
-        common name. The meaning of a string depends on its
-        interpretation and is limited to 64 characters.
-        """
-        if len(access) == 0 or len(access) > 64:
-            exc_str = _('Invalid CN (common name). Must be 1-64 chars long')
-            raise webob.exc.HTTPBadRequest(explanation=exc_str)
-
-    @staticmethod
-    def _validate_username(access):
-        valid_username_re = '[\w\.\-_\`;\'\{\}\[\]\\\\]{4,32}$'
-        username = access
-        if not re.match(valid_username_re, username):
-            exc_str = ('Invalid user or group name. Must be 4-32 characters '
-                       'and consist of alphanumeric characters and '
-                       'special characters ]{.-_\'`;}[\\')
-            raise webob.exc.HTTPBadRequest(explanation=exc_str)
-
-    @staticmethod
-    def _validate_ip_range(ip_range):
-        ip_range = ip_range.split('/')
-        exc_str = ('Supported ip format examples:\n'
-                   '\t10.0.0.2, 10.0.0.0/24')
-        if len(ip_range) > 2:
-            raise webob.exc.HTTPBadRequest(explanation=exc_str)
-        if len(ip_range) == 2:
-            try:
-                prefix = int(ip_range[1])
-                if prefix < 0 or prefix > 32:
-                    raise ValueError()
-            except ValueError:
-                msg = 'IP prefix should be in range from 0 to 32'
-                raise webob.exc.HTTPBadRequest(explanation=msg)
-        ip_range = ip_range[0].split('.')
-        if len(ip_range) != 4:
-            raise webob.exc.HTTPBadRequest(explanation=exc_str)
-        for item in ip_range:
-            try:
-                if 0 <= int(item) <= 255:
-                    continue
-                raise ValueError()
-            except ValueError:
-                raise webob.exc.HTTPBadRequest(explanation=exc_str)
-
-    @staticmethod
-    def _validate_cephx_id(cephx_id):
-        if not cephx_id:
-            raise webob.exc.HTTPBadRequest(explanation=_(
-                'Ceph IDs may not be empty'))
-
-        # This restriction may be lifted in Ceph in the future:
-        # http://tracker.ceph.com/issues/14626
-        if not set(cephx_id) <= set(string.printable):
-            raise webob.exc.HTTPBadRequest(explanation=_(
-                'Ceph IDs must consist of ASCII printable characters'))
-
-        # Periods are technically permitted, but we restrict them here
-        # to avoid confusion where users are unsure whether they should
-        # include the "client." prefix: otherwise they could accidentally
-        # create "client.client.foobar".
-        if '.' in cephx_id:
-            raise webob.exc.HTTPBadRequest(explanation=_(
-                'Ceph IDs may not contain periods'))
-
-    def _allow_access(self, req, id, body, enable_ceph=False):
+    @wsgi.Controller.authorize('allow_access')
+    def _allow_access(self, req, id, body, enable_ceph=False,
+                      allow_on_error_status=False):
         """Add share access rule."""
         context = req.environ['manila.context']
         access_data = body.get('allow_access', body.get('os-allow_access'))
         share = self.share_api.get(context, id)
 
+        if (not allow_on_error_status and
+                self._any_instance_has_errored_rules(share)):
+            msg = _("Access rules cannot be added while the share or any of "
+                    "its replicas or migration copies has its "
+                    "access_rules_status set to %(instance_rules_status)s. "
+                    "Deny any rules in %(rule_state)s state and try "
+                    "again.") % {
+                'instance_rules_status': constants.SHARE_INSTANCE_RULES_ERROR,
+                'rule_state': constants.ACCESS_STATE_ERROR,
+            }
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+
         access_type = access_data['access_type']
         access_to = access_data['access_to']
-        if access_type == 'ip':
-            self._validate_ip_range(access_to)
-        elif access_type == 'user':
-            self._validate_username(access_to)
-        elif access_type == 'cert':
-            self._validate_common_name(access_to.strip())
-        elif access_type == "cephx" and enable_ceph:
-            self._validate_cephx_id(access_to)
-        else:
-            if enable_ceph:
-                exc_str = _("Only 'ip', 'user', 'cert' or 'cephx' access "
-                            "types are supported.")
-            else:
-                exc_str = _("Only 'ip', 'user' or 'cert' access types "
-                            "are supported.")
-
-            raise webob.exc.HTTPBadRequest(explanation=exc_str)
+        common.validate_access(access_type=access_type,
+                               access_to=access_to,
+                               enable_ceph=enable_ceph)
         try:
             access = self.share_api.allow_access(
                 context, share, access_type, access_to,
@@ -424,6 +372,7 @@ class ShareMixin(object):
 
         return self._access_view_builder.view(req, access)
 
+    @wsgi.Controller.authorize('deny_access')
     def _deny_access(self, req, id, body):
         """Remove share access rule."""
         context = req.environ['manila.context']
