@@ -105,11 +105,6 @@ def service_get_all_by_topic(context, topic):
     return IMPL.service_get_all_by_topic(context, topic)
 
 
-def service_get_all_by_host(context, host):
-    """Get all services for a given host."""
-    return IMPL.service_get_all_by_host(context, host)
-
-
 def service_get_all_share_sorted(context):
     """Get all share services sorted by share count.
 
@@ -236,21 +231,6 @@ def quota_usage_update(context, project_id, user_id, resource, **kwargs):
 ###################
 
 
-def reservation_create(context, uuid, usage, project_id, user_id, resource,
-                       delta, expire):
-    """Create a reservation for the given project and resource."""
-    return IMPL.reservation_create(context, uuid, usage, project_id,
-                                   user_id, resource, delta, expire)
-
-
-def reservation_get(context, uuid):
-    """Retrieve a reservation or raise if it does not exist."""
-    return IMPL.reservation_get(context, uuid)
-
-
-###################
-
-
 def quota_reserve(context, resources, quotas, user_quotas, deltas, expire,
                   until_refresh, max_age, project_id=None, user_id=None):
     """Check quotas and create appropriate reservations."""
@@ -325,9 +305,10 @@ def share_instances_get_all_by_share_server(context, share_server_id):
                                                         share_server_id)
 
 
-def share_instances_get_all_by_host(context, host):
+def share_instances_get_all_by_host(context, host, with_share_data=False):
     """Returns all share instances with given host."""
-    return IMPL.share_instances_get_all_by_host(context, host)
+    return IMPL.share_instances_get_all_by_host(
+        context, host, with_share_data=with_share_data)
 
 
 def share_instances_get_all_by_share_network(context, share_network_id):
@@ -341,24 +322,20 @@ def share_instances_get_all_by_share(context, share_id):
     return IMPL.share_instances_get_all_by_share(context, share_id)
 
 
-def share_instances_get_all_by_consistency_group_id(context, cg_id):
-    """Returns list of share instances that belong to given cg."""
-    return IMPL.share_instances_get_all_by_consistency_group_id(context, cg_id)
+def share_instances_get_all_by_share_group_id(context, share_group_id):
+    """Returns list of share instances that belong to given share group."""
+    return IMPL.share_instances_get_all_by_share_group_id(
+        context, share_group_id)
 
 ###################
 
 
-def share_create(context, values, create_share_instance=True):
+def share_create(context, share_values, create_share_instance=True):
     """Create new share."""
-    return IMPL.share_create(context, values,
+    return IMPL.share_create(context, share_values,
                              create_share_instance=create_share_instance)
 
 
-def share_data_get_for_project(context, project_id, session=None):
-    """Get (share_count, gigabytes) for project."""
-    return IMPL.share_data_get_for_project(context, project_id)
-
-
 def share_update(context, share_id, values):
     """Update share fields."""
     return IMPL.share_update(context, share_id, values)
@@ -385,12 +362,12 @@ def share_get_all_by_project(context, project_id, filters=None,
     )
 
 
-def share_get_all_by_consistency_group_id(context, cg_id,
-                                          filters=None, sort_key=None,
-                                          sort_dir=None):
-    """Returns all shares with given project ID and CG id."""
-    return IMPL.share_get_all_by_consistency_group_id(
-        context, cg_id, filters=filters,
+def share_get_all_by_share_group_id(context, share_group_id,
+                                    filters=None, sort_key=None,
+                                    sort_dir=None):
+    """Returns all shares with given project ID and share group id."""
+    return IMPL.share_get_all_by_share_group_id(
+        context, share_group_id, filters=filters,
         sort_key=sort_key, sort_dir=sort_dir)
 
 
@@ -416,39 +393,22 @@ def share_access_create(context, values):
     return IMPL.share_access_create(context, values)
 
 
-def share_instance_access_copy(context, share_id, instance_id):
-    """Maps the existing access rules for the share to the instance in the DB.
-
-    Adds the instance mapping to the share's access rules and
-    returns the share's access rules.
-    """
-    return IMPL.share_instance_access_copy(context, share_id, instance_id)
-
-
 def share_access_get(context, access_id):
     """Get share access rule."""
     return IMPL.share_access_get(context, access_id)
 
 
-def share_instance_access_get(context, access_id, instance_id):
-    """Get access rule mapping for share instance."""
-    return IMPL.share_instance_access_get(context, access_id, instance_id)
-
-
 def share_access_get_all_for_share(context, share_id):
     """Get all access rules for given share."""
     return IMPL.share_access_get_all_for_share(context, share_id)
 
 
-def share_instance_access_get_all(context, access_id, session=None):
-    """Get access rules to all share instances."""
-    return IMPL.share_instance_access_get_all(context, access_id, session=None)
-
-
-def share_access_get_all_for_instance(context, instance_id, session=None):
+def share_access_get_all_for_instance(context, instance_id, filters=None,
+                                      with_share_access_data=True):
     """Get all access rules related to a certain share instance."""
     return IMPL.share_access_get_all_for_instance(
-        context, instance_id, session=None)
+        context, instance_id, filters=filters,
+        with_share_access_data=with_share_access_data)
 
 
 def share_access_get_all_by_type_and_access(context, share_id, access_type,
@@ -458,22 +418,39 @@ def share_access_get_all_by_type_and_access(context, share_id, access_type,
         context, share_id, access_type, access)
 
 
-def share_access_delete(context, access_id):
-    """Deny access to share."""
-    return IMPL.share_access_delete(context, access_id)
+def share_instance_access_create(context, values, share_instance_id):
+    """Allow access to share instance."""
+    return IMPL.share_instance_access_create(
+        context, values, share_instance_id)
+
+
+def share_instance_access_copy(context, share_id, instance_id):
+    """Maps the existing access rules for the share to the instance in the DB.
+
+    Adds the instance mapping to the share's access rules and
+    returns the share's access rules.
+    """
+    return IMPL.share_instance_access_copy(context, share_id, instance_id)
+
+
+def share_instance_access_get(context, access_id, instance_id,
+                              with_share_access_data=True):
+    """Get access rule mapping for share instance."""
+    return IMPL.share_instance_access_get(
+        context, access_id, instance_id,
+        with_share_access_data=with_share_access_data)
+
+
+def share_instance_access_update(context, access_id, instance_id, updates):
+    """Update the access mapping row for a given share instance and access."""
+    return IMPL.share_instance_access_update(
+        context, access_id, instance_id, updates)
 
 
 def share_instance_access_delete(context, mapping_id):
     """Deny access to share instance."""
     return IMPL.share_instance_access_delete(context, mapping_id)
 
-
-def share_instance_update_access_status(context, share_instance_id, status):
-    """Update access rules status of share instance."""
-    return IMPL.share_instance_update_access_status(context, share_instance_id,
-                                                    status)
-
-
 ####################
 
 
@@ -517,16 +494,6 @@ def share_snapshot_create(context, values):
     return IMPL.share_snapshot_create(context, values)
 
 
-def snapshot_data_get_for_project(context, project_id, session=None):
-    """Get (snapshot_count, gigabytes) for project."""
-    return IMPL.snapshot_data_get_for_project(context, project_id)
-
-
-def share_snapshot_destroy(context, snapshot_id):
-    """Destroy the snapshot or raise if it does not exist."""
-    return IMPL.share_snapshot_destroy(context, snapshot_id)
-
-
 def share_snapshot_get(context, snapshot_id):
     """Get a snapshot or raise if it does not exist."""
     return IMPL.share_snapshot_get(context, snapshot_id)
@@ -558,6 +525,11 @@ def share_snapshot_get_all_for_share(context, share_id, filters=None,
     )
 
 
+def share_snapshot_get_latest_for_share(context, share_id):
+    """Get the most recent snapshot for a share."""
+    return IMPL.share_snapshot_get_latest_for_share(context, share_id)
+
+
 def share_snapshot_update(context, snapshot_id, values):
     """Set the given properties on an snapshot and update it.
 
@@ -566,11 +538,79 @@ def share_snapshot_update(context, snapshot_id, values):
     return IMPL.share_snapshot_update(context, snapshot_id, values)
 
 
-def share_snapshot_data_get_for_project(context, project_id, session=None):
-    """Get count and gigabytes used for snapshots for specified project."""
-    return IMPL.share_snapshot_data_get_for_project(context,
-                                                    project_id,
-                                                    session=None)
+###################
+def share_snapshot_access_create(context, values):
+    """Create a share snapshot access from the values dictionary."""
+    return IMPL.share_snapshot_access_create(context, values)
+
+
+def share_snapshot_access_get(context, access_id):
+    """Get share snapshot access rule from given access_id."""
+    return IMPL.share_snapshot_access_get(context, access_id)
+
+
+def share_snapshot_access_get_all_for_snapshot_instance(
+        context, snapshot_instance_id, session=None):
+    """Get all access rules related to a certain snapshot instance."""
+    return IMPL.share_snapshot_access_get_all_for_snapshot_instance(
+        context, snapshot_instance_id, session)
+
+
+def share_snapshot_access_get_all_for_share_snapshot(context,
+                                                     share_snapshot_id,
+                                                     filters):
+    """Get all access rules for a given share snapshot according to filters."""
+    return IMPL.share_snapshot_access_get_all_for_share_snapshot(
+        context, share_snapshot_id, filters)
+
+
+def share_snapshot_export_locations_get(context, snapshot_id):
+    """Get all export locations for a given share snapshot."""
+    return IMPL.share_snapshot_export_locations_get(context, snapshot_id)
+
+
+def share_snapshot_instance_access_update(
+        context, access_id, instance_id, updates):
+    """Update the state of the share snapshot instance access."""
+    return IMPL.share_snapshot_instance_access_update(
+        context, access_id, instance_id, updates)
+
+
+def share_snapshot_instance_access_get(context, share_snapshot_instance_id,
+                                       access_id):
+    """Get the share snapshot instance access related to given ids."""
+    return IMPL.share_snapshot_instance_access_get(
+        context, share_snapshot_instance_id, access_id)
+
+
+def share_snapshot_instance_access_delete(context, access_id,
+                                          snapshot_instance_id):
+    """Delete share snapshot instance access given its id."""
+    return IMPL.share_snapshot_instance_access_delete(
+        context, access_id, snapshot_instance_id)
+
+
+def share_snapshot_instance_export_location_create(context, values):
+    """Create a share snapshot instance export location."""
+    return IMPL.share_snapshot_instance_export_location_create(context, values)
+
+
+def share_snapshot_instance_export_locations_get_all(
+        context, share_snapshot_instance_id):
+    """Get the share snapshot instance export locations for given id."""
+    return IMPL.share_snapshot_instance_export_locations_get_all(
+        context, share_snapshot_instance_id)
+
+
+def share_snapshot_instance_export_location_get(context, el_id):
+    """Get the share snapshot instance export location for given id."""
+    return IMPL.share_snapshot_instance_export_location_get(
+        context, el_id)
+
+
+def share_snapshot_instance_export_location_delete(context, el_id):
+    """Delete share snapshot instance export location given its id."""
+    return IMPL.share_snapshot_instance_export_location_delete(context, el_id)
 
 
 ###################
@@ -636,10 +676,12 @@ def share_export_locations_get(context, share_id):
 
 
 def share_export_locations_get_by_share_id(context, share_id,
-                                           include_admin_only=True):
+                                           include_admin_only=True,
+                                           ignore_migration_destination=False):
     """Get all export locations of a share by its ID."""
     return IMPL.share_export_locations_get_by_share_id(
-        context, share_id, include_admin_only=include_admin_only)
+        context, share_id, include_admin_only=include_admin_only,
+        ignore_migration_destination=ignore_migration_destination)
 
 
 def share_export_locations_get_by_share_instance_id(context,
@@ -854,6 +896,11 @@ def share_type_get_by_name(context, name):
     return IMPL.share_type_get_by_name(context, name)
 
 
+def share_type_get_by_name_or_id(context, name_or_id):
+    """Get share type by name or ID and return None if not found."""
+    return IMPL.share_type_get_by_name_or_id(context, name_or_id)
+
+
 def share_type_access_get_all(context, type_id):
     """Get all share type access of a share type."""
     return IMPL.share_type_access_get_all(context, type_id)
@@ -874,14 +921,6 @@ def share_type_destroy(context, id):
     return IMPL.share_type_destroy(context, id)
 
 
-def volume_get_active_by_window(context, begin, end=None, project_id=None):
-    """Get all the volumes inside the window.
-
-    Specifying a project_id will filter for a certain project.
-    """
-    return IMPL.volume_get_active_by_window(context, begin, end, project_id)
-
-
 ####################
 
 
@@ -907,21 +946,21 @@ def share_type_extra_specs_update_or_create(context, share_type_id,
                                                         extra_specs)
 
 
-def driver_private_data_get(context, host, entity_id, key=None, default=None):
-    """Get one, list or all key-value pairs for given host and entity_id."""
-    return IMPL.driver_private_data_get(context, host, entity_id, key, default)
+def driver_private_data_get(context, entity_id, key=None, default=None):
+    """Get one, list or all key-value pairs for given entity_id."""
+    return IMPL.driver_private_data_get(context, entity_id, key, default)
 
 
-def driver_private_data_update(context, host, entity_id, details,
+def driver_private_data_update(context, entity_id, details,
                                delete_existing=False):
-    """Update key-value pairs for given host and entity_id."""
-    return IMPL.driver_private_data_update(context, host, entity_id, details,
+    """Update key-value pairs for given entity_id."""
+    return IMPL.driver_private_data_update(context, entity_id, details,
                                            delete_existing)
 
 
-def driver_private_data_delete(context, host, entity_id, key=None):
-    """Remove one, list or all key-value pairs for given host and entity_id."""
-    return IMPL.driver_private_data_delete(context, host, entity_id, key)
+def driver_private_data_delete(context, entity_id, key=None):
+    """Remove one, list or all key-value pairs for given entity_id."""
+    return IMPL.driver_private_data_delete(context, entity_id, key)
 
 
 ####################
@@ -938,126 +977,148 @@ def availability_zone_get_all(context):
 
 ####################
 
-def consistency_group_get(context, consistency_group_id):
-    """Get a consistency group or raise if it does not exist."""
-    return IMPL.consistency_group_get(context, consistency_group_id)
+def share_group_get(context, share_group_id):
+    """Get a share group or raise if it does not exist."""
+    return IMPL.share_group_get(context, share_group_id)
 
 
-def consistency_group_get_all(context, detailed=True):
-    """Get all consistency groups."""
-    return IMPL.consistency_group_get_all(context, detailed=detailed)
+def share_group_get_all(context, detailed=True, filters=None, sort_key=None,
+                        sort_dir=None):
+    """Get all share groups."""
+    return IMPL.share_group_get_all(
+        context, detailed=detailed, filters=filters, sort_key=sort_key,
+        sort_dir=sort_dir)
 
 
-def consistency_group_get_all_by_host(context, host, detailed=True):
-    """Get all consistency groups belonging to a host."""
-    return IMPL.consistency_group_get_all_by_host(context, host,
-                                                  detailed=detailed)
+def share_group_get_all_by_host(context, host, detailed=True, filters=None,
+                                sort_key=None, sort_dir=None):
+    """Get all share groups belonging to a host."""
+    return IMPL.share_group_get_all_by_host(
+        context, host, detailed=detailed, filters=filters, sort_key=sort_key,
+        sort_dir=sort_dir)
 
 
-def consistency_group_create(context, values):
-    """Create a consistency group from the values dictionary."""
-    return IMPL.consistency_group_create(context, values)
+def share_group_create(context, values):
+    """Create a share group from the values dictionary."""
+    return IMPL.share_group_create(context, values)
 
 
-def consistency_group_get_all_by_share_server(context, share_server_id):
-    """Get all consistency groups associated with a share server."""
-    return IMPL.consistency_group_get_all_by_share_server(context,
-                                                          share_server_id)
+def share_group_get_all_by_share_server(context, share_server_id,
+                                        filters=None, sort_key=None,
+                                        sort_dir=None):
+    """Get all share groups associated with a share server."""
+    return IMPL.share_group_get_all_by_share_server(
+        context, share_server_id, filters=filters, sort_key=sort_key,
+        sort_dir=sort_dir)
 
 
-def consistency_group_get_all_by_project(context, project_id, detailed=True):
-    """Get all consistency groups belonging to a project."""
-    return IMPL.consistency_group_get_all_by_project(context, project_id,
-                                                     detailed=detailed)
+def share_group_get_all_by_project(context, project_id, detailed=True,
+                                   filters=None, sort_key=None,
+                                   sort_dir=None):
+    """Get all share groups belonging to a project."""
+    return IMPL.share_group_get_all_by_project(
+        context, project_id, detailed=detailed, filters=filters,
+        sort_key=sort_key, sort_dir=sort_dir)
 
 
-def consistency_group_update(context, consistency_group_id, values):
-    """Set the given properties on a consistency group and update it.
+def share_group_update(context, share_group_id, values):
+    """Set the given properties on a share group and update it.
 
-    Raises NotFound if consistency group does not exist.
+    Raises NotFound if share group does not exist.
     """
-    return IMPL.consistency_group_update(context, consistency_group_id, values)
+    return IMPL.share_group_update(context, share_group_id, values)
 
 
-def consistency_group_destroy(context, consistency_group_id):
-    """Destroy the consistency group or raise if it does not exist."""
-    return IMPL.consistency_group_destroy(context, consistency_group_id)
+def share_group_destroy(context, share_group_id):
+    """Destroy the share group or raise if it does not exist."""
+    return IMPL.share_group_destroy(context, share_group_id)
 
 
-def count_shares_in_consistency_group(context, consistency_group_id):
-    """Returns the number of undeleted shares with the specified cg."""
-    return IMPL.count_shares_in_consistency_group(context,
-                                                  consistency_group_id)
+def count_shares_in_share_group(context, share_group_id):
+    """Returns the number of undeleted shares with the specified group."""
+    return IMPL.count_shares_in_share_group(context, share_group_id)
 
 
-def count_cgsnapshots_in_consistency_group(context, consistency_group_id):
-    """Returns the number of undeleted cgsnapshots with the specified cg."""
-    return IMPL.count_cgsnapshots_in_consistency_group(context,
-                                                       consistency_group_id)
+def get_all_shares_by_share_group(context, share_group_id):
+    return IMPL.get_all_shares_by_share_group(context, share_group_id)
 
 
-def count_consistency_groups_in_share_network(context, share_network_id,
-                                              session=None):
-    """Returns the number of undeleted cgs with the specified share network."""
-    return IMPL.count_consistency_groups_in_share_network(context,
-                                                          share_network_id)
+def count_share_group_snapshots_in_share_group(context, share_group_id):
+    """Returns the number of sg snapshots with the specified share group."""
+    return IMPL.count_share_group_snapshots_in_share_group(
+        context, share_group_id)
 
 
-def count_cgsnapshot_members_in_share(context, share_id, session=None):
-    """Returns the number of cgsnapshot members linked to the share."""
-    return IMPL.count_cgsnapshot_members_in_share(context, share_id)
+def count_share_groups_in_share_network(context, share_network_id,
+                                        session=None):
+    """Return the number of groups with the specified share network."""
+    return IMPL.count_share_groups_in_share_network(context, share_network_id)
 
 
-def cgsnapshot_get(context, cgsnapshot_id):
-    """Get a cgsnapshot."""
-    return IMPL.cgsnapshot_get(context, cgsnapshot_id)
+def count_share_group_snapshot_members_in_share(context, share_id,
+                                                session=None):
+    """Returns the number of group snapshot members linked to the share."""
+    return IMPL.count_share_group_snapshot_members_in_share(context, share_id)
 
 
-def cgsnapshot_get_all(context, detailed=True):
-    """Get all cgsnapshots."""
-    return IMPL.cgsnapshot_get_all(context, detailed=detailed)
+def share_group_snapshot_get(context, share_group_snapshot_id):
+    """Get a share group snapshot."""
+    return IMPL.share_group_snapshot_get(context, share_group_snapshot_id)
 
 
-def cgsnapshot_get_all_by_project(context, project_id, detailed=True):
-    """Get all cgsnapshots belonging to a project."""
-    return IMPL.cgsnapshot_get_all_by_project(context, project_id,
-                                              detailed=detailed)
+def share_group_snapshot_get_all(context, detailed=True, filters=None,
+                                 sort_key=None, sort_dir=None):
+    """Get all share group snapshots."""
+    return IMPL.share_group_snapshot_get_all(
+        context, detailed=detailed, filters=filters, sort_key=sort_key,
+        sort_dir=sort_dir)
 
 
-def cgsnapshot_create(context, values):
-    """Create a cgsnapshot from the values dictionary."""
-    return IMPL.cgsnapshot_create(context, values)
+def share_group_snapshot_get_all_by_project(context, project_id, detailed=True,
+                                            filters=None, sort_key=None,
+                                            sort_dir=None):
+    """Get all share group snapshots belonging to a project."""
+    return IMPL.share_group_snapshot_get_all_by_project(
+        context, project_id, detailed=detailed, filters=filters,
+        sort_key=sort_key, sort_dir=sort_dir)
 
 
-def cgsnapshot_update(context, cgsnapshot_id, values):
-    """Set the given properties on a cgsnapshot and update it.
+def share_group_snapshot_create(context, values):
+    """Create a share group snapshot from the values dictionary."""
+    return IMPL.share_group_snapshot_create(context, values)
 
-    Raises NotFound if cgsnapshot does not exist.
+
+def share_group_snapshot_update(context, share_group_snapshot_id, values):
+    """Set the given properties on a share group snapshot and update it.
+
+    Raises NotFound if share group snapshot does not exist.
     """
-    return IMPL.cgsnapshot_update(context, cgsnapshot_id, values)
+    return IMPL.share_group_snapshot_update(
+        context, share_group_snapshot_id, values)
 
 
-def cgsnapshot_destroy(context, cgsnapshot_id):
-    """Destroy the cgsnapshot or raise if it does not exist."""
-    return IMPL.cgsnapshot_destroy(context, cgsnapshot_id)
+def share_group_snapshot_destroy(context, share_group_snapshot_id):
+    """Destroy the share_group_snapshot or raise if it does not exist."""
+    return IMPL.share_group_snapshot_destroy(context, share_group_snapshot_id)
 
 
-def cgsnapshot_members_get_all(context, cgsnapshot_id):
-    """Return the members of a cgsnapshot."""
-    return IMPL.cgsnapshot_members_get_all(context, cgsnapshot_id)
+def share_group_snapshot_members_get_all(context, share_group_snapshot_id):
+    """Return the members of a share group snapshot."""
+    return IMPL.share_group_snapshot_members_get_all(
+        context, share_group_snapshot_id)
 
 
-def cgsnapshot_member_create(context, values):
-    """Create a cgsnapshot member from the values dictionary."""
-    return IMPL.cgsnapshot_member_create(context, values)
+def share_group_snapshot_member_create(context, values):
+    """Create a share group snapshot member from the values dictionary."""
+    return IMPL.share_group_snapshot_member_create(context, values)
 
 
-def cgsnapshot_member_update(context, member_id, values):
-    """Set the given properties on a cgsnapshot member and update it.
+def share_group_snapshot_member_update(context, member_id, values):
+    """Set the given properties on a share group snapshot member and update it.
 
-    Raises NotFound if cgsnapshot member does not exist.
+    Raises NotFound if share_group_snapshot member does not exist.
     """
-    return IMPL.cgsnapshot_member_update(context, member_id, values)
+    return IMPL.share_group_snapshot_member_update(context, member_id, values)
 
 
 ####################
@@ -1087,15 +1148,6 @@ def share_replicas_get_available_active_replica(context, share_id,
         with_share_data=with_share_data)
 
 
-def share_replicas_get_active_replicas_by_share(context, share_id,
-                                                with_share_server=False,
-                                                with_share_data=False):
-    """Returns all active replicas for a given share."""
-    return IMPL.share_replicas_get_active_replicas_by_share(
-        context, share_id, with_share_server=with_share_server,
-        with_share_data=with_share_data)
-
-
 def share_replica_get(context, replica_id, with_share_server=False,
                       with_share_data=False):
     """Get share replica by id."""
@@ -1114,3 +1166,96 @@ def share_replica_update(context, share_replica_id, values,
 def share_replica_delete(context, share_replica_id):
     """Deletes a share replica."""
     return IMPL.share_replica_delete(context, share_replica_id)
+
+
+def purge_deleted_records(context, age_in_days):
+    """Purge deleted rows older than given age from all tables
+
+    :raises: InvalidParameterValue if age_in_days is incorrect.
+    """
+    return IMPL.purge_deleted_records(context, age_in_days=age_in_days)
+
+
+####################
+
+
+def share_group_type_create(context, values, projects=None):
+    """Create a new share group type."""
+    return IMPL.share_group_type_create(context, values, projects)
+
+
+def share_group_type_get_all(context, inactive=False, filters=None):
+    """Get all share group types.
+
+    :param context: context to query under
+    :param inactive: Include inactive share group types to the result set
+    :param filters: Filters for the query in the form of key/value.
+        :is_public: Filter share group types based on visibility:
+
+            * **True**: List public group types only
+            * **False**: List private group types only
+            * **None**: List both public and private group types
+
+    :returns: list of matching share group types
+    """
+    return IMPL.share_group_type_get_all(context, inactive, filters)
+
+
+def share_group_type_get(context, type_id, inactive=False,
+                         expected_fields=None):
+    """Get share_group type by id.
+
+    :param context: context to query under
+    :param type_id: group type id to get.
+    :param inactive: Consider inactive group types when searching
+    :param expected_fields: Return those additional fields.
+                            Supported fields are: projects.
+    :returns: share group type
+    """
+    return IMPL.share_group_type_get(
+        context, type_id, inactive, expected_fields)
+
+
+def share_group_type_get_by_name(context, name):
+    """Get share group type by name."""
+    return IMPL.share_group_type_get_by_name(context, name)
+
+
+def share_group_type_access_get_all(context, type_id):
+    """Get all share group type access of a share group type."""
+    return IMPL.share_group_type_access_get_all(context, type_id)
+
+
+def share_group_type_access_add(context, type_id, project_id):
+    """Add share group type access for project."""
+    return IMPL.share_group_type_access_add(context, type_id, project_id)
+
+
+def share_group_type_access_remove(context, type_id, project_id):
+    """Remove share group type access for project."""
+    return IMPL.share_group_type_access_remove(context, type_id, project_id)
+
+
+def share_group_type_destroy(context, type_id):
+    """Delete a share group type."""
+    return IMPL.share_group_type_destroy(context, type_id)
+
+
+def share_group_type_specs_get(context, type_id):
+    """Get all group specs for a share group type."""
+    return IMPL.share_group_type_specs_get(context, type_id)
+
+
+def share_group_type_specs_delete(context, type_id, key):
+    """Delete the given group specs item."""
+    return IMPL.share_group_type_specs_delete(context, type_id, key)
+
+
+def share_group_type_specs_update_or_create(context, type_id, group_specs):
+    """Create or update share group type specs.
+
+    This adds or modifies the key/value pairs specified in the group
+    specs dict argument.
+    """
+    return IMPL.share_group_type_specs_update_or_create(
+        context, type_id, group_specs)
