@@ -189,7 +189,7 @@ class Share(BASE, ManilaBase):
     __tablename__ = 'shares'
     _extra_keys = ['name', 'export_location', 'export_locations', 'status',
                    'host', 'share_server_id', 'share_network_id',
-                   'availability_zone', 'access_rules_status']
+                   'availability_zone', 'access_rules_status', 'share_type_id']
 
     @property
     def name(self):
@@ -227,7 +227,8 @@ class Share(BASE, ManilaBase):
 
     def __getattr__(self, item):
         deprecated_properties = ('host', 'share_server_id', 'share_network_id',
-                                 'availability_zone')
+                                 'availability_zone', 'share_type_id',
+                                 'share_type')
         proxified_properties = ('status',) + deprecated_properties
 
         if item in deprecated_properties:
@@ -263,7 +264,8 @@ class Share(BASE, ManilaBase):
         # preferred.
         result = None
         if len(self.instances) > 0:
-            order = (constants.STATUS_REPLICATION_CHANGE,
+            order = (constants.STATUS_REVERTING,
+                     constants.STATUS_REPLICATION_CHANGE,
                      constants.STATUS_MIGRATING, constants.STATUS_AVAILABLE,
                      constants.STATUS_ERROR)
             other_statuses = (
@@ -301,16 +303,17 @@ class Share(BASE, ManilaBase):
     display_description = Column(String(255))
     snapshot_id = Column(String(36))
     snapshot_support = Column(Boolean, default=True)
+    create_share_from_snapshot_support = Column(Boolean, default=True)
+    revert_to_snapshot_support = Column(Boolean, default=False)
     replication_type = Column(String(255), nullable=True)
+    mount_snapshot_support = Column(Boolean, default=False)
     share_proto = Column(String(255))
-    share_type_id = Column(String(36), ForeignKey('share_types.id'),
-                           nullable=True)
     is_public = Column(Boolean, default=False)
-    consistency_group_id = Column(String(36),
-                                  ForeignKey('consistency_groups.id'),
-                                  nullable=True)
+    share_group_id = Column(String(36),
+                            ForeignKey('share_groups.id'),
+                            nullable=True)
 
-    source_cgsnapshot_member_id = Column(String(36), nullable=True)
+    source_share_group_snapshot_member_id = Column(String(36), nullable=True)
     task_state = Column(String(255))
     instances = orm.relationship(
         "ShareInstance",
@@ -323,13 +326,6 @@ class Share(BASE, ManilaBase):
         viewonly=True,
         join_depth=2,
     )
-    share_type = orm.relationship(
-        "ShareTypes",
-        lazy=True,
-        foreign_keys=share_type_id,
-        primaryjoin='and_('
-                    'Share.share_type_id == ShareTypes.id, '
-                    'ShareTypes.deleted == "False")')
 
 
 class ShareInstance(BASE, ManilaBase):
@@ -339,9 +335,10 @@ class ShareInstance(BASE, ManilaBase):
                    'replica_state']
     _proxified_properties = ('user_id', 'project_id', 'size',
                              'display_name', 'display_description',
-                             'snapshot_id', 'share_proto', 'share_type_id',
-                             'is_public', 'consistency_group_id',
-                             'source_cgsnapshot_member_id')
+                             'snapshot_id', 'share_proto', 'is_public',
+                             'share_group_id', 'replication_type',
+                             'source_share_group_snapshot_member_id',
+                             'mount_snapshot_support')
 
     def set_share_data(self, share):
         for share_property in self._proxified_properties:
@@ -369,24 +366,22 @@ class ShareInstance(BASE, ManilaBase):
 
     ACCESS_STATUS_PRIORITIES = {
         constants.STATUS_ACTIVE: 0,
-        constants.STATUS_OUT_OF_SYNC: 1,
-        constants.STATUS_UPDATING: 2,
-        constants.STATUS_UPDATING_MULTIPLE: 3,
-        constants.STATUS_ERROR: 4,
+        constants.SHARE_INSTANCE_RULES_SYNCING: 1,
+        constants.SHARE_INSTANCE_RULES_ERROR: 2,
     }
 
     access_rules_status = Column(Enum(constants.STATUS_ACTIVE,
-                                      constants.STATUS_OUT_OF_SYNC,
-                                      constants.STATUS_UPDATING,
-                                      constants.STATUS_UPDATING_MULTIPLE,
-                                      constants.STATUS_ERROR),
+                                      constants.SHARE_INSTANCE_RULES_SYNCING,
+                                      constants.SHARE_INSTANCE_RULES_ERROR),
                                  default=constants.STATUS_ACTIVE)
 
     scheduled_at = Column(DateTime)
     launched_at = Column(DateTime)
     terminated_at = Column(DateTime)
     replica_state = Column(String(255), nullable=True)
-
+    cast_rules_to_readonly = Column(Boolean, default=False, nullable=False)
+    share_type_id = Column(String(36), ForeignKey('share_types.id'),
+                           nullable=True)
     availability_zone_id = Column(String(36),
                                   ForeignKey('availability_zones.id'),
                                   nullable=True)
@@ -416,6 +411,13 @@ class ShareInstance(BASE, ManilaBase):
                               nullable=True)
     share_server_id = Column(String(36), ForeignKey('share_servers.id'),
                              nullable=True)
+    share_type = orm.relationship(
+        "ShareTypes",
+        lazy='immediate',
+        foreign_keys=share_type_id,
+        primaryjoin='and_('
+                    'ShareInstance.share_type_id == ShareTypes.id, '
+                    'ShareTypes.deleted == "False")')
 
 
 class ShareInstanceExportLocations(BASE, ManilaBase):
@@ -538,10 +540,20 @@ class ShareAccessMapping(BASE, ManilaBase):
     share_id = Column(String(36), ForeignKey('shares.id'))
     access_type = Column(String(255))
     access_to = Column(String(255))
+    access_key = Column(String(255), nullable=True)
 
     access_level = Column(Enum(*constants.ACCESS_LEVELS),
                           default=constants.ACCESS_LEVEL_RW)
 
+    @property
+    def state(self):
+        """Get the aggregated 'state' from all the instance mapping states.
+
+        An access rule is supposed to be truly 'active' when it has been
+        applied across all of the share instances of the parent share object.
+        """
+        return get_aggregated_access_rules_state(self.instance_mappings)
+
     instance_mappings = orm.relationship(
         "ShareInstanceAccessMapping",
         lazy='immediate',
@@ -553,28 +565,23 @@ class ShareAccessMapping(BASE, ManilaBase):
         )
     )
 
-    @property
-    def state(self):
-        instances = [im.instance for im in self.instance_mappings]
-        access_rules_status = get_access_rules_status(instances)
-
-        if access_rules_status in (
-                constants.STATUS_OUT_OF_SYNC,
-                constants.STATUS_UPDATING,
-                constants.STATUS_UPDATING_MULTIPLE):
-            return constants.STATUS_NEW
-        else:
-            return access_rules_status
-
 
 class ShareInstanceAccessMapping(BASE, ManilaBase):
     """Represents access to individual share instances."""
 
     __tablename__ = 'share_instance_access_map'
+    _proxified_properties = ('share_id', 'access_type', 'access_key',
+                             'access_to', 'access_level')
+
+    def set_share_access_data(self, share_access):
+        for share_access_attr in self._proxified_properties:
+            setattr(self, share_access_attr, share_access[share_access_attr])
+
     id = Column(String(36), primary_key=True)
     deleted = Column(String(36), default='False')
     share_instance_id = Column(String(36), ForeignKey('share_instances.id'))
     access_id = Column(String(36), ForeignKey('share_access_map.id'))
+    state = Column(String(255), default=constants.ACCESS_STATE_QUEUED_TO_APPLY)
 
     instance = orm.relationship(
         "ShareInstance",
@@ -602,6 +609,25 @@ class ShareSnapshot(BASE, ManilaBase):
 
         raise AttributeError(item)
 
+    @property
+    def export_locations(self):
+        # TODO(gouthamr): Return AZ specific export locations for replicated
+        # snapshots.
+        # NOTE(gouthamr): For a replicated snapshot, export locations of the
+        # 'active' instances are chosen, if 'available'.
+        all_export_locations = []
+        select_instances = list(filter(
+            lambda x: (x['share_instance']['replica_state'] ==
+                       constants.REPLICA_STATE_ACTIVE),
+            self.instances)) or self.instances
+
+        for instance in select_instances:
+            if instance['status'] == constants.STATUS_AVAILABLE:
+                for export_location in instance.export_locations:
+                    all_export_locations.append(export_location)
+
+        return all_export_locations
+
     @property
     def name(self):
         return CONF.share_snapshot_name_template % self.id
@@ -621,7 +647,13 @@ class ShareSnapshot(BASE, ManilaBase):
             replica_snapshots = list(filter(
                 lambda x: qualified_replica(x.share_instance), self.instances))
 
-            snapshot_instances = replica_snapshots or self.instances
+            migrating_snapshots = list(filter(
+                lambda x: x.share_instance['status'] ==
+                constants.STATUS_MIGRATING, self.instances))
+
+            snapshot_instances = (replica_snapshots or migrating_snapshots
+                                  or self.instances)
+
             result = snapshot_instances[0]
 
         return result
@@ -648,7 +680,8 @@ class ShareSnapshot(BASE, ManilaBase):
             return self.status
 
         order = (constants.STATUS_DELETING, constants.STATUS_CREATING,
-                 constants.STATUS_ERROR, constants.STATUS_AVAILABLE)
+                 constants.STATUS_ERROR, constants.STATUS_MIGRATING,
+                 constants.STATUS_AVAILABLE)
         other_statuses = [x['status'] for x in self.instances if
                           x['status'] not in order]
         order = (order + tuple(other_statuses))
@@ -723,6 +756,92 @@ class ShareSnapshotInstance(BASE, ManilaBase):
             'ShareSnapshotInstance.deleted == "False")')
     )
 
+    export_locations = orm.relationship(
+        "ShareSnapshotInstanceExportLocation",
+        lazy='immediate',
+        primaryjoin=(
+            'and_('
+            'ShareSnapshotInstance.id == '
+            'ShareSnapshotInstanceExportLocation.share_snapshot_instance_id, '
+            'ShareSnapshotInstanceExportLocation.deleted == "False")'
+        )
+    )
+
+
+class ShareSnapshotAccessMapping(BASE, ManilaBase):
+    """Represents access to share snapshot."""
+    __tablename__ = 'share_snapshot_access_map'
+
+    @property
+    def state(self):
+        """Get the aggregated 'state' from all the instance mapping states.
+
+        An access rule is supposed to be truly 'active' when it has been
+        applied across all of the share snapshot instances of the parent
+        share snapshot object.
+        """
+        return get_aggregated_access_rules_state(self.instance_mappings)
+
+    id = Column(String(36), primary_key=True)
+    deleted = Column(String(36), default='False')
+    share_snapshot_id = Column(String(36), ForeignKey('share_snapshots.id'))
+    access_type = Column(String(255))
+    access_to = Column(String(255))
+
+    instance_mappings = orm.relationship(
+        "ShareSnapshotInstanceAccessMapping",
+        lazy='immediate',
+        primaryjoin=(
+            'and_('
+            'ShareSnapshotAccessMapping.id == '
+            'ShareSnapshotInstanceAccessMapping.access_id, '
+            'ShareSnapshotInstanceAccessMapping.deleted == "False")'
+        )
+    )
+
+
+class ShareSnapshotInstanceAccessMapping(BASE, ManilaBase):
+    """Represents access to individual share snapshot instances."""
+
+    __tablename__ = 'share_snapshot_instance_access_map'
+    _proxified_properties = ('share_snapshot_id', 'access_type', 'access_to')
+
+    def set_snapshot_access_data(self, snapshot_access):
+        for snapshot_access_attr in self._proxified_properties:
+            setattr(self, snapshot_access_attr,
+                    snapshot_access[snapshot_access_attr])
+
+    id = Column(String(36), primary_key=True)
+    deleted = Column(String(36), default='False')
+    share_snapshot_instance_id = Column(String(36), ForeignKey(
+        'share_snapshot_instances.id'))
+    access_id = Column(String(36), ForeignKey('share_snapshot_access_map.id'))
+    state = Column(Enum(*constants.ACCESS_RULES_STATES),
+                   default=constants.ACCESS_STATE_QUEUED_TO_APPLY)
+
+    instance = orm.relationship(
+        "ShareSnapshotInstance",
+        lazy='immediate',
+        primaryjoin=(
+            'and_('
+            'ShareSnapshotInstanceAccessMapping.share_snapshot_instance_id == '
+            'ShareSnapshotInstance.id, '
+            'ShareSnapshotInstanceAccessMapping.deleted == "False")'
+        )
+    )
+
+
+class ShareSnapshotInstanceExportLocation(BASE, ManilaBase):
+    """Represents export locations of share snapshot instances."""
+    __tablename__ = 'share_snapshot_instance_export_locations'
+
+    id = Column(String(36), primary_key=True)
+    share_snapshot_instance_id = Column(
+        String(36), ForeignKey('share_snapshot_instances.id'), nullable=False)
+    path = Column(String(2000))
+    is_admin_only = Column(Boolean, default=False, nullable=False)
+    deleted = Column(String(36), default='False')
+
 
 class SecurityService(BASE, ManilaBase):
     """Security service information for manila shares."""
@@ -730,7 +849,7 @@ class SecurityService(BASE, ManilaBase):
     __tablename__ = 'security_services'
     id = Column(String(36), primary_key=True)
     deleted = Column(String(36), default='False')
-    project_id = Column(String(36), nullable=False)
+    project_id = Column(String(255), nullable=False)
     type = Column(String(32), nullable=False)
     dns_ip = Column(String(64), nullable=True)
     server = Column(String(255), nullable=True)
@@ -746,14 +865,15 @@ class ShareNetwork(BASE, ManilaBase):
     __tablename__ = 'share_networks'
     id = Column(String(36), primary_key=True, nullable=False)
     deleted = Column(String(36), default='False')
-    project_id = Column(String(36), nullable=False)
-    user_id = Column(String(36), nullable=False)
-    nova_net_id = Column(String(36), nullable=True)
+    project_id = Column(String(255), nullable=False)
+    user_id = Column(String(255), nullable=False)
     neutron_net_id = Column(String(36), nullable=True)
     neutron_subnet_id = Column(String(36), nullable=True)
     network_type = Column(String(32), nullable=True)
     segmentation_id = Column(Integer, nullable=True)
     cidr = Column(String(64), nullable=True)
+    gateway = Column(String(64), nullable=True)
+    mtu = Column(Integer, nullable=True)
     ip_version = Column(Integer, nullable=True)
     name = Column(String(255), nullable=True)
     description = Column(String(255), nullable=True)
@@ -806,10 +926,10 @@ class ShareServer(BASE, ManilaBase):
                     'ShareServer.id == ShareInstance.share_server_id,'
                     'ShareInstance.deleted == "False")')
 
-    consistency_groups = orm.relationship(
-        "ConsistencyGroup", backref='share_server', primaryjoin='and_('
-        'ShareServer.id == ConsistencyGroup.share_server_id,'
-        'ConsistencyGroup.deleted == "False")')
+    share_groups = orm.relationship(
+        "ShareGroup", backref='share_server', primaryjoin='and_('
+        'ShareServer.id == ShareGroup.share_server_id,'
+        'ShareGroup.deleted == "False")')
 
     _backend_details = orm.relationship(
         "ShareServerBackendDetails",
@@ -862,6 +982,8 @@ class NetworkAllocation(BASE, ManilaBase):
     ip_address = Column(String(64), nullable=True)
     ip_version = Column(Integer, nullable=True)
     cidr = Column(String(64), nullable=True)
+    gateway = Column(String(64), nullable=True)
+    mtu = Column(Integer, nullable=True)
     network_type = Column(String(32), nullable=True)
     segmentation_id = Column(Integer, nullable=True)
     mac_address = Column(String(32), nullable=True)
@@ -872,7 +994,6 @@ class NetworkAllocation(BASE, ManilaBase):
 class DriverPrivateData(BASE, ManilaBase):
     """Represents a private data as key-value pairs for a driver."""
     __tablename__ = 'drivers_private_data'
-    host = Column(String(255), nullable=False, primary_key=True)
     entity_uuid = Column(String(36), nullable=False, primary_key=True)
     key = Column(String(255), nullable=False, primary_key=True)
     value = Column(String(1023), nullable=False)
@@ -886,95 +1007,180 @@ class AvailabilityZone(BASE, ManilaBase):
     name = Column(String(255), nullable=False)
 
 
-class ConsistencyGroup(BASE, ManilaBase):
-    """Represents a consistency group."""
-    __tablename__ = 'consistency_groups'
+class ShareGroupTypes(BASE, ManilaBase):
+    """Represent possible share group types of shares offered."""
+    __tablename__ = "share_group_types"
+    __table_args__ = (
+        schema.UniqueConstraint(
+            "name", "deleted", name="uniq_share_group_type_name"),
+    )
     id = Column(String(36), primary_key=True)
+    deleted = Column(String(36), default='False')
+    name = Column(String(255))
+    is_public = Column(Boolean, default=True)
 
+
+class ShareGroup(BASE, ManilaBase):
+    """Represents a share group."""
+    __tablename__ = 'share_groups'
+    id = Column(String(36), primary_key=True)
     user_id = Column(String(255), nullable=False)
     project_id = Column(String(255), nullable=False)
     deleted = Column(String(36), default='False')
-
     host = Column(String(255))
     name = Column(String(255))
     description = Column(String(255))
     status = Column(String(255))
-    source_cgsnapshot_id = Column(String(36))
-    share_network_id = Column(String(36), ForeignKey('share_networks.id'),
-                              nullable=True)
-    share_server_id = Column(String(36), ForeignKey('share_servers.id'),
-                             nullable=True)
+    source_share_group_snapshot_id = Column(String(36))
+    share_network_id = Column(
+        String(36), ForeignKey('share_networks.id'), nullable=True)
+    share_server_id = Column(
+        String(36), ForeignKey('share_servers.id'), nullable=True)
+    share_group_type_id = Column(
+        String(36), ForeignKey('share_group_types.id'), nullable=True)
+    consistent_snapshot_support = Column(Enum('pool', 'host'), default=None)
+    share_group_type = orm.relationship(
+        ShareGroupTypes,
+        backref="share_groups",
+        foreign_keys=share_group_type_id,
+        primaryjoin='and_('
+                    'ShareGroup.share_group_type_id == '
+                    'ShareGroupTypes.id,'
+                    'ShareGroup.deleted == 0)')
 
 
-class CGSnapshot(BASE, ManilaBase):
-    """Represents a cgsnapshot."""
-    __tablename__ = 'cgsnapshots'
+class ShareGroupTypeProjects(BASE, ManilaBase):
+    """Represent projects associated share group types."""
+    __tablename__ = "share_group_type_projects"
+    __table_args__ = (schema.UniqueConstraint(
+        "share_group_type_id", "project_id", "deleted",
+        name=("uniq_share_group_type_projects0share_group_type_id"
+              "0project_id0deleted")),
+    )
+    id = Column(Integer, primary_key=True)
+    share_group_type_id = Column(
+        String, ForeignKey('share_group_types.id'), nullable=False)
+    project_id = Column(String(255))
+    share_group_type = orm.relationship(
+        ShareGroupTypes,
+        backref="projects",
+        foreign_keys=share_group_type_id,
+        primaryjoin='and_('
+                    'ShareGroupTypeProjects.share_group_type_id == '
+                    'ShareGroupTypes.id,'
+                    'ShareGroupTypeProjects.deleted == 0)')
+
+
+class ShareGroupTypeSpecs(BASE, ManilaBase):
+    """Represents additional specs for a share group type."""
+    __tablename__ = 'share_group_type_specs'
+    id = Column(Integer, primary_key=True)
+    key = Column("spec_key", String(255))
+    value = Column("spec_value", String(255))
+    share_group_type_id = Column(
+        String(36), ForeignKey('share_group_types.id'), nullable=False)
+    share_group_type = orm.relationship(
+        ShareGroupTypes,
+        backref="group_specs",
+        foreign_keys=share_group_type_id,
+        primaryjoin='and_('
+        'ShareGroupTypeSpecs.share_group_type_id == ShareGroupTypes.id,'
+        'ShareGroupTypeSpecs.deleted == 0)'
+    )
+
+
+class ShareGroupSnapshot(BASE, ManilaBase):
+    """Represents a share group snapshot."""
+    __tablename__ = 'share_group_snapshots'
     id = Column(String(36), primary_key=True)
-
-    consistency_group_id = Column(String(36),
-                                  ForeignKey('consistency_groups.id'))
+    share_group_id = Column(String(36), ForeignKey('share_groups.id'))
     user_id = Column(String(255), nullable=False)
     project_id = Column(String(255), nullable=False)
     deleted = Column(String(36), default='False')
-
     name = Column(String(255))
     description = Column(String(255))
     status = Column(String(255))
-
-    consistency_group = orm.relationship(
-        ConsistencyGroup,
-        backref="cgsnapshots",
-        foreign_keys=consistency_group_id,
+    share_group = orm.relationship(
+        ShareGroup,
+        backref="snapshots",
+        foreign_keys=share_group_id,
         primaryjoin=('and_('
-                     'CGSnapshot.consistency_group_id == ConsistencyGroup.id,'
-                     'CGSnapshot.deleted == "False")')
+                     'ShareGroupSnapshot.share_group_id == ShareGroup.id,'
+                     'ShareGroupSnapshot.deleted == "False")')
     )
 
 
-class ConsistencyGroupShareTypeMapping(BASE, ManilaBase):
-    """Represents the share types in a consistency group."""
-    __tablename__ = 'consistency_group_share_type_mappings'
+class ShareGroupTypeShareTypeMapping(BASE, ManilaBase):
+    """Represents the share types supported by a share group type."""
+    __tablename__ = 'share_group_type_share_type_mappings'
     id = Column(String(36), primary_key=True)
     deleted = Column(String(36), default='False')
-    consistency_group_id = Column(String(36),
-                                  ForeignKey('consistency_groups.id'),
-                                  nullable=False)
-    share_type_id = Column(String(36),
-                           ForeignKey('share_types.id'),
-                           nullable=False)
-
-    consistency_group = orm.relationship(
-        ConsistencyGroup,
+    share_group_type_id = Column(
+        String(36), ForeignKey('share_group_types.id'), nullable=False)
+    share_type_id = Column(
+        String(36), ForeignKey('share_types.id'), nullable=False)
+    share_group_type = orm.relationship(
+        ShareGroupTypes,
         backref="share_types",
-        foreign_keys=consistency_group_id,
+        foreign_keys=share_group_type_id,
         primaryjoin=('and_('
-                     'ConsistencyGroupShareTypeMapping.consistency_group_id '
-                     '== ConsistencyGroup.id,'
-                     'ConsistencyGroupShareTypeMapping.deleted == "False")')
+                     'ShareGroupTypeShareTypeMapping.share_group_type_id '
+                     '== ShareGroupTypes.id,'
+                     'ShareGroupTypeShareTypeMapping.deleted == "False")')
     )
 
 
-class CGSnapshotMember(BASE, ManilaBase):
-    """Represents the share snapshots in a consistency group snapshot."""
-    __tablename__ = 'cgsnapshot_members'
+class ShareGroupShareTypeMapping(BASE, ManilaBase):
+    """Represents the share types in a share group."""
+    __tablename__ = 'share_group_share_type_mappings'
     id = Column(String(36), primary_key=True)
-    cgsnapshot_id = Column(String(36), ForeignKey('cgsnapshots.id'))
+    deleted = Column(String(36), default='False')
+    share_group_id = Column(
+        String(36), ForeignKey('share_groups.id'), nullable=False)
+    share_type_id = Column(
+        String(36), ForeignKey('share_types.id'), nullable=False)
+    share_group = orm.relationship(
+        ShareGroup,
+        backref="share_types",
+        foreign_keys=share_group_id,
+        primaryjoin=('and_('
+                     'ShareGroupShareTypeMapping.share_group_id '
+                     '== ShareGroup.id,'
+                     'ShareGroupShareTypeMapping.deleted == "False")')
+    )
+
+# TODO(vponomaryov): add 'share_group_snapshot_member_export_locations' model
+# to support mountable share group snapshots and add its relationship to
+# 'share_group_snapshot_members' table.
+
+
+class ShareGroupSnapshotMember(BASE, ManilaBase):
+    """Represents the share snapshots in a share group snapshot."""
+    __tablename__ = 'share_group_snapshot_members'
+    id = Column(String(36), primary_key=True)
+    # TODO(vponomaryov): make 'share_group_snapshot_id' not nullable.
+    share_group_snapshot_id = Column(
+        String(36), ForeignKey('share_group_snapshots.id'))
     share_id = Column(String(36), ForeignKey('shares.id'))
+    # TODO(vponomaryov): make 'share_instance_id' not nullable.
     share_instance_id = Column(String(36), ForeignKey('share_instances.id'))
     size = Column(Integer)
     status = Column(String(255))
     share_proto = Column(String(255))
-    share_type_id = Column(String(36), ForeignKey('share_types.id'),
-                           nullable=True)
     user_id = Column(String(255))
     project_id = Column(String(255))
     deleted = Column(String(36), default='False')
-
-    cgsnapshot = orm.relationship(
-        CGSnapshot,
-        backref="cgsnapshot_members",
-        foreign_keys=cgsnapshot_id,
-        primaryjoin='CGSnapshot.id == CGSnapshotMember.cgsnapshot_id')
+    provider_location = Column(String(255))
+    # TODO(vponomaryov): add relationship to source share instance as it is
+    # done for share snapshot instances.
+    # TODO(vponomaryov): add share group snapshot member export locations
+    # relationship.
+    share_group_snapshot = orm.relationship(
+        ShareGroupSnapshot,
+        backref="share_group_snapshot_members",
+        foreign_keys=share_group_snapshot_id,
+        primaryjoin='ShareGroupSnapshot.id == '
+                    'ShareGroupSnapshotMember.share_group_snapshot_id')
 
 
 def register_models():
@@ -1013,7 +1219,24 @@ def get_access_rules_status(instances):
                 share_access_status):
             share_access_status = instance_access_status
 
-        if share_access_status == constants.STATUS_ERROR:
+        if share_access_status == constants.SHARE_INSTANCE_RULES_ERROR:
             break
 
     return share_access_status
+
+
+def get_aggregated_access_rules_state(instance_mappings):
+    state = None
+    if len(instance_mappings) > 0:
+        order = (constants.ACCESS_STATE_ERROR,
+                 constants.ACCESS_STATE_DENYING,
+                 constants.ACCESS_STATE_QUEUED_TO_DENY,
+                 constants.ACCESS_STATE_QUEUED_TO_APPLY,
+                 constants.ACCESS_STATE_APPLYING,
+                 constants.ACCESS_STATE_ACTIVE)
+
+        sorted_instance_mappings = sorted(
+            instance_mappings, key=lambda x: order.index(x['state']))
+
+        state = sorted_instance_mappings[0].state
+    return state
