@@ -35,7 +35,6 @@ neutron_network_plugin_opts = [
         help="The name of the physical network to determine which net segment "
              "is used. This opt is optional and will only be used for "
              "networks configured with multiple segments.",
-        default=None,
         deprecated_group='DEFAULT'),
 ]
 
@@ -321,10 +320,6 @@ class NeutronSingleNetworkPlugin(NeutronNetworkPlugin):
     def _update_share_network_net_data(self, context, share_network):
         upd = dict()
 
-        if share_network.get('nova_net_id') is not None:
-            raise exception.NetworkBadConfigurationException(
-                "Share network has nova_net_id set.")
-
         if not share_network.get('neutron_net_id') == self.net:
             if share_network.get('neutron_net_id') is not None:
                 raise exception.NetworkBadConfigurationException(
