@@ -65,6 +65,12 @@ standalone_network_plugin_opts = [
         help="IP version of network. Optional."
              "Allowed values are '4' and '6'. Default value is '4'.",
         deprecated_group='DEFAULT'),
+    cfg.IntOpt(
+        'standalone_network_plugin_mtu',
+        default=1500,
+        help="Maximum Transmission Unit (MTU) value of the network. Default "
+             "value is 1500.",
+        deprecated_group='DEFAULT'),
 ]
 
 CONF = cfg.CONF
@@ -77,7 +83,7 @@ class StandaloneNetworkPlugin(network.NetworkBaseAPI):
     This network plugin can be used with any network platform.
     It can serve flat networks as well as segmented.
     It does not require some specific network services in OpenStack like
-    Neutron or Nova.
+    the Neutron plugin.
     The only thing that plugin does is reservation and release of IP addresses
     from some network.
     """
@@ -135,6 +141,7 @@ class StandaloneNetworkPlugin(network.NetworkBaseAPI):
             six.text_type(self.net.network),
             self.gateway,
             six.text_type(self.net.broadcast))
+        self.mtu = self.configuration.standalone_network_plugin_mtu
 
     def _get_network(self):
         """Returns IPNetwork object calculated from gateway and netmask."""
@@ -249,7 +256,9 @@ class StandaloneNetworkPlugin(network.NetworkBaseAPI):
             'network_type': self.network_type,
             'segmentation_id': self.segmentation_id,
             'cidr': six.text_type(self.net.cidr),
+            'gateway': six.text_type(self.gateway),
             'ip_version': self.ip_version,
+            'mtu': self.mtu,
         }
         share_network.update(data)
         if self.label != 'admin':
@@ -281,7 +290,9 @@ class StandaloneNetworkPlugin(network.NetworkBaseAPI):
                 'network_type': share_network['network_type'],
                 'segmentation_id': share_network['segmentation_id'],
                 'cidr': share_network['cidr'],
+                'gateway': share_network['gateway'],
                 'ip_version': share_network['ip_version'],
+                'mtu': share_network['mtu'],
             }
             allocations.append(
                 self.db.network_allocation_create(context, data))
