@@ -29,6 +29,7 @@ import manila.api.middleware.auth
 import manila.common.config
 import manila.compute
 import manila.compute.nova
+import manila.coordination
 import manila.db.api
 import manila.db.base
 import manila.exception
@@ -36,7 +37,6 @@ import manila.network
 import manila.network.linux.interface
 import manila.network.neutron.api
 import manila.network.neutron.neutron_network_plugin
-import manila.network.nova_network_plugin
 import manila.network.standalone_network_plugin
 import manila.quota
 import manila.scheduler.drivers.base
@@ -53,8 +53,9 @@ import manila.share.driver
 import manila.share.drivers.cephfs.cephfs_native
 import manila.share.drivers.container.driver
 import manila.share.drivers.container.storage_helper
-import manila.share.drivers.emc.driver
-import manila.share.drivers.emc.plugins.isilon.isilon
+import manila.share.drivers.dell_emc.driver
+import manila.share.drivers.dell_emc.plugins.isilon.isilon
+import manila.share.drivers.dell_emc.plugins.vmax.connection
 import manila.share.drivers.generic
 import manila.share.drivers.glusterfs
 import manila.share.drivers.glusterfs.common
@@ -68,8 +69,10 @@ import manila.share.drivers.hpe.hpe_3par_driver
 import manila.share.drivers.huawei.huawei_nas
 import manila.share.drivers.ibm.gpfs
 import manila.share.drivers.lvm
+import manila.share.drivers.maprfs.maprfs_native
 import manila.share.drivers.netapp.options
 import manila.share.drivers.nexenta.options
+import manila.share.drivers.qnap.qnap
 import manila.share.drivers.quobyte.quobyte
 import manila.share.drivers.service_instance
 import manila.share.drivers.tegile.tegile
@@ -95,6 +98,7 @@ _global_opt_lists = [
     manila.common.config.debug_opts,
     manila.common.config.global_opts,
     manila.compute._compute_opts,
+    manila.coordination.coordination_opts,
     manila.db.api.db_opts,
     [manila.db.base.db_driver_opt],
     manila.exception.exc_log_opts,
@@ -104,7 +108,6 @@ _global_opt_lists = [
     neutron_bind_network_plugin_opts,
     manila.network.neutron.neutron_network_plugin.
     neutron_single_network_plugin_opts,
-    manila.network.nova_network_plugin.nova_single_network_plugin_opts,
     manila.network.standalone_network_plugin.standalone_network_plugin_opts,
     manila.quota.quota_opts,
     manila.scheduler.drivers.base.scheduler_driver_opts,
@@ -123,7 +126,8 @@ _global_opt_lists = [
     manila.share.drivers.cephfs.cephfs_native.cephfs_native_opts,
     manila.share.drivers.container.driver.container_opts,
     manila.share.drivers.container.storage_helper.lv_opts,
-    manila.share.drivers.emc.driver.EMC_NAS_OPTS,
+    manila.share.drivers.dell_emc.driver.EMC_NAS_OPTS,
+    manila.share.drivers.dell_emc.plugins.vmax.connection.VMAX_OPTS,
     manila.share.drivers.generic.share_opts,
     manila.share.drivers.glusterfs.common.glusterfs_common_opts,
     manila.share.drivers.glusterfs.GlusterfsManilaShare_opts,
@@ -137,16 +141,18 @@ _global_opt_lists = [
     manila.share.drivers.hpe.hpe_3par_driver.HPE3PAR_OPTS,
     manila.share.drivers.huawei.huawei_nas.huawei_opts,
     manila.share.drivers.ibm.gpfs.gpfs_share_opts,
+    manila.share.drivers.maprfs.maprfs_native.maprfs_native_share_opts,
     manila.share.drivers.lvm.share_opts,
     manila.share.drivers.netapp.options.netapp_proxy_opts,
     manila.share.drivers.netapp.options.netapp_connection_opts,
     manila.share.drivers.netapp.options.netapp_transport_opts,
     manila.share.drivers.netapp.options.netapp_basicauth_opts,
     manila.share.drivers.netapp.options.netapp_provisioning_opts,
-    manila.share.drivers.netapp.options.netapp_replication_opts,
+    manila.share.drivers.netapp.options.netapp_data_motion_opts,
     manila.share.drivers.nexenta.options.nexenta_connection_opts,
     manila.share.drivers.nexenta.options.nexenta_dataset_opts,
     manila.share.drivers.nexenta.options.nexenta_nfs_opts,
+    manila.share.drivers.qnap.qnap.qnap_manila_opts,
     manila.share.drivers.quobyte.quobyte.quobyte_manila_share_opts,
     manila.share.drivers.service_instance.common_opts,
     manila.share.drivers.service_instance.no_share_servers_handling_mode_opts,
