@@ -35,17 +35,18 @@ from manila import db
 from manila import exception
 from manila.i18n import _LI, _LW
 from manila.scheduler.filters import base_host as base_host_filter
+from manila.scheduler import utils as scheduler_utils
 from manila.scheduler.weighers import base_host as base_host_weigher
 from manila.share import utils as share_utils
 from manila import utils
 
+
 host_manager_opts = [
     cfg.ListOpt('scheduler_default_filters',
                 default=[
                     'AvailabilityZoneFilter',
                     'CapacityFilter',
                     'CapabilitiesFilter',
-                    'ConsistencyGroupFilter',
                     'DriverFilter',
                     'ShareReplicationFilter',
                 ],
@@ -128,7 +129,9 @@ class HostState(object):
         self.thin_provisioning = False
         self.driver_handles_share_servers = False
         self.snapshot_support = True
-        self.consistency_group_support = False
+        self.create_share_from_snapshot_support = True
+        self.revert_to_snapshot_support = False
+        self.mount_snapshot_support = False
         self.dedupe = False
         self.compression = False
         self.replication_type = None
@@ -292,9 +295,16 @@ class HostState(object):
         if 'snapshot_support' not in pool_cap:
             pool_cap['snapshot_support'] = self.snapshot_support
 
-        if not pool_cap.get('consistency_group_support'):
-            pool_cap['consistency_group_support'] = \
-                self.consistency_group_support
+        if 'create_share_from_snapshot_support' not in pool_cap:
+            pool_cap['create_share_from_snapshot_support'] = (
+                self.create_share_from_snapshot_support)
+
+        if 'revert_to_snapshot_support' not in pool_cap:
+            pool_cap['revert_to_snapshot_support'] = (
+                self.revert_to_snapshot_support)
+
+        if 'mount_snapshot_support' not in pool_cap:
+            pool_cap['mount_snapshot_support'] = self.mount_snapshot_support
 
         if 'dedupe' not in pool_cap:
             pool_cap['dedupe'] = self.dedupe
@@ -316,8 +326,12 @@ class HostState(object):
         self.driver_handles_share_servers = capability.get(
             'driver_handles_share_servers')
         self.snapshot_support = capability.get('snapshot_support')
-        self.consistency_group_support = capability.get(
-            'consistency_group_support', False)
+        self.create_share_from_snapshot_support = capability.get(
+            'create_share_from_snapshot_support')
+        self.revert_to_snapshot_support = capability.get(
+            'revert_to_snapshot_support', False)
+        self.mount_snapshot_support = capability.get(
+            'mount_snapshot_support', False)
         self.updated = capability['timestamp']
         self.replication_type = capability.get('replication_type')
         self.replication_domain = capability.get('replication_domain')
@@ -584,7 +598,6 @@ class HostManager(object):
 
     def get_pools(self, context, filters=None):
         """Returns a dict of all pools on all hosts HostManager knows about."""
-
         self._update_host_state_map(context)
 
         all_pools = []
@@ -629,7 +642,11 @@ class HostManager(object):
         for filter_key, filter_value in filter_dict.items():
             if filter_key not in dict_to_check:
                 return False
-            if not re.match(filter_value, dict_to_check.get(filter_key)):
+            if filter_key == 'capabilities':
+                if not scheduler_utils.capabilities_satisfied(
+                        dict_to_check.get(filter_key), filter_value):
+                    return False
+            elif not re.match(filter_value, dict_to_check.get(filter_key)):
                 return False
 
         return True
