@@ -23,7 +23,6 @@ from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import excutils
 from oslo_utils import importutils
-import six
 
 from manila.common import constants
 from manila import context
@@ -59,7 +58,7 @@ MAPPING = {
 class SchedulerManager(manager.Manager):
     """Chooses a host to create shares."""
 
-    RPC_API_VERSION = '1.6'
+    RPC_API_VERSION = '1.8'
 
     def __init__(self, scheduler_driver=None, service_name=None,
                  *args, **kwargs):
@@ -148,8 +147,9 @@ class SchedulerManager(manager.Manager):
 
     def migrate_share_to_host(
             self, context, share_id, host, force_host_assisted_migration,
-            preserve_metadata, writable, nondisruptive, new_share_network_id,
-            new_share_type_id, request_spec, filter_properties=None):
+            preserve_metadata, writable, nondisruptive, preserve_snapshots,
+            new_share_network_id, new_share_type_id, request_spec,
+            filter_properties=None):
         """Ensure that the host exists and can accept the share."""
 
         share_ref = db.share_get(context, share_id)
@@ -180,7 +180,8 @@ class SchedulerManager(manager.Manager):
                 share_rpcapi.ShareAPI().migration_start(
                     context, share_ref, tgt_host.host,
                     force_host_assisted_migration, preserve_metadata, writable,
-                    nondisruptive, new_share_network_id, new_share_type_id)
+                    nondisruptive, preserve_snapshots, new_share_network_id,
+                    new_share_type_id)
             except Exception as ex:
                 with excutils.save_and_reraise_exception():
                     _migrate_share_set_error(self, context, ex, request_spec)
@@ -189,7 +190,7 @@ class SchedulerManager(manager.Manager):
                                     request_spec):
 
         LOG.error(_LE("Failed to schedule %(method)s: %(ex)s"),
-                  {"method": method, "ex": six.text_type(ex)})
+                  {"method": method, "ex": ex})
 
         properties = request_spec.get('share_properties', {})
 
@@ -211,34 +212,29 @@ class SchedulerManager(manager.Manager):
     def request_service_capabilities(self, context):
         share_rpcapi.ShareAPI().publish_service_capabilities(context)
 
-    def _set_cg_error_state(self, method, context, ex, request_spec):
+    def _set_share_group_error_state(self, method, context, ex, request_spec):
         LOG.warning(_LW("Failed to schedule_%(method)s: %(ex)s"),
                     {"method": method, "ex": ex})
 
-        cg_state = {'status': constants.STATUS_ERROR}
+        share_group_state = {'status': constants.STATUS_ERROR}
 
-        consistency_group_id = request_spec.get('consistency_group_id')
+        share_group_id = request_spec.get('share_group_id')
 
-        if consistency_group_id:
-            db.consistency_group_update(context,
-                                        consistency_group_id,
-                                        cg_state)
+        if share_group_id:
+            db.share_group_update(context, share_group_id, share_group_state)
 
-        # TODO(ameade): add notifications
-
-    def create_consistency_group(self, context, cg_id, request_spec=None,
-                                 filter_properties=None):
+    def create_share_group(self, context, share_group_id, request_spec=None,
+                           filter_properties=None):
         try:
-            self.driver.schedule_create_consistency_group(context, cg_id,
-                                                          request_spec,
-                                                          filter_properties)
+            self.driver.schedule_create_share_group(
+                context, share_group_id, request_spec, filter_properties)
         except exception.NoValidHost as ex:
-            self._set_cg_error_state('create_consistency_group',
-                                     context, ex, request_spec)
+            self._set_share_group_error_state(
+                'create_share_group', context, ex, request_spec)
         except Exception as ex:
             with excutils.save_and_reraise_exception():
-                self._set_cg_error_state('create_consistency_group',
-                                         context, ex, request_spec)
+                self._set_share_group_error_state(
+                    'create_share_group', context, ex, request_spec)
 
     def _set_share_replica_error_state(self, context, method, exc,
                                        request_spec):
