@@ -160,6 +160,11 @@ class GenericShareDriver(driver.ExecuteMixin, driver.ShareDriver):
             ssh_pool.remove(ssh)
             ssh = ssh_pool.create()
             self.ssh_connections[server['instance_id']] = (ssh_pool, ssh)
+
+        # (aovchinnikov): ssh_execute does not behave well when passed
+        # parameters with spaces.
+        wrap = lambda token: "\"" + token + "\""
+        command = [wrap(tkn) if tkn.count(' ') else tkn for tkn in command]
         return processutils.ssh_execute(ssh, ' '.join(command),
                                         check_exit_code=check_exit_code)
 
@@ -215,41 +220,26 @@ class GenericShareDriver(driver.ExecuteMixin, driver.ShareDriver):
     @ensure_server
     def create_share(self, context, share, share_server=None):
         """Creates share."""
+        return self._create_share(
+            context, share,
+            snapshot=None,
+            share_server=share_server,
+        )
+
+    def _create_share(self, context, share, snapshot, share_server=None):
         helper = self._get_helper(share)
         server_details = share_server['backend_details']
-        volume = self._allocate_container(self.admin_context, share)
+        volume = self._allocate_container(
+            self.admin_context, share, snapshot=snapshot)
         volume = self._attach_volume(
-            self.admin_context,
-            share,
-            server_details['instance_id'],
-            volume)
-        self._format_device(server_details, volume)
+            self.admin_context, share, server_details['instance_id'], volume)
+        if not snapshot:
+            self._format_device(server_details, volume)
+
         self._mount_device(share, server_details, volume)
-        location = helper.create_export(
-            server_details,
-            share['name'])
-        export_list = [{
-            "path": location,
-            "is_admin_only": False,
-            "metadata": {
-                # TODO(vponomaryov): remove this fake metadata when
-                # proper appears.
-                "export_location_metadata_example": "example",
-            },
-        }]
-        if server_details.get('admin_ip'):
-            admin_location = location.replace(
-                server_details['public_address'], server_details['admin_ip'])
-            export_list.append({
-                "path": admin_location,
-                "is_admin_only": True,
-                "metadata": {
-                    # TODO(vponomaryov): remove this fake metadata when
-                    #  proper appears.
-                    "export_location_metadata_example": "example",
-                },
-            })
-        return export_list
+        export_locations = helper.create_exports(
+            server_details, share['name'])
+        return export_locations
 
     @utils.retry(exception.ProcessExecutionError, backoff_rate=1)
     def _is_device_file_available(self, server_details, volume):
@@ -325,8 +315,9 @@ class GenericShareDriver(driver.ExecuteMixin, driver.ShareDriver):
                             '%s' % server_details['instance_id'])
         def _mount_device_with_lock():
             mount_path = self._get_mount_path(share)
+            device_path = volume['mountpoint']
             log_data = {
-                'dev': volume['mountpoint'],
+                'dev': device_path,
                 'path': mount_path,
                 'server': server_details['instance_id'],
             }
@@ -335,10 +326,21 @@ class GenericShareDriver(driver.ExecuteMixin, driver.ShareDriver):
                                                volume):
                     LOG.debug("Mounting '%(dev)s' to path '%(path)s' on "
                               "server '%(server)s'.", log_data)
-                    mount_cmd = ['sudo mkdir -p', mount_path, '&&']
-                    mount_cmd.extend(['sudo mount', volume['mountpoint'],
-                                      mount_path])
-                    mount_cmd.extend(['&& sudo chmod 777', mount_path])
+                    mount_cmd = (
+                        'sudo', 'mkdir', '-p', mount_path,
+                        '&&', 'sudo', 'mount', device_path, mount_path,
+                        '&&', 'sudo', 'chmod', '777', mount_path,
+                        '&&', 'sudo', 'umount', mount_path,
+                        # NOTE(vponomaryov): 'tune2fs' is required to make
+                        # filesystem of share created from snapshot have
+                        # unique ID, in case of LVM volumes, by default,
+                        # it will have the same UUID as source volume one.
+                        # 'tune2fs' command can be executed only when device
+                        # is not mounted and also, in current case, it takes
+                        # effect only after it was mounted. Closes #1645751
+                        '&&', 'sudo', 'tune2fs', '-U', 'random', device_path,
+                        '&&', 'sudo', 'mount', device_path, mount_path,
+                    )
                     self._ssh_exec(server_details, mount_cmd)
 
                     # Add mount permanently
@@ -365,8 +367,8 @@ class GenericShareDriver(driver.ExecuteMixin, driver.ShareDriver):
             if self._is_device_mounted(mount_path, server_details):
                 LOG.debug("Unmounting path '%(path)s' on server "
                           "'%(server)s'.", log_data)
-                unmount_cmd = ['sudo umount', mount_path, '&& sudo rmdir',
-                               mount_path]
+                unmount_cmd = ['sudo', 'umount', mount_path, '&&', 'sudo',
+                               'rmdir', mount_path]
                 self._ssh_exec(server_details, unmount_cmd)
                 # Remove mount permanently
                 self._sync_mount_temp_and_perm_files(server_details)
@@ -614,7 +616,6 @@ class GenericShareDriver(driver.ExecuteMixin, driver.ShareDriver):
             share_backend_name=self.backend_name,
             storage_protocol='NFS_CIFS',
             reserved_percentage=self.configuration.reserved_share_percentage,
-            consistency_group_support='pool',
         )
         super(GenericShareDriver, self)._update_share_stats(data)
 
@@ -622,37 +623,11 @@ class GenericShareDriver(driver.ExecuteMixin, driver.ShareDriver):
     def create_share_from_snapshot(self, context, share, snapshot,
                                    share_server=None):
         """Is called to create share from snapshot."""
-        helper = self._get_helper(share)
-        server_details = share_server['backend_details']
-        volume = self._allocate_container(self.admin_context, share, snapshot)
-        volume = self._attach_volume(
-            self.admin_context, share,
-            share_server['backend_details']['instance_id'], volume)
-        self._mount_device(share, share_server['backend_details'], volume)
-        location = helper.create_export(share_server['backend_details'],
-                                        share['name'])
-        export_list = [{
-            "path": location,
-            "is_admin_only": False,
-            "metadata": {
-                # TODO(vponomaryov): remove this fake metadata when
-                # proper appears.
-                "export_location_metadata_example": "example",
-            },
-        }]
-        if server_details.get('admin_ip'):
-            admin_location = location.replace(
-                server_details['public_address'], server_details['admin_ip'])
-            export_list.append({
-                "path": admin_location,
-                "is_admin_only": True,
-                "metadata": {
-                    # TODO(vponomaryov): remove this fake metadata when
-                    #  proper appears.
-                    "export_location_metadata_example": "example",
-                },
-            })
-        return export_list
+        return self._create_share(
+            context, share,
+            snapshot=snapshot,
+            share_server=share_server,
+        )
 
     @ensure_server
     def extend_share(self, share, new_size, share_server=None):
@@ -661,16 +636,18 @@ class GenericShareDriver(driver.ExecuteMixin, driver.ShareDriver):
         helper = self._get_helper(share)
         helper.disable_access_for_maintenance(server_details, share['name'])
         self._unmount_device(share, server_details)
-        self._detach_volume(self.admin_context, share, server_details)
-
         volume = self._get_volume(self.admin_context, share['id'])
-        volume = self._extend_volume(self.admin_context, volume, new_size)
 
-        volume = self._attach_volume(
-            self.admin_context,
-            share,
-            server_details['instance_id'],
-            volume)
+        if int(new_size) > volume['size']:
+            self._detach_volume(self.admin_context, share, server_details)
+            volume = self._extend_volume(self.admin_context, volume, new_size)
+
+            volume = self._attach_volume(
+                self.admin_context,
+                share,
+                server_details['instance_id'],
+                volume)
+
         self._resize_filesystem(server_details, volume)
         self._mount_device(share, server_details, volume)
         helper.restore_access_after_maintenance(server_details,
@@ -759,7 +736,7 @@ class GenericShareDriver(driver.ExecuteMixin, driver.ShareDriver):
         if not self.driver_handles_share_servers:
             share_server = self.service_instance_manager.get_common_server()
         if self._is_share_server_active(context, share_server):
-            helper.remove_export(
+            helper.remove_exports(
                 share_server['backend_details'], share['name'])
             self._unmount_device(share, share_server['backend_details'])
             self._detach_volume(self.admin_context, share,
@@ -845,7 +822,7 @@ class GenericShareDriver(driver.ExecuteMixin, driver.ShareDriver):
                 share_server['backend_details']['instance_id'],
                 volume)
             self._mount_device(share, share_server['backend_details'], volume)
-            helper.create_export(
+            helper.create_exports(
                 share_server['backend_details'], share['name'], recreate=True)
 
     @ensure_server
@@ -1067,177 +1044,3 @@ class GenericShareDriver(driver.ExecuteMixin, driver.ShareDriver):
             raise exception.InvalidShare(reason=msg)
 
         return size
-
-    @ensure_server
-    def create_consistency_group(self, context, cg_dict, share_server=None):
-        """Creates a consistency group.
-
-        Since we are faking the CG object, apart from verifying if the
-        share_server is valid, we do nothing else here.
-        """
-
-        LOG.debug('Created a Consistency Group with ID: %s.', cg_dict['id'])
-
-        msg = _LW('The Generic driver has no means to guarantee consistency '
-                  'group snapshots are actually consistent. This '
-                  'implementation is for reference and testing purposes only.')
-        LOG.warning(msg)
-
-    def delete_consistency_group(self, context, cg_dict, share_server=None):
-        """Deletes a consistency group.
-
-        Since we are faking the CG object we do nothing here.
-        """
-
-        LOG.debug('Deleted the consistency group with ID %s.', cg_dict['id'])
-
-    def _cleanup_cg_share_snapshot(self, context, share_snapshot,
-                                   share_server):
-        """Deletes the snapshot of a share belonging to a consistency group."""
-
-        try:
-            self.delete_snapshot(context, share_snapshot, share_server)
-        except exception.ManilaException:
-            msg = _LE('Could not delete CG Snapshot %(snap)s '
-                      'for share %(share)s.')
-            LOG.error(msg % {
-                'snap': share_snapshot['id'],
-                'share': share_snapshot['share_id'],
-            })
-            raise
-
-    @ensure_server
-    def create_cgsnapshot(self, context, snap_dict, share_server=None):
-        """Creates a consistency group snapshot one or more shares."""
-
-        LOG.debug('Attempting to create a CG snapshot %s.' % snap_dict['id'])
-
-        msg = _LW('The Consistency Group Snapshot being created is '
-                  'not expected to be consistent. This implementation is '
-                  'for reference and testing purposes only.')
-        LOG.warning(msg)
-
-        cg_members = snap_dict.get('cgsnapshot_members', [])
-        if not cg_members:
-            LOG.warning(_LW('No shares in Consistency Group to Create CG '
-                            'snapshot.'))
-        else:
-            share_snapshots = []
-            for member in cg_members:
-                share_snapshot = {
-                    'share_id': member['share_id'],
-                    'id': member['id'],
-                }
-                try:
-                    self.create_snapshot(context, share_snapshot, share_server)
-                    share_snapshots.append(share_snapshot)
-                except exception.ManilaException as e:
-                    msg = _LE('Could not create CG Snapshot. Failed '
-                              'to create share snapshot %(snap)s for '
-                              'share %(share)s.')
-                    LOG.exception(msg % {
-                        'snap': share_snapshot['id'],
-                        'share': share_snapshot['share_id']
-                    })
-
-                    # clean up any share snapshots previously created
-                    LOG.debug('Attempting to clean up snapshots due to '
-                              'failure...')
-                    for share_snapshot in share_snapshots:
-                        self._cleanup_cg_share_snapshot(context,
-                                                        share_snapshot,
-                                                        share_server)
-                    raise e
-
-            LOG.debug('Successfully created CG snapshot %s.' % snap_dict['id'])
-
-        return None, None
-
-    @ensure_server
-    def delete_cgsnapshot(self, context, snap_dict, share_server=None):
-        """Deletes a consistency group snapshot."""
-
-        cg_members = snap_dict.get('cgsnapshot_members', [])
-
-        LOG.debug('Deleting CG snapshot %s.' % snap_dict['id'])
-
-        for member in cg_members:
-            share_snapshot = {
-                'share_id': member['share_id'],
-                'id': member['id'],
-            }
-
-            self._cleanup_cg_share_snapshot(context,
-                                            share_snapshot,
-                                            share_server)
-
-        LOG.debug('Deleted CG snapshot %s.' % snap_dict['id'])
-
-        return None, None
-
-    @ensure_server
-    def create_consistency_group_from_cgsnapshot(self, context, cg_dict,
-                                                 cgsnapshot_dict,
-                                                 share_server=None):
-        """Creates a consistency group from an existing CG snapshot."""
-
-        # Ensure that the consistency group snapshot has members
-        if not cgsnapshot_dict['cgsnapshot_members']:
-            return None, None
-
-        clone_list = self._collate_cg_snapshot_info(cg_dict, cgsnapshot_dict)
-        share_update_list = list()
-
-        LOG.debug('Creating consistency group from CG snapshot %s.',
-                  cgsnapshot_dict['id'])
-
-        for clone in clone_list:
-
-            kwargs = {}
-            if self.driver_handles_share_servers:
-                kwargs['share_server'] = share_server
-            export_location = (
-                self.create_share_from_snapshot(
-                    context,
-                    clone['share'],
-                    clone['snapshot'],
-                    **kwargs))
-
-            share_update_list.append({
-                'id': clone['share']['id'],
-                'export_locations': export_location,
-            })
-
-        return None, share_update_list
-
-    def _collate_cg_snapshot_info(self, cg_dict, cgsnapshot_dict):
-        """Collate the data for a clone of the CG snapshot.
-
-        Given two data structures, a CG snapshot (cgsnapshot_dict) and a new
-        CG to be cloned from the snapshot (cg_dict), match up both
-        structures into a list of dicts (share & snapshot) suitable for use
-        by existing method that clones individual share snapshots.
-        """
-
-        clone_list = list()
-
-        for share in cg_dict['shares']:
-
-            clone_info = {'share': share}
-
-            for cgsnapshot_member in cgsnapshot_dict['cgsnapshot_members']:
-                if (share['source_cgsnapshot_member_id'] ==
-                        cgsnapshot_member['id']):
-                    clone_info['snapshot'] = {
-                        'id': cgsnapshot_member['id'],
-                    }
-                    break
-
-            if len(clone_info) != 2:
-                msg = _("Invalid data supplied for creating consistency "
-                        "group from CG snapshot %s.") % cgsnapshot_dict['id']
-                raise exception.InvalidConsistencyGroup(reason=msg)
-
-            clone_list.append(clone_info)
-
-        return clone_list
