@@ -19,7 +19,9 @@ import string
 import tempfile
 import time
 
+from oslo_config import cfg
 from oslo_log import log
+import oslo_messaging as messaging
 from oslo_serialization import jsonutils
 from oslo_utils import excutils
 from oslo_utils import strutils
@@ -29,36 +31,62 @@ import six
 from manila.common import constants as common_constants
 from manila.data import utils as data_utils
 from manila import exception
-from manila.i18n import _
-from manila.i18n import _LE
-from manila.i18n import _LI
-from manila.i18n import _LW
+from manila.i18n import _, _LE, _LI, _LW
+from manila import rpc
 from manila.share.drivers.huawei import base as driver
 from manila.share.drivers.huawei import constants
 from manila.share.drivers.huawei import huawei_utils
 from manila.share.drivers.huawei.v3 import helper
+from manila.share.drivers.huawei.v3 import replication
+from manila.share.drivers.huawei.v3 import rpcapi as v3_rpcapi
 from manila.share.drivers.huawei.v3 import smartx
 from manila.share import share_types
 from manila.share import utils as share_utils
 from manila import utils
 
+
+CONF = cfg.CONF
+
 LOG = log.getLogger(__name__)
 
 
 class V3StorageConnection(driver.HuaweiBase):
     """Helper class for Huawei OceanStor V3 storage system."""
 
-    def __init__(self, configuration):
+    def __init__(self, configuration, **kwargs):
         super(V3StorageConnection, self).__init__(configuration)
+        self.helper = helper.RestHelper(self.configuration)
+        self.replica_mgr = replication.ReplicaPairManager(self.helper)
+        self.rpc_client = v3_rpcapi.HuaweiV3API()
+        self.private_storage = kwargs.get('private_storage')
         self.qos_support = False
+        self.snapshot_support = False
+        self.replication_support = False
+
+    def _setup_rpc_server(self, endpoints):
+        host = "%s@%s" % (CONF.host, self.configuration.config_group)
+        target = messaging.Target(topic=self.rpc_client.topic, server=host)
+        self.rpc_server = rpc.get_server(target, endpoints)
+        self.rpc_server.start()
 
     def connect(self):
         """Try to connect to V3 server."""
-        if self.configuration:
-            self.helper = helper.RestHelper(self.configuration)
-        else:
-            raise exception.InvalidInput(_("Huawei configuration missing."))
         self.helper.login()
+        self._setup_rpc_server([self.replica_mgr])
+        self._setup_conf()
+
+    def _setup_conf(self):
+        root = self.helper._read_xml()
+
+        snapshot_support = root.findtext('Storage/SnapshotSupport')
+        if snapshot_support:
+            self.snapshot_support = strutils.bool_from_string(
+                snapshot_support, strict=True)
+
+        replication_support = root.findtext('Storage/ReplicationSupport')
+        if replication_support:
+            self.replication_support = strutils.bool_from_string(
+                replication_support, strict=True)
 
     def create_share(self, share, share_server=None):
         """Create a share."""
@@ -107,14 +135,14 @@ class V3StorageConnection(driver.HuaweiBase):
                 if qos_id:
                     self.remove_qos_fs(fs_id, qos_id)
                 self.helper._delete_fs(fs_id)
-            message = (_('Failed to create share %(name)s.'
+            message = (_('Failed to create share %(name)s. '
                          'Reason: %(err)s.')
                        % {'name': share_name,
                           'err': err})
             raise exception.InvalidShare(reason=message)
 
         try:
-            self.helper._create_share(share_name, fs_id, share_proto)
+            self.helper.create_share(share_name, fs_id, share_proto)
         except Exception as err:
             if fs_id is not None:
                 qos_id = self.helper.get_qosid_by_fsid(fs_id)
@@ -248,13 +276,14 @@ class V3StorageConnection(driver.HuaweiBase):
         snap_id = self.helper._create_snapshot(sharefsid,
                                                snapshot_name)
         LOG.info(_LI('Creating snapshot id %s.'), snap_id)
+        return snapshot_name.replace("-", "_")
 
     def delete_snapshot(self, snapshot, share_server=None):
         """Delete a snapshot."""
         LOG.debug("Delete a snapshot.")
         snap_name = snapshot['id']
 
-        sharefsid = self.helper._get_fsid_by_name(snapshot['share_name'])
+        sharefsid = self.helper.get_fsid_by_name(snapshot['share_name'])
 
         if sharefsid is None:
             LOG.warning(_LW('Delete snapshot share id %s fs has been '
@@ -262,7 +291,8 @@ class V3StorageConnection(driver.HuaweiBase):
             return
 
         snapshot_id = self.helper._get_snapshot_id(sharefsid, snap_name)
-        snapshot_flag = self.helper._check_snapshot_id_exist(snapshot_id)
+        snapshot_info = self.helper._get_snapshot_by_id(snapshot_id)
+        snapshot_flag = self.helper._check_snapshot_id_exist(snapshot_info)
 
         if snapshot_flag:
             self.helper._delete_snapshot(snapshot_id)
@@ -280,6 +310,8 @@ class V3StorageConnection(driver.HuaweiBase):
         for pool_name in pool_name_list:
             pool_name = pool_name.strip().strip('\n')
             capacity = self._get_capacity(pool_name, all_pool_info)
+            disk_type = self._get_disk_type(pool_name, all_pool_info)
+
             if capacity:
                 pool = dict(
                     pool_name=pool_name,
@@ -298,7 +330,12 @@ class V3StorageConnection(driver.HuaweiBase):
                     compression=[True, False],
                     huawei_smartcache=[True, False],
                     huawei_smartpartition=[True, False],
+                    huawei_sectorsize=[True, False],
                 )
+
+                if disk_type:
+                    pool['huawei_disk_type'] = disk_type
+
                 stats_dict["pools"].append(pool)
 
         if not stats_dict["pools"]:
@@ -323,7 +360,7 @@ class V3StorageConnection(driver.HuaweiBase):
         if not share:
             LOG.warning(_LW('The share was not found. Share name:%s'),
                         share_name)
-            fsid = self.helper._get_fsid_by_name(share_name)
+            fsid = self.helper.get_fsid_by_name(share_name)
             if fsid:
                 self.helper._delete_fs(fsid)
                 return
@@ -348,7 +385,7 @@ class V3StorageConnection(driver.HuaweiBase):
     def create_share_from_snapshot(self, share, snapshot,
                                    share_server=None):
         """Create a share from snapshot."""
-        share_fs_id = self.helper._get_fsid_by_name(snapshot['share_name'])
+        share_fs_id = self.helper.get_fsid_by_name(snapshot['share_name'])
         if not share_fs_id:
             err_msg = (_("The source filesystem of snapshot %s "
                          "does not exist.")
@@ -358,7 +395,8 @@ class V3StorageConnection(driver.HuaweiBase):
                 name=snapshot['share_name'])
 
         snapshot_id = self.helper._get_snapshot_id(share_fs_id, snapshot['id'])
-        snapshot_flag = self.helper._check_snapshot_id_exist(snapshot_id)
+        snapshot_info = self.helper._get_snapshot_by_id(snapshot_id)
+        snapshot_flag = self.helper._check_snapshot_id_exist(snapshot_info)
         if not snapshot_flag:
             err_msg = (_("Cannot find snapshot %s on array.")
                        % snapshot['snapshot_id'])
@@ -388,6 +426,7 @@ class V3StorageConnection(driver.HuaweiBase):
             "mount_path": new_share_path.replace("\\", "/"),
             "mount_src":
                 tempfile.mkdtemp(prefix=constants.TMP_PATH_DST_PREFIX),
+            "id": snapshot['share_id'],
         }
 
         old_share_path = self._get_location_path(old_share_name,
@@ -399,7 +438,8 @@ class V3StorageConnection(driver.HuaweiBase):
             "mount_src":
                 tempfile.mkdtemp(prefix=constants.TMP_PATH_SRC_PREFIX),
             "snapshot_name": ("share_snapshot_" +
-                              snapshot['id'].replace("-", "_"))
+                              snapshot['id'].replace("-", "_")),
+            "id": snapshot['share_id'],
         }
 
         try:
@@ -412,11 +452,10 @@ class V3StorageConnection(driver.HuaweiBase):
                 try:
                     os.rmdir(item['mount_src'])
                 except Exception as err:
-                    err_msg = (_('Failed to remove temp file. '
-                                 'File path: %(file_path)s. Reason: %(err)s.')
-                               % {'file_path': item['mount_src'],
-                                  'err': six.text_type(err)})
-                    LOG.warning(err_msg)
+                    LOG.warning(_LW('Failed to remove temp file. File path: '
+                                    '%(file_path)s. Reason: %(err)s.'),
+                                {'file_path': item['mount_src'],
+                                 'err': err})
 
         return new_share_path
 
@@ -431,7 +470,7 @@ class V3StorageConnection(driver.HuaweiBase):
                     LOG.error(_LE('Failed to add access to share %(name)s. '
                                   'Reason: %(err)s.'),
                               {'name': old_share['name'],
-                               'err': six.text_type(err)})
+                               'err': err})
 
         new_access = self.get_access(new_share)
         try:
@@ -442,7 +481,7 @@ class V3StorageConnection(driver.HuaweiBase):
                     LOG.error(_LE('Failed to mount old share %(name)s. '
                                   'Reason: %(err)s.'),
                               {'name': old_share['name'],
-                               'err': six.text_type(err)})
+                               'err': err})
 
             try:
                 self.allow_access(new_share, new_access)
@@ -453,7 +492,7 @@ class V3StorageConnection(driver.HuaweiBase):
                     LOG.error(_LE('Failed to mount new share %(name)s. '
                                   'Reason: %(err)s.'),
                               {'name': new_share['name'],
-                               'err': six.text_type(err)})
+                               'err': err})
 
             copied = self.copy_snapshot_data(old_share, new_share)
 
@@ -464,7 +503,7 @@ class V3StorageConnection(driver.HuaweiBase):
                     LOG.warning(_LW('Failed to unmount share %(name)s. '
                                     'Reason: %(err)s.'),
                                 {'name': item['name'],
-                                 'err': six.text_type(err)})
+                                 'err': err})
 
             self.deny_access(new_share, new_access)
 
@@ -534,9 +573,7 @@ class V3StorageConnection(driver.HuaweiBase):
             if copy.get_progress()['total_progress'] == 100:
                 copy_finish = True
         except Exception as err:
-            err_msg = (_("Failed to copy data, reason: %s.")
-                       % six.text_type(err))
-            LOG.error(err_msg)
+            LOG.error(_LE("Failed to copy data, reason: %s."), err)
 
         return copy_finish
 
@@ -599,6 +636,22 @@ class V3StorageConnection(driver.HuaweiBase):
 
         return poolinfo
 
+    def _get_disk_type(self, pool_name, result):
+        """Get disk type of the pool."""
+        pool_info = self.helper._find_pool_info(pool_name, result)
+        if not pool_info:
+            return None
+
+        pool_disk = []
+        for i, x in enumerate(['ssd', 'sas', 'nl_sas']):
+            if pool_info['TIER%dCAPACITY' % i] != '0':
+                pool_disk.append(x)
+
+        if len(pool_disk) > 1:
+            pool_disk = ['mix']
+
+        return pool_disk[0] if pool_disk else None
+
     def _init_filesys_para(self, share, poolinfo, extra_specs):
         """Init basic filesystem parameters."""
         name = share['name']
@@ -630,6 +683,8 @@ class V3StorageConnection(driver.HuaweiBase):
                     ' so dedupe or compression cannot be set.')
                 LOG.error(err_msg)
                 raise exception.InvalidInput(reason=err_msg)
+        if extra_specs['sectorsize']:
+            fileparam['SECTORSIZE'] = extra_specs['sectorsize'] * units.Ki
 
         return fileparam
 
@@ -649,6 +704,10 @@ class V3StorageConnection(driver.HuaweiBase):
             return
 
         access_to = access['access_to']
+        # Huawei array uses * to represent IP addresses of all clients
+        if (share_proto == 'NFS' and access_type == 'ip' and
+                access_to == '0.0.0.0/0'):
+            access_to = '*'
         share = self.helper._get_share_by_name(share_name, share_url_type)
         if not share:
             LOG.warning(_LW('Can not get share %s.'), share_name)
@@ -690,6 +749,10 @@ class V3StorageConnection(driver.HuaweiBase):
                 access_level = constants.ACCESS_NFS_RW
             else:
                 access_level = constants.ACCESS_NFS_RO
+            # Huawei array uses * to represent IP addresses of all clients
+            if access_to == '0.0.0.0/0':
+                access_to = '*'
+
         elif share_proto == 'CIFS':
             if access_type == 'user':
                 if access_level == common_constants.ACCESS_LEVEL_RW:
@@ -701,14 +764,15 @@ class V3StorageConnection(driver.HuaweiBase):
                             ' for CIFS shares.')
                 raise exception.InvalidShareAccess(reason=message)
 
-        share = self.helper._get_share_by_name(share_name, share_url_type)
-        if not share:
-            err_msg = (_("Can not get share ID by share %s.")
+        share_stor = self.helper._get_share_by_name(share_name,
+                                                    share_url_type)
+        if not share_stor:
+            err_msg = (_("Share %s does not exist on the backend.")
                        % share_name)
             LOG.error(err_msg)
-            raise exception.InvalidShareAccess(reason=err_msg)
+            raise exception.ShareResourceNotFound(share_id=share['id'])
 
-        share_id = share['ID']
+        share_id = share_stor['ID']
 
         # Check if access already exists
         access_id = self.helper._get_access_from_share(share_id,
@@ -873,6 +937,51 @@ class V3StorageConnection(driver.HuaweiBase):
         location = self._get_location_path(share_name, share_proto)
         return (share_size, [location])
 
+    def _check_snapshot_valid_for_manage(self, snapshot_info):
+        snapshot_name = snapshot_info['data']['NAME']
+
+        # Check whether the snapshot is normal.
+        if (snapshot_info['data']['HEALTHSTATUS']
+                != constants.STATUS_FSSNAPSHOT_HEALTH):
+            msg = (_("Can't import snapshot %(snapshot)s to Manila. "
+                     "Snapshot status is not normal, snapshot status: "
+                     "%(status)s.")
+                   % {'snapshot': snapshot_name,
+                      'status': snapshot_info['data']['HEALTHSTATUS']})
+            raise exception.ManageInvalidShareSnapshot(
+                reason=msg)
+
+    def manage_existing_snapshot(self, snapshot, driver_options):
+        """Manage existing snapshot."""
+
+        share_proto = snapshot['share']['share_proto']
+        share_url_type = self.helper._get_share_url_type(share_proto)
+        share_storage = self.helper._get_share_by_name(snapshot['share_name'],
+                                                       share_url_type)
+        if not share_storage:
+            err_msg = (_("Failed to import snapshot %(snapshot)s to Manila. "
+                         "Snapshot source share %(share)s doesn't exist "
+                         "on array.")
+                       % {'snapshot': snapshot['provider_location'],
+                          'share': snapshot['share_name']})
+            raise exception.InvalidShare(reason=err_msg)
+        sharefsid = share_storage['FSID']
+
+        provider_location = snapshot.get('provider_location')
+        snapshot_id = sharefsid + "@" + provider_location
+        snapshot_info = self.helper._get_snapshot_by_id(snapshot_id)
+        snapshot_flag = self.helper._check_snapshot_id_exist(snapshot_info)
+        if not snapshot_flag:
+            err_msg = (_("Cannot find snapshot %s on array.")
+                       % snapshot['provider_location'])
+            raise exception.ManageInvalidShareSnapshot(reason=err_msg)
+        else:
+            self._check_snapshot_valid_for_manage(snapshot_info)
+            snapshot_name = ("share_snapshot_"
+                             + snapshot['id'].replace("-", "_"))
+            self.helper._rename_share_snapshot(snapshot_id, snapshot_name)
+        return snapshot_name
+
     def check_retype_change_opts(self, opts, poolinfo, fs):
         change_opts = {
             "partitionid": None,
@@ -1147,6 +1256,12 @@ class V3StorageConnection(driver.HuaweiBase):
             LOG.error(err_msg)
             raise exception.InvalidInput(reason=err_msg)
 
+        if self.snapshot_support and self.replication_support:
+            err_msg = _('Config file invalid. SnapshotSupport and '
+                        'ReplicationSupport can not both be set to True.')
+            LOG.error(err_msg)
+            raise exception.BadConfigurationException(reason=err_msg)
+
     def check_service(self):
         running_status = self.helper._get_cifs_service_status()
         if running_status != constants.STATUS_SERVICE_RUNNING:
@@ -1586,3 +1701,149 @@ class V3StorageConnection(driver.HuaweiBase):
         ip = self._get_share_ip(share_server)
         location = self._get_location_path(share_name, share_proto, ip)
         return [location]
+
+    def create_replica(self, context, replica_list, new_replica,
+                       access_rules, replica_snapshots, share_server=None):
+        """Create a new share, and create a remote replication pair."""
+
+        active_replica = share_utils.get_active_replica(replica_list)
+
+        if (self.private_storage.get(active_replica['share_id'],
+                                     'replica_pair_id')):
+            # for huawei array, only one replication can be created for
+            # each active replica, so if a replica pair id is recorded for
+            # this share, it means active replica already has a replication,
+            # can not create anymore.
+            msg = _('Cannot create more than one replica for share %s.')
+            LOG.error(msg, active_replica['share_id'])
+            raise exception.ReplicationException(
+                reason=msg % active_replica['share_id'])
+
+        # Create a new share
+        new_share_name = new_replica['name']
+        location = self.create_share(new_replica, share_server)
+
+        # create a replication pair.
+        # replication pair only can be created by master node,
+        # so here is a remote call to trigger master node to
+        # start the creating progress.
+        try:
+            replica_pair_id = self.rpc_client.create_replica_pair(
+                context,
+                active_replica['host'],
+                local_share_info=active_replica,
+                remote_device_wwn=self.helper.get_array_wwn(),
+                remote_fs_id=self.helper.get_fsid_by_name(new_share_name)
+            )
+        except Exception:
+            LOG.exception(_LE('Failed to create a replication pair '
+                              'with host %s.'),
+                          active_replica['host'])
+            raise
+
+        self.private_storage.update(new_replica['share_id'],
+                                    {'replica_pair_id': replica_pair_id})
+
+        # Get the state of the new created replica
+        replica_state = self.replica_mgr.get_replica_state(replica_pair_id)
+        replica_ref = {
+            'export_locations': [location],
+            'replica_state': replica_state,
+            'access_rules_status': common_constants.STATUS_ACTIVE,
+        }
+
+        return replica_ref
+
+    def update_replica_state(self, context, replica_list, replica,
+                             access_rules, replica_snapshots,
+                             share_server=None):
+        replica_pair_id = self.private_storage.get(replica['share_id'],
+                                                   'replica_pair_id')
+        if replica_pair_id is None:
+            msg = _LE("No replication pair ID recorded for share %s.")
+            LOG.error(msg, replica['share_id'])
+            return common_constants.STATUS_ERROR
+
+        self.replica_mgr.update_replication_pair_state(replica_pair_id)
+        return self.replica_mgr.get_replica_state(replica_pair_id)
+
+    def promote_replica(self, context, replica_list, replica, access_rules,
+                        share_server=None):
+        replica_pair_id = self.private_storage.get(replica['share_id'],
+                                                   'replica_pair_id')
+        if replica_pair_id is None:
+            msg = _("No replication pair ID recorded for share %s.")
+            LOG.error(msg, replica['share_id'])
+            raise exception.ReplicationException(
+                reason=msg % replica['share_id'])
+
+        try:
+            self.replica_mgr.switch_over(replica_pair_id)
+        except Exception:
+            LOG.exception(_LE('Failed to promote replica %s.'),
+                          replica['id'])
+            raise
+
+        updated_new_active_access = True
+        cleared_old_active_access = True
+
+        try:
+            self.update_access(replica, access_rules, [], [], share_server)
+        except Exception:
+            LOG.warning(_LW('Failed to set access rules to '
+                            'new active replica %s.'),
+                        replica['id'])
+            updated_new_active_access = False
+
+        old_active_replica = share_utils.get_active_replica(replica_list)
+
+        try:
+            self.clear_access(old_active_replica, share_server)
+        except Exception:
+            LOG.warning(_LW("Failed to clear access rules from "
+                            "old active replica %s."),
+                        old_active_replica['id'])
+            cleared_old_active_access = False
+
+        new_active_update = {
+            'id': replica['id'],
+            'replica_state': common_constants.REPLICA_STATE_ACTIVE,
+        }
+        new_active_update['access_rules_status'] = (
+            common_constants.STATUS_ACTIVE
+            if updated_new_active_access
+            else common_constants.SHARE_INSTANCE_RULES_SYNCING)
+
+        # get replica state for new secondary after switch over
+        replica_state = self.replica_mgr.get_replica_state(replica_pair_id)
+
+        old_active_update = {
+            'id': old_active_replica['id'],
+            'replica_state': replica_state,
+        }
+        old_active_update['access_rules_status'] = (
+            common_constants.SHARE_INSTANCE_RULES_SYNCING
+            if cleared_old_active_access
+            else common_constants.STATUS_ACTIVE)
+
+        return [new_active_update, old_active_update]
+
+    def delete_replica(self, context, replica_list, replica_snapshots,
+                       replica, share_server=None):
+        replica_pair_id = self.private_storage.get(replica['share_id'],
+                                                   'replica_pair_id')
+        if replica_pair_id is None:
+            msg = _LW("No replication pair ID recorded for share %(share)s. "
+                      "Continue to delete replica %(replica)s.")
+            LOG.warning(msg, {'share': replica['share_id'],
+                              'replica': replica['id']})
+        else:
+            self.replica_mgr.delete_replication_pair(replica_pair_id)
+            self.private_storage.delete(replica['share_id'])
+
+        try:
+            self.delete_share(replica, share_server)
+        except Exception:
+            LOG.exception(_LE('Failed to delete replica %s.'),
+                          replica['id'])
+            raise
