@@ -53,7 +53,10 @@ class NetAppCmodeMultiSvmShareDriver(driver.ShareDriver):
                                                        snapshot, **kwargs)
 
     def create_snapshot(self, context, snapshot, **kwargs):
-        self.library.create_snapshot(context, snapshot, **kwargs)
+        return self.library.create_snapshot(context, snapshot, **kwargs)
+
+    def revert_to_snapshot(self, context, snapshot, access_rules, **kwargs):
+        return self.library.revert_to_snapshot(context, snapshot, **kwargs)
 
     def delete_share(self, context, share, **kwargs):
         self.library.delete_share(context, share, **kwargs)
@@ -95,16 +98,30 @@ class NetAppCmodeMultiSvmShareDriver(driver.ShareDriver):
     def unmanage(self, share):
         raise NotImplementedError
 
+    def manage_existing_snapshot(self, snapshot, driver_options):
+        raise NotImplementedError
+
+    def unmanage_snapshot(self, snapshot):
+        raise NotImplementedError
+
     def update_access(self, context, share, access_rules, add_rules,
                       delete_rules, **kwargs):
         self.library.update_access(context, share, access_rules, add_rules,
                                    delete_rules, **kwargs)
 
     def _update_share_stats(self, data=None):
-        data = self.library.get_share_stats()
+        data = self.library.get_share_stats(
+            filter_function=self.get_filter_function(),
+            goodness_function=self.get_goodness_function())
         super(NetAppCmodeMultiSvmShareDriver, self)._update_share_stats(
             data=data)
 
+    def get_default_filter_function(self):
+        return self.library.get_default_filter_function()
+
+    def get_default_goodness_function(self):
+        return self.library.get_default_goodness_function()
+
     def get_share_server_pools(self, share_server):
         return self.library.get_share_server_pools(share_server)
 
@@ -150,3 +167,59 @@ class NetAppCmodeMultiSvmShareDriver(driver.ShareDriver):
                                    share_replica, replica_snapshots,
                                    replica_snapshot, share_server=None):
         raise NotImplementedError()
+
+    def revert_to_replicated_snapshot(self, context, active_replica,
+                                      replica_list, active_replica_snapshot,
+                                      replica_snapshots, access_rules,
+                                      share_server=None):
+        raise NotImplementedError()
+
+    def migration_check_compatibility(self, context, source_share,
+                                      destination_share, share_server=None,
+                                      destination_share_server=None):
+        return self.library.migration_check_compatibility(
+            context, source_share, destination_share,
+            share_server=share_server,
+            destination_share_server=destination_share_server)
+
+    def migration_start(self, context, source_share, destination_share,
+                        source_snapshots, snapshot_mappings,
+                        share_server=None, destination_share_server=None):
+        return self.library.migration_start(
+            context, source_share, destination_share,
+            source_snapshots, snapshot_mappings,
+            share_server=share_server,
+            destination_share_server=destination_share_server)
+
+    def migration_continue(self, context, source_share, destination_share,
+                           source_snapshots, snapshot_mappings,
+                           share_server=None, destination_share_server=None):
+        return self.library.migration_continue(
+            context, source_share, destination_share,
+            source_snapshots, snapshot_mappings, share_server=share_server,
+            destination_share_server=destination_share_server)
+
+    def migration_get_progress(self, context, source_share,
+                               destination_share, source_snapshots,
+                               snapshot_mappings, share_server=None,
+                               destination_share_server=None):
+        return self.library.migration_get_progress(
+            context, source_share, destination_share,
+            source_snapshots, snapshot_mappings, share_server=share_server,
+            destination_share_server=destination_share_server)
+
+    def migration_cancel(self, context, source_share, destination_share,
+                         source_snapshots, snapshot_mappings,
+                         share_server=None, destination_share_server=None):
+        return self.library.migration_cancel(
+            context, source_share, destination_share,
+            source_snapshots, snapshot_mappings, share_server=share_server,
+            destination_share_server=destination_share_server)
+
+    def migration_complete(self, context, source_share, destination_share,
+                           source_snapshots, snapshot_mappings,
+                           share_server=None, destination_share_server=None):
+        return self.library.migration_complete(
+            context, source_share, destination_share,
+            source_snapshots, snapshot_mappings, share_server=share_server,
+            destination_share_server=destination_share_server)
