@@ -20,6 +20,7 @@ single-SVM or multi-SVM functionality needed by the cDOT Manila drivers.
 """
 
 import copy
+import json
 import math
 import socket
 
@@ -36,12 +37,14 @@ from manila.i18n import _, _LE, _LI, _LW
 from manila.share.drivers.netapp.dataontap.client import api as netapp_api
 from manila.share.drivers.netapp.dataontap.client import client_cmode
 from manila.share.drivers.netapp.dataontap.cluster_mode import data_motion
+from manila.share.drivers.netapp.dataontap.cluster_mode import performance
 from manila.share.drivers.netapp.dataontap.protocols import cifs_cmode
 from manila.share.drivers.netapp.dataontap.protocols import nfs_cmode
 from manila.share.drivers.netapp import options as na_opts
 from manila.share.drivers.netapp import utils as na_utils
 from manila.share import share_types
 from manila.share import utils as share_utils
+from manila import utils as manila_utils
 
 LOG = log.getLogger(__name__)
 CONF = cfg.CONF
@@ -55,6 +58,9 @@ class NetAppCmodeFileStorageLibrary(object):
 
     SUPPORTED_PROTOCOLS = ('nfs', 'cifs')
 
+    DEFAULT_FILTER_FUNCTION = 'capabilities.utilization < 70'
+    DEFAULT_GOODNESS_FUNCTION = '100 - capabilities.utilization'
+
     # Maps NetApp qualified extra specs keys to corresponding backend API
     # client library argument keywords.  When we expose more backend
     # capabilities here, we will add them to this map.
@@ -91,7 +97,7 @@ class NetAppCmodeFileStorageLibrary(object):
         self.configuration.append_config_values(
             na_opts.netapp_provisioning_opts)
         self.configuration.append_config_values(
-            na_opts.netapp_replication_opts)
+            na_opts.netapp_data_motion_opts)
 
         self._licenses = []
         self._client = None
@@ -110,6 +116,9 @@ class NetAppCmodeFileStorageLibrary(object):
         self._client = self._get_api_client()
         self._have_cluster_creds = self._client.check_for_cluster_credentials()
 
+        # Performance monitoring library
+        self._perf_library = performance.PerformanceLibrary(self._client)
+
     @na_utils.trace
     def check_for_setup_error(self):
         self._licenses = self._get_licenses()
@@ -214,8 +223,16 @@ class NetAppCmodeFileStorageLibrary(object):
         else:
             return None
 
+    def get_default_filter_function(self):
+        """Get the default filter_function string."""
+        return self.DEFAULT_FILTER_FUNCTION
+
+    def get_default_goodness_function(self):
+        """Get the default goodness_function string."""
+        return self.DEFAULT_GOODNESS_FUNCTION
+
     @na_utils.trace
-    def get_share_stats(self):
+    def get_share_stats(self, filter_function=None, goodness_function=None):
         """Retrieve stats info from Data ONTAP backend."""
 
         data = {
@@ -225,8 +242,8 @@ class NetAppCmodeFileStorageLibrary(object):
             'driver_version': '1.0',
             'netapp_storage_family': 'ontap_cluster',
             'storage_protocol': 'NFS_CIFS',
-            'consistency_group_support': 'host',
-            'pools': self._get_pools(),
+            'pools': self._get_pools(filter_function=filter_function,
+                                     goodness_function=goodness_function),
         }
 
         if (self.configuration.replication_domain and
@@ -249,28 +266,35 @@ class NetAppCmodeFileStorageLibrary(object):
         return self._get_pools()
 
     @na_utils.trace
-    def _get_pools(self):
+    def _get_pools(self, filter_function=None, goodness_function=None):
         """Retrieve list of pools available to this backend."""
 
         pools = []
         aggr_space = self._get_aggregate_space()
+        aggregates = aggr_space.keys()
 
-        for aggr_name in sorted(aggr_space.keys()):
+        # Get up-to-date node utilization metrics just once.
+        if self._have_cluster_creds:
+            self._perf_library.update_performance_cache({}, self._ssc_stats)
+
+        for aggr_name in sorted(aggregates):
 
             reserved_percentage = self.configuration.reserved_share_percentage
 
             total_capacity_gb = na_utils.round_down(float(
-                aggr_space[aggr_name].get('total', 0)) / units.Gi, '0.01')
+                aggr_space[aggr_name].get('total', 0)) / units.Gi)
             free_capacity_gb = na_utils.round_down(float(
-                aggr_space[aggr_name].get('available', 0)) / units.Gi, '0.01')
+                aggr_space[aggr_name].get('available', 0)) / units.Gi)
             allocated_capacity_gb = na_utils.round_down(float(
-                aggr_space[aggr_name].get('used', 0)) / units.Gi, '0.01')
+                aggr_space[aggr_name].get('used', 0)) / units.Gi)
 
             if total_capacity_gb == 0.0:
                 total_capacity_gb = 'unknown'
 
             pool = {
                 'pool_name': aggr_name,
+                'filter_function': filter_function,
+                'goodness_function': goodness_function,
                 'total_capacity_gb': total_capacity_gb,
                 'free_capacity_gb': free_capacity_gb,
                 'allocated_capacity_gb': allocated_capacity_gb,
@@ -279,7 +303,9 @@ class NetAppCmodeFileStorageLibrary(object):
                 'dedupe': [True, False],
                 'compression': [True, False],
                 'thin_provisioning': [True, False],
-                'netapp_aggregate': aggr_name,
+                'snapshot_support': True,
+                'create_share_from_snapshot_support': True,
+                'revert_to_snapshot_support': True,
             }
 
             # Add storage service catalog data.
@@ -287,6 +313,11 @@ class NetAppCmodeFileStorageLibrary(object):
             if pool_ssc_stats:
                 pool.update(pool_ssc_stats)
 
+            # Add utilization info, or nominal value if not available.
+            utilization = self._perf_library.get_node_utilization_for_pool(
+                aggr_name)
+            pool['utilization'] = na_utils.round_down(utilization)
+
             pools.append(pool)
 
         return pools
@@ -294,24 +325,49 @@ class NetAppCmodeFileStorageLibrary(object):
     @na_utils.trace
     def _handle_ems_logging(self):
         """Build and send an EMS log message."""
-        self._client.send_ems_log_message(self._build_ems_log_message())
+        self._client.send_ems_log_message(self._build_ems_log_message_0())
+        self._client.send_ems_log_message(self._build_ems_log_message_1())
 
-    @na_utils.trace
-    def _build_ems_log_message(self):
-        """Construct EMS Autosupport log message."""
+    def _build_base_ems_log_message(self):
+        """Construct EMS Autosupport log message common to all events."""
 
         ems_log = {
-            'computer-name': socket.getfqdn() or 'Manila_node',
-            'event-id': '0',
+            'computer-name': socket.gethostname() or 'Manila_node',
             'event-source': 'Manila driver %s' % self.driver_name,
             'app-version': self._app_version,
             'category': 'provisioning',
-            'event-description': 'OpenStack Manila connected to cluster node',
-            'log-level': '6',
+            'log-level': '5',
             'auto-support': 'false',
         }
         return ems_log
 
+    @na_utils.trace
+    def _build_ems_log_message_0(self):
+        """Construct EMS Autosupport log message with deployment info."""
+
+        ems_log = self._build_base_ems_log_message()
+        ems_log.update({
+            'event-id': '0',
+            'event-description': 'OpenStack Manila connected to cluster node',
+        })
+        return ems_log
+
+    @na_utils.trace
+    def _build_ems_log_message_1(self):
+        """Construct EMS Autosupport log message with storage pool info."""
+
+        message = self._get_ems_pool_info()
+
+        ems_log = self._build_base_ems_log_message()
+        ems_log.update({
+            'event-id': '1',
+            'event-description': json.dumps(message),
+        })
+        return ems_log
+
+    def _get_ems_pool_info(self):
+        raise NotImplementedError()
+
     @na_utils.trace
     def _handle_housekeeping_tasks(self):
         """Handle various cleanup activities."""
@@ -622,7 +678,8 @@ class NetAppCmodeFileStorageLibrary(object):
         vserver_client.delete_volume(share_name)
 
     @na_utils.trace
-    def _create_export(self, share, share_server, vserver, vserver_client):
+    def _create_export(self, share, share_server, vserver, vserver_client,
+                       clear_current_export_policy=True):
         """Creates NAS storage."""
         helper = self._get_helper(share)
         helper.set_client(vserver_client)
@@ -642,7 +699,9 @@ class NetAppCmodeFileStorageLibrary(object):
             share, share_server, interfaces)
 
         # Create the share and get a callback for generating export locations
-        callback = helper.create_share(share, share_name)
+        callback = helper.create_share(
+            share, share_name,
+            clear_current_export_policy=clear_current_export_policy)
 
         # Generate export locations using addresses, metadata and callback
         export_locations = [
@@ -735,6 +794,14 @@ class NetAppCmodeFileStorageLibrary(object):
         vserver_client.create_snapshot(share_name, snapshot_name)
         return {'provider_location': snapshot_name}
 
+    def revert_to_snapshot(self, context, snapshot, share_server=None):
+        """Reverts a share (in place) to the specified snapshot."""
+        vserver, vserver_client = self._get_vserver(share_server=share_server)
+        share_name = self._get_backend_share_name(snapshot['share_id'])
+        snapshot_name = self._get_backend_snapshot_name(snapshot['id'])
+        LOG.debug('Restoring snapshot %s', snapshot_name)
+        vserver_client.restore_snapshot(share_name, snapshot_name)
+
     @na_utils.trace
     def delete_snapshot(self, context, snapshot, share_server=None,
                         snapshot_name=None):
@@ -1013,7 +1080,7 @@ class NetAppCmodeFileStorageLibrary(object):
             else:
                 msg = _("Invalid data supplied for creating consistency group "
                         "from CG snapshot %s.") % cgsnapshot_dict['id']
-                raise exception.InvalidConsistencyGroup(reason=msg)
+                raise exception.InvalidShareGroup(reason=msg)
 
             clone_list.append(clone_info)
 
@@ -1160,7 +1227,9 @@ class NetAppCmodeFileStorageLibrary(object):
         # Initialize entries for each aggregate.
         for aggregate_name in aggregate_names:
             if aggregate_name not in ssc_stats:
-                ssc_stats[aggregate_name] = {}
+                ssc_stats[aggregate_name] = {
+                    'netapp_aggregate': aggregate_name,
+                }
 
         if aggregate_names:
             self._update_ssc_aggr_info(aggregate_names, ssc_stats)
@@ -1198,6 +1267,11 @@ class NetAppCmodeFileStorageLibrary(object):
             if r['replica_state'] == constants.REPLICA_STATE_ACTIVE:
                 return r
 
+    def _find_nonactive_replicas(self, replica_list):
+        """Returns a list of all except the active replica."""
+        return [replica for replica in replica_list
+                if replica['replica_state'] != constants.REPLICA_STATE_ACTIVE]
+
     def create_replica(self, context, replica_list, new_replica,
                        access_rules, share_snapshots, share_server=None):
         """Creates the new replica on this backend and sets up SnapMirror."""
@@ -1265,6 +1339,12 @@ class NetAppCmodeFileStorageLibrary(object):
             msg_args = {'share_name': share_name, 'vserver': vserver}
             raise exception.ShareResourceNotFound(msg % msg_args)
 
+        # NOTE(cknight): The SnapMirror may have been intentionally broken by
+        # a revert-to-snapshot operation, in which case this method should not
+        # attempt to change anything.
+        if active_replica['status'] == constants.STATUS_REVERTING:
+            return None
+
         dm_session = data_motion.DataMotionSession()
         try:
             snapmirrors = dm_session.get_snapmirrors(active_replica, replica)
@@ -1414,7 +1494,7 @@ class NetAppCmodeFileStorageLibrary(object):
             helper.update_access(replica, share_name, access_rules)
         except Exception:
             new_active_replica['access_rules_status'] = (
-                constants.STATUS_OUT_OF_SYNC)
+                constants.SHARE_INSTANCE_RULES_SYNCING)
         else:
             new_active_replica['access_rules_status'] = constants.STATUS_ACTIVE
 
@@ -1563,3 +1643,319 @@ class NetAppCmodeFileStorageLibrary(object):
         except netapp_api.NaApiError as e:
             if e.code != netapp_api.EOBJECTNOTFOUND:
                 raise
+
+    def revert_to_replicated_snapshot(self, context, active_replica,
+                                      replica_list, active_replica_snapshot,
+                                      replica_snapshots, share_server=None):
+        """Reverts a replicated share (in place) to the specified snapshot."""
+        vserver, vserver_client = self._get_vserver(share_server=share_server)
+        share_name = self._get_backend_share_name(
+            active_replica_snapshot['share_id'])
+        snapshot_name = self._get_backend_snapshot_name(
+            active_replica_snapshot['id'])
+        LOG.debug('Restoring snapshot %s', snapshot_name)
+
+        dm_session = data_motion.DataMotionSession()
+        non_active_replica_list = self._find_nonactive_replicas(replica_list)
+
+        # Ensure source snapshot exists
+        vserver_client.get_snapshot(share_name, snapshot_name)
+
+        # Break all mirrors
+        for replica in non_active_replica_list:
+            try:
+                dm_session.break_snapmirror(
+                    active_replica, replica, mount=False)
+            except netapp_api.NaApiError as e:
+                if e.code != netapp_api.EOBJECTNOTFOUND:
+                    raise
+
+        # Delete source SnapMirror snapshots that will prevent a snap restore
+        snapmirror_snapshot_names = vserver_client.list_snapmirror_snapshots(
+            share_name)
+        for snapmirror_snapshot_name in snapmirror_snapshot_names:
+            vserver_client.delete_snapshot(
+                share_name, snapmirror_snapshot_name, ignore_owners=True)
+
+        # Restore source snapshot of interest
+        vserver_client.restore_snapshot(share_name, snapshot_name)
+
+        # Reestablish mirrors
+        for replica in non_active_replica_list:
+            try:
+                dm_session.resync_snapmirror(active_replica, replica)
+            except netapp_api.NaApiError as e:
+                if e.code != netapp_api.EOBJECTNOTFOUND:
+                    raise
+
+    def _check_destination_vserver_for_vol_move(self, source_share,
+                                                source_vserver,
+                                                dest_share_server):
+        try:
+            destination_vserver, __ = self._get_vserver(
+                share_server=dest_share_server)
+        except exception.InvalidParameterValue:
+            destination_vserver = None
+
+        if source_vserver != destination_vserver:
+            msg = _("Cannot migrate %(shr)s efficiently from source "
+                    "VServer %(src)s to destination VServer %(dest)s.")
+            msg_args = {
+                'shr': source_share['id'],
+                'src': source_vserver,
+                'dest': destination_vserver,
+            }
+            raise exception.NetAppException(msg % msg_args)
+
+    def migration_check_compatibility(self, context, source_share,
+                                      destination_share, share_server=None,
+                                      destination_share_server=None):
+        """Checks compatibility between self.host and destination host."""
+        # We need cluster creds to perform an intra-cluster data motion
+        compatible = False
+        destination_host = destination_share['host']
+
+        if self._have_cluster_creds:
+            try:
+                backend = share_utils.extract_host(
+                    destination_host, level='backend_name')
+                data_motion.get_backend_configuration(backend)
+
+                source_vserver, __ = self._get_vserver(
+                    share_server=share_server)
+                share_volume = self._get_backend_share_name(
+                    source_share['id'])
+                destination_aggregate = share_utils.extract_host(
+                    destination_host, level='pool')
+
+                self._check_destination_vserver_for_vol_move(
+                    source_share, source_vserver, destination_share_server)
+
+                self._client.check_volume_move(
+                    share_volume, source_vserver, destination_aggregate)
+
+            except Exception:
+                msg = _LE("Cannot migrate share %(shr)s efficiently between "
+                          "%(src)s and %(dest)s.")
+                msg_args = {
+                    'shr': source_share['id'],
+                    'src': source_share['host'],
+                    'dest': destination_host,
+                }
+                LOG.exception(msg, msg_args)
+            else:
+                compatible = True
+        else:
+            msg = _LW("Cluster credentials have not been configured "
+                      "with this share driver. Cannot perform volume move "
+                      "operations.")
+            LOG.warning(msg)
+
+        compatibility = {
+            'compatible': compatible,
+            'writable': compatible,
+            'nondisruptive': compatible,
+            'preserve_metadata': compatible,
+            'preserve_snapshots': compatible,
+        }
+
+        return compatibility
+
+    def migration_start(self, context, source_share, destination_share,
+                        source_snapshots, snapshot_mappings,
+                        share_server=None, destination_share_server=None):
+        """Begins data motion from source_share to destination_share."""
+        # Intra-cluster migration
+        vserver, vserver_client = self._get_vserver(share_server=share_server)
+        share_volume = self._get_backend_share_name(source_share['id'])
+        destination_aggregate = share_utils.extract_host(
+            destination_share['host'], level='pool')
+
+        self._client.start_volume_move(
+            share_volume, vserver, destination_aggregate)
+
+        msg = _LI("Began volume move operation of share %(shr)s from %(src)s "
+                  "to %(dest)s.")
+        msg_args = {
+            'shr': source_share['id'],
+            'src': source_share['host'],
+            'dest': destination_share['host'],
+        }
+        LOG.info(msg, msg_args)
+
+    def _get_volume_move_status(self, source_share, share_server):
+        vserver, vserver_client = self._get_vserver(share_server=share_server)
+        share_volume = self._get_backend_share_name(source_share['id'])
+        status = self._client.get_volume_move_status(share_volume, vserver)
+        return status
+
+    def migration_continue(self, context, source_share, destination_share,
+                           source_snapshots, snapshot_mappings,
+                           share_server=None, destination_share_server=None):
+        """Check progress of migration, try to repair data motion errors."""
+        status = self._get_volume_move_status(source_share, share_server)
+        completed_phases = (
+            'cutover_hard_deferred', 'cutover_soft_deferred', 'completed')
+
+        move_phase = status['phase'].lower()
+        if move_phase == 'failed':
+            msg_args = {
+                'shr': source_share['id'],
+                'reason': status['details'],
+            }
+            msg = _("Volume move operation for share %(shr)s failed. Reason: "
+                    "%(reason)s") % msg_args
+            LOG.exception(msg)
+            raise exception.NetAppException(msg)
+        elif move_phase in completed_phases:
+            return True
+
+        return False
+
+    def migration_get_progress(self, context, source_share,
+                               destination_share, source_snapshots,
+                               snapshot_mappings, share_server=None,
+                               destination_share_server=None):
+        """Return detailed progress of the migration in progress."""
+        status = self._get_volume_move_status(source_share, share_server)
+
+        # NOTE (gouthamr): If the volume move is waiting for a manual
+        # intervention to cut-over, the copy is done with respect to the
+        # user. Volume move copies the rest of the data before cut-over anyway.
+        if status['phase'] in ('cutover_hard_deferred',
+                               'cutover_soft_deferred'):
+            status['percent-complete'] = 100
+
+        msg = _LI("Volume move status for share %(share)s: (State) %(state)s. "
+                  "(Phase) %(phase)s. Details: %(details)s")
+        msg_args = {
+            'state': status['state'],
+            'details': status['details'],
+            'share': source_share['id'],
+            'phase': status['phase'],
+        }
+        LOG.info(msg, msg_args)
+
+        return {
+            'total_progress': status['percent-complete'] or 0,
+            'state': status['state'],
+            'estimated_completion_time': status['estimated-completion-time'],
+            'phase': status['phase'],
+            'details': status['details'],
+        }
+
+    def migration_cancel(self, context, source_share, destination_share,
+                         source_snapshots, snapshot_mappings,
+                         share_server=None, destination_share_server=None):
+        """Abort an ongoing migration."""
+        vserver, vserver_client = self._get_vserver(share_server=share_server)
+        share_volume = self._get_backend_share_name(source_share['id'])
+
+        try:
+            self._get_volume_move_status(source_share, share_server)
+        except exception.NetAppException:
+            LOG.exception(_LE("Could not get volume move status."))
+            return
+
+        self._client.abort_volume_move(share_volume, vserver)
+
+        msg = _LI("Share volume move operation for share %(shr)s from host "
+                  "%(src)s to %(dest)s was successfully aborted.")
+        msg_args = {
+            'shr': source_share['id'],
+            'src': source_share['host'],
+            'dest': destination_share['host'],
+        }
+        LOG.info(msg, msg_args)
+
+    def migration_complete(self, context, source_share, destination_share,
+                           source_snapshots, snapshot_mappings,
+                           share_server=None, destination_share_server=None):
+        """Initiate the cutover to destination share after move is complete."""
+        vserver, vserver_client = self._get_vserver(share_server=share_server)
+        share_volume = self._get_backend_share_name(source_share['id'])
+
+        status = self._get_volume_move_status(source_share, share_server)
+
+        move_phase = status['phase'].lower()
+        if move_phase == 'completed':
+            LOG.debug("Volume move operation was already successfully "
+                      "completed for share %(shr)s.",
+                      {'shr': source_share['id']})
+        elif move_phase in ('cutover_hard_deferred', 'cutover_soft_deferred'):
+            self._client.trigger_volume_move_cutover(share_volume, vserver)
+            self._wait_for_cutover_completion(
+                source_share, share_server)
+        else:
+            msg_args = {
+                'shr': source_share['id'],
+                'status': status['state'],
+                'phase': status['phase'],
+                'details': status['details'],
+            }
+            msg = _("Cannot complete volume move operation for share %(shr)s. "
+                    "Current volume move status: %(status)s, phase: "
+                    "%(phase)s. Details: %(details)s") % msg_args
+            LOG.exception(msg)
+            raise exception.NetAppException(msg)
+
+        new_share_volume_name = self._get_backend_share_name(
+            destination_share['id'])
+        vserver_client.set_volume_name(share_volume, new_share_volume_name)
+
+        msg = _LI("Volume move operation for share %(shr)s has completed "
+                  "successfully. Share has been moved from %(src)s to "
+                  "%(dest)s.")
+        msg_args = {
+            'shr': source_share['id'],
+            'src': source_share['host'],
+            'dest': destination_share['host'],
+        }
+        LOG.info(msg, msg_args)
+
+        # NOTE(gouthamr): For nondisruptive migration, current export
+        # policy will not be cleared, the export policy will be renamed to
+        # match the name of the share.
+        export_locations = self._create_export(
+            destination_share, share_server, vserver, vserver_client,
+            clear_current_export_policy=False)
+        src_snaps_dict = {s['id']: s for s in source_snapshots}
+        snapshot_updates = {}
+
+        for source_snap_id, destination_snap in snapshot_mappings.items():
+            p_location = src_snaps_dict[source_snap_id]['provider_location']
+
+            snapshot_updates.update(
+                {destination_snap['id']: {'provider_location': p_location}})
+
+        return {
+            'export_locations': export_locations,
+            'snapshot_updates': snapshot_updates,
+        }
+
+    def _wait_for_cutover_completion(self, source_share, share_server):
+
+        retries = (self.configuration.netapp_volume_move_cutover_timeout / 5
+                   or 1)
+
+        @manila_utils.retry(exception.ShareBusyException, interval=5,
+                            retries=retries, backoff_rate=1)
+        def check_move_completion():
+            status = self._get_volume_move_status(source_share, share_server)
+            if status['phase'].lower() != 'completed':
+                msg_args = {
+                    'shr': source_share['id'],
+                    'phs': status['phase'],
+                }
+                msg = _('Volume move operation for share %(shr)s is not '
+                        'complete. Current Phase: %(phs)s. '
+                        'Retrying.') % msg_args
+                LOG.warning(msg)
+                raise exception.ShareBusyException(reason=msg)
+
+        try:
+            check_move_completion()
+        except exception.ShareBusyException:
+            msg = _("Volume move operation did not complete after cut-over "
+                    "was triggered. Retries exhausted. Not retrying.")
+            raise exception.NetAppException(message=msg)
