@@ -17,8 +17,6 @@ NetApp cDOT CIFS protocol helper class.
 
 import re
 
-from oslo_log import log
-
 from manila.common import constants
 from manila import exception
 from manila.i18n import _
@@ -26,17 +24,16 @@ from manila.share.drivers.netapp.dataontap.protocols import base
 from manila.share.drivers.netapp import utils as na_utils
 
 
-LOG = log.getLogger(__name__)
-
-
 class NetAppCmodeCIFSHelper(base.NetAppBaseHelper):
     """NetApp cDOT CIFS protocol helper class."""
 
     @na_utils.trace
-    def create_share(self, share, share_name):
+    def create_share(self, share, share_name,
+                     clear_current_export_policy=True):
         """Creates CIFS share on Data ONTAP Vserver."""
         self._client.create_cifs_share(share_name)
-        self._client.remove_cifs_share_access(share_name, 'Everyone')
+        if clear_current_export_policy:
+            self._client.remove_cifs_share_access(share_name, 'Everyone')
 
         # Return a callback that may be used for generating export paths
         # for this share.
