@@ -45,7 +45,8 @@ class ShareAPI(object):
             migrate_share()
             get_migration_info()
             get_driver_migration_info()
-        1.7  - Update target call API in allow/deny access methods
+        1.7  - Update target call API in allow/deny access methods (Removed
+            in 1.14)
         1.8  - Introduce Share Replication:
             create_share_replica()
             delete_share_replica()
@@ -59,6 +60,20 @@ class ShareAPI(object):
             migration_get_driver_info()
         1.11 - Add create_replicated_snapshot() and
             delete_replicated_snapshot() methods
+        1.12 - Add provide_share_server(), create_share_server() and
+            migration_driver_recovery(), remove migration_get_driver_info(),
+            update migration_cancel(), migration_complete() and
+            migration_get_progress method signature, rename
+            migration_get_info() to connection_get_info()
+        1.13 - Introduce share revert to snapshot: revert_to_snapshot()
+        1.14 - Add update_access() and remove allow_access() and deny_access().
+        1.15 - Updated migration_start() method with new parameter
+            "preserve_snapshots"
+        1.16  - Convert create_consistency_group, delete_consistency_group
+                create_cgsnapshot, and delete_cgsnapshot methods to
+                create_share_group, delete_share_group
+                create_share_group_snapshot, and delete_share_group_snapshot
+        1.17 - Add snapshot_update_access()
     """
 
     BASE_RPC_API_VERSION = '1.0'
@@ -67,7 +82,7 @@ class ShareAPI(object):
         super(ShareAPI, self).__init__()
         target = messaging.Target(topic=CONF.share_topic,
                                   version=self.BASE_RPC_API_VERSION)
-        self.client = rpc.get_client(target, version_cap='1.11')
+        self.client = rpc.get_client(target, version_cap='1.17')
 
     def create_share_instance(self, context, share_instance, host,
                               request_spec, filter_properties,
@@ -111,6 +126,15 @@ class ShareAPI(object):
                           'unmanage_snapshot',
                           snapshot_id=snapshot['id'])
 
+    def revert_to_snapshot(self, context, share, snapshot, host, reservations):
+        host = utils.extract_host(host)
+        call_context = self.client.prepare(server=host, version='1.13')
+        call_context.cast(context,
+                          'revert_to_snapshot',
+                          share_id=share['id'],
+                          snapshot_id=snapshot['id'],
+                          reservations=reservations)
+
     def delete_share_instance(self, context, share_instance, force=False):
         host = utils.extract_host(share_instance['host'])
         call_context = self.client.prepare(server=host, version='1.4')
@@ -119,31 +143,30 @@ class ShareAPI(object):
                           share_instance_id=share_instance['id'],
                           force=force)
 
-    def migration_start(self, context, share, dest_host, force_host_copy,
-                        notify):
+    def migration_start(self, context, share, dest_host,
+                        force_host_assisted_migration, preserve_metadata,
+                        writable, nondisruptive, preserve_snapshots,
+                        new_share_network_id, new_share_type_id):
         new_host = utils.extract_host(share['instance']['host'])
-        call_context = self.client.prepare(server=new_host, version='1.6')
-        host_p = {'host': dest_host.host,
-                  'capabilities': dest_host.capabilities}
-        call_context.cast(context,
-                          'migration_start',
-                          share_id=share['id'],
-                          host=host_p,
-                          force_host_copy=force_host_copy,
-                          notify=notify)
+        call_context = self.client.prepare(server=new_host, version='1.15')
+        call_context.cast(
+            context,
+            'migration_start',
+            share_id=share['id'],
+            dest_host=dest_host,
+            force_host_assisted_migration=force_host_assisted_migration,
+            preserve_metadata=preserve_metadata,
+            writable=writable,
+            nondisruptive=nondisruptive,
+            preserve_snapshots=preserve_snapshots,
+            new_share_network_id=new_share_network_id,
+            new_share_type_id=new_share_type_id)
 
-    def migration_get_info(self, context, share_instance):
+    def connection_get_info(self, context, share_instance):
         new_host = utils.extract_host(share_instance['host'])
-        call_context = self.client.prepare(server=new_host, version='1.6')
+        call_context = self.client.prepare(server=new_host, version='1.12')
         return call_context.call(context,
-                                 'migration_get_info',
-                                 share_instance_id=share_instance['id'])
-
-    def migration_get_driver_info(self, context, share_instance):
-        new_host = utils.extract_host(share_instance['host'])
-        call_context = self.client.prepare(server=new_host, version='1.6')
-        return call_context.call(context,
-                                 'migration_get_driver_info',
+                                 'connection_get_info',
                                  share_instance_id=share_instance['id'])
 
     def delete_share_server(self, context, share_server):
@@ -161,12 +184,13 @@ class ShareAPI(object):
                           share_id=share['id'],
                           snapshot_id=snapshot['id'])
 
-    def delete_snapshot(self, context, snapshot, host):
+    def delete_snapshot(self, context, snapshot, host, force=False):
         new_host = utils.extract_host(host)
         call_context = self.client.prepare(server=new_host)
         call_context.cast(context,
                           'delete_snapshot',
-                          snapshot_id=snapshot['id'])
+                          snapshot_id=snapshot['id'],
+                          force=force)
 
     def create_replicated_snapshot(self, context, share, replicated_snapshot):
         host = utils.extract_host(share['instance']['host'])
@@ -186,28 +210,11 @@ class ShareAPI(object):
                           share_id=share_id,
                           force=force)
 
-    @staticmethod
-    def _get_access_rules(access):
-        if isinstance(access, list):
-            return [rule['id'] for rule in access]
-        else:
-            return [access['id']]
-
-    def allow_access(self, context, share_instance, access):
+    def update_access(self, context, share_instance):
         host = utils.extract_host(share_instance['host'])
-        call_context = self.client.prepare(server=host, version='1.7')
-        call_context.cast(context,
-                          'allow_access',
-                          share_instance_id=share_instance['id'],
-                          access_rules=self._get_access_rules(access))
-
-    def deny_access(self, context, share_instance, access):
-        host = utils.extract_host(share_instance['host'])
-        call_context = self.client.prepare(server=host, version='1.7')
-        call_context.cast(context,
-                          'deny_access',
-                          share_instance_id=share_instance['id'],
-                          access_rules=self._get_access_rules(access))
+        call_context = self.client.prepare(server=host, version='1.14')
+        call_context.cast(context, 'update_access',
+                          share_instance_id=share_instance['id'])
 
     def publish_service_capabilities(self, context):
         call_context = self.client.prepare(fanout=True, version='1.0')
@@ -230,33 +237,31 @@ class ShareAPI(object):
                           share_id=share['id'],
                           new_size=new_size)
 
-    def create_consistency_group(self, context, cg, host):
+    def create_share_group(self, context, share_group, host):
         new_host = utils.extract_host(host)
-        call_context = self.client.prepare(server=new_host, version='1.5')
-        call_context.cast(context,
-                          'create_consistency_group',
-                          cg_id=cg['id'])
+        call_context = self.client.prepare(server=new_host, version='1.16')
+        call_context.cast(
+            context, 'create_share_group', share_group_id=share_group['id'])
 
-    def delete_consistency_group(self, context, cg):
-        new_host = utils.extract_host(cg['host'])
-        call_context = self.client.prepare(server=new_host, version='1.5')
-        call_context.cast(context,
-                          'delete_consistency_group',
-                          cg_id=cg['id'])
+    def delete_share_group(self, context, share_group):
+        new_host = utils.extract_host(share_group['host'])
+        call_context = self.client.prepare(server=new_host, version='1.16')
+        call_context.cast(
+            context, 'delete_share_group', share_group_id=share_group['id'])
 
-    def create_cgsnapshot(self, context, cgsnapshot, host):
+    def create_share_group_snapshot(self, context, share_group_snapshot, host):
         new_host = utils.extract_host(host)
-        call_context = self.client.prepare(server=new_host, version='1.5')
-        call_context.cast(context,
-                          'create_cgsnapshot',
-                          cgsnapshot_id=cgsnapshot['id'])
+        call_context = self.client.prepare(server=new_host, version='1.16')
+        call_context.cast(
+            context, 'create_share_group_snapshot',
+            share_group_snapshot_id=share_group_snapshot['id'])
 
-    def delete_cgsnapshot(self, context, cgsnapshot, host):
+    def delete_share_group_snapshot(self, context, share_group_snapshot, host):
         new_host = utils.extract_host(host)
-        call_context = self.client.prepare(server=new_host, version='1.5')
-        call_context.cast(context,
-                          'delete_cgsnapshot',
-                          cgsnapshot_id=cgsnapshot['id'])
+        call_context = self.client.prepare(server=new_host, version='1.16')
+        call_context.cast(
+            context, 'delete_share_group_snapshot',
+            share_group_snapshot_id=share_group_snapshot['id'])
 
     def create_share_replica(self, context, share_replica, host,
                              request_spec, filter_properties):
@@ -295,24 +300,52 @@ class ShareAPI(object):
                           share_replica_id=share_replica['id'],
                           share_id=share_replica['share_id'])
 
-    def migration_complete(self, context, share, share_instance_id,
-                           new_share_instance_id):
-        new_host = utils.extract_host(share['host'])
-        call_context = self.client.prepare(server=new_host, version='1.10')
+    def migration_complete(self, context, src_share_instance,
+                           dest_instance_id):
+        new_host = utils.extract_host(src_share_instance['host'])
+        call_context = self.client.prepare(server=new_host, version='1.12')
         call_context.cast(context,
                           'migration_complete',
-                          share_id=share['id'],
-                          share_instance_id=share_instance_id,
-                          new_share_instance_id=new_share_instance_id)
+                          src_instance_id=src_share_instance['id'],
+                          dest_instance_id=dest_instance_id)
 
-    def migration_cancel(self, context, share):
-        new_host = utils.extract_host(share['host'])
-        call_context = self.client.prepare(server=new_host, version='1.10')
-        call_context.call(context, 'migration_cancel', share_id=share['id'])
+    def migration_cancel(self, context, src_share_instance, dest_instance_id):
+        new_host = utils.extract_host(src_share_instance['host'])
+        call_context = self.client.prepare(server=new_host, version='1.12')
+        call_context.cast(context,
+                          'migration_cancel',
+                          src_instance_id=src_share_instance['id'],
+                          dest_instance_id=dest_instance_id)
 
-    def migration_get_progress(self, context, share):
-        new_host = utils.extract_host(share['host'])
-        call_context = self.client.prepare(server=new_host, version='1.10')
+    def migration_get_progress(self, context, src_share_instance,
+                               dest_instance_id):
+        new_host = utils.extract_host(src_share_instance['host'])
+        call_context = self.client.prepare(server=new_host, version='1.12')
         return call_context.call(context,
                                  'migration_get_progress',
-                                 share_id=share['id'])
+                                 src_instance_id=src_share_instance['id'],
+                                 dest_instance_id=dest_instance_id)
+
+    def provide_share_server(self, context, share_instance, share_network_id,
+                             snapshot_id=None):
+        new_host = utils.extract_host(share_instance['host'])
+        call_context = self.client.prepare(server=new_host, version='1.12')
+        return call_context.call(context,
+                                 'provide_share_server',
+                                 share_instance_id=share_instance['id'],
+                                 share_network_id=share_network_id,
+                                 snapshot_id=snapshot_id)
+
+    def create_share_server(self, context, share_instance, share_server_id):
+        new_host = utils.extract_host(share_instance['host'])
+        call_context = self.client.prepare(server=new_host, version='1.12')
+        call_context.cast(context,
+                          'create_share_server',
+                          share_server_id=share_server_id)
+
+    def snapshot_update_access(self, context, snapshot_instance):
+        host = utils.extract_host(snapshot_instance['share_instance']['host'])
+        call_context = self.client.prepare(server=host, version='1.17')
+        call_context.cast(context,
+                          'snapshot_update_access',
+                          snapshot_instance_id=snapshot_instance['id'])
