@@ -23,7 +23,6 @@ inline callbacks.
 
 import os
 import shutil
-import uuid
 
 import fixtures
 import mock
@@ -32,8 +31,11 @@ from oslo_config import cfg
 from oslo_config import fixture as config_fixture
 import oslo_i18n
 from oslo_messaging import conffixture as messaging_conffixture
+from oslo_utils import uuidutils
 import oslotest.base as base_test
 
+from manila.api.openstack import api_version_request as api_version
+from manila import coordination
 from manila.db import migration
 from manila.db.sqlalchemy import api as db_api
 from manila.db.sqlalchemy import models as db_models
@@ -144,6 +146,12 @@ class TestCase(base_test.BaseTestCase):
 
         fake_notifier.stub_notifier(self)
 
+        # Locks must be cleaned up after tests
+        CONF.set_override('backend_url', 'file://' + lock_path,
+                          group='coordination')
+        coordination.LOCK_COORDINATOR.start()
+        self.addCleanup(coordination.LOCK_COORDINATOR.stop)
+
     def tearDown(self):
         """Runs after each test method to tear down test environment."""
         super(TestCase, self).tearDown()
@@ -173,10 +181,10 @@ class TestCase(base_test.BaseTestCase):
     def flags(self, **kw):
         """Override flag variables for a test."""
         for k, v in kw.items():
-            CONF.set_override(k, v)
+            CONF.set_override(k, v, enforce_type=True)
 
     def start_service(self, name, host=None, **kwargs):
-        host = host and host or uuid.uuid4().hex
+        host = host and host or uuidutils.generate_uuid()
         kwargs.setdefault('host', host)
         kwargs.setdefault('binary', 'manila-%s' % name)
         svc = service.Service.create(**kwargs)
@@ -351,3 +359,7 @@ class TestCase(base_test.BaseTestCase):
         conv_and_sort = lambda obj: sorted(map(obj_to_dict, obj), key=sort_key)
 
         self.assertEqual(conv_and_sort(objs1), conv_and_sort(objs2))
+
+    def is_microversion_ge(self, left, right):
+        return (api_version.APIVersionRequest(left) >=
+                api_version.APIVersionRequest(right))
