@@ -13,14 +13,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import uuid
-
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 import routes
 import webob
 import webob.dec
 import webob.request
 
+from manila.api import common as api_common
 from manila.api.middleware import auth
 from manila.api.middleware import fault
 from manila.api.openstack import api_version_request as api_version
@@ -165,7 +165,7 @@ class FakeRateLimiter(object):
 
 def get_fake_uuid(token=0):
     if token not in FAKE_UUIDS:
-        FAKE_UUIDS[token] = str(uuid.uuid4())
+        FAKE_UUIDS[token] = uuidutils.generate_uuid()
     return FAKE_UUIDS[token]
 
 
@@ -277,8 +277,58 @@ fixture_invalid_reset_status_body = (
 )
 
 
+fixture_valid_reset_status_body = (
+    ({'os-reset_status': {'status': 'creating'}}, '2.6'),
+    ({'os-reset_status': {'status': 'available'}}, '2.6'),
+    ({'os-reset_status': {'status': 'deleting'}}, '2.6'),
+    ({'os-reset_status': {'status': 'error_deleting'}}, '2.6'),
+    ({'os-reset_status': {'status': 'error'}}, '2.6'),
+    ({'os-reset_status': {'status': 'migrating'}}, '2.6'),
+    ({'os-reset_status': {'status': 'migrating_to'}}, '2.6'),
+    ({'reset_status': {'status': 'creating'}}, '2.7'),
+    ({'reset_status': {'status': 'available'}}, '2.7'),
+    ({'reset_status': {'status': 'deleting'}}, '2.7'),
+    ({'reset_status': {'status': 'error_deleting'}}, '2.7'),
+    ({'reset_status': {'status': 'error'}}, '2.7'),
+    ({'reset_status': {'status': 'migrating'}}, '2.7'),
+    ({'reset_status': {'status': 'migrating_to'}}, '2.7'),
+)
+
+
 def mock_fake_admin_check(context, resource_name, action, *args, **kwargs):
     if context.is_admin:
         return
     else:
         raise exception.PolicyNotAuthorized(action=action)
+
+
+class FakeResourceViewBuilder(api_common.ViewBuilder):
+
+        _collection_name = 'fake_resource'
+        _detail_version_modifiers = [
+            "add_field_xyzzy",
+            "add_field_spoon_for_admins",
+            "remove_field_foo",
+        ]
+
+        def view(self, req, resource):
+
+            keys = ('id', 'foo', 'fred', 'alice')
+            resource_dict = {key: resource.get(key) for key in keys}
+
+            self.update_versioned_resource_dict(req, resource_dict, resource)
+
+            return resource_dict
+
+        @api_common.ViewBuilder.versioned_method("1.41")
+        def add_field_xyzzy(self, context, resource_dict, resource):
+            resource_dict['xyzzy'] = resource.get('xyzzy')
+
+        @api_common.ViewBuilder.versioned_method("1.6")
+        def add_field_spoon_for_admins(self, context, resource_dict, resource):
+            if context.is_admin:
+                resource_dict['spoon'] = resource.get('spoon')
+
+        @api_common.ViewBuilder.versioned_method("3.14")
+        def remove_field_foo(self, context, resource_dict, resource):
+            resource_dict.pop('foo', None)
