@@ -17,11 +17,14 @@
 Test suites for 'common' code used throughout the OpenStack HTTP API.
 """
 
+import ddt
 import webob
 import webob.exc
 
 from manila.api import common
 from manila import test
+from manila.tests.api import fakes
+from manila.tests.db import fakes as db_fakes
 
 
 NS = "{http://docs.openstack.org/compute/api/v1.1}"
@@ -191,6 +194,7 @@ class PaginationParamsTest(test.TestCase):
                          common.get_pagination_params(req))
 
 
+@ddt.ddt
 class MiscFunctionsTest(test.TestCase):
 
     def test_remove_major_version_from_href(self):
@@ -240,3 +244,91 @@ class MiscFunctionsTest(test.TestCase):
         self.assertRaises(ValueError,
                           common.remove_version_from_href,
                           fixture)
+
+    def test_validate_cephx_id_invalid_with_period(self):
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          common.validate_cephx_id,
+                          "client.manila")
+
+    def test_validate_cephx_id_invalid_with_non_ascii(self):
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          common.validate_cephx_id,
+                          u"bj\u00F6rn")
+
+    @ddt.data("alice", "alice_bob", "alice bob")
+    def test_validate_cephx_id_valid(self, test_id):
+        common.validate_cephx_id(test_id)
+
+    @ddt.data(['ip', '1.1.1.1', False], ['user', 'alice', False],
+              ['cert', 'alice', False], ['cephx', 'alice', True],
+              ['ip', '172.24.41.0/24', False],)
+    @ddt.unpack
+    def test_validate_access(self, access_type, access_to, ceph):
+        common.validate_access(access_type=access_type, access_to=access_to,
+                               enable_ceph=ceph)
+
+    @ddt.data(['ip', 'alice', False], ['ip', '1.1.1.0/10/12', False],
+              ['ip', '255.255.255.265', False], ['ip', '1.1.1.0/34', False],
+              ['cert', '', False], ['cephx', 'client.alice', True],
+              ['group', 'alice', True], ['cephx', 'alice', False],
+              ['cephx', '', True], ['user', 'bob', False])
+    @ddt.unpack
+    def test_validate_access_exception(self, access_type, access_to, ceph):
+        self.assertRaises(webob.exc.HTTPBadRequest, common.validate_access,
+                          access_type=access_type, access_to=access_to,
+                          enable_ceph=ceph)
+
+
+@ddt.ddt
+class ViewBuilderTest(test.TestCase):
+
+    def setUp(self):
+        super(ViewBuilderTest, self).setUp()
+        self.expected_resource_dict = {
+            'id': 'fake_resource_id',
+            'foo': 'quz',
+            'fred': 'bob',
+            'alice': 'waldo',
+            'spoon': 'spam',
+            'xyzzy': 'qwerty',
+        }
+        self.fake_resource = db_fakes.FakeModel(self.expected_resource_dict)
+        self.view_builder = fakes.FakeResourceViewBuilder()
+
+    @ddt.data('1.0', '1.40')
+    def test_versioned_method_no_updates(self, version):
+        req = fakes.HTTPRequest.blank('/my_resource', version=version)
+
+        actual_resource = self.view_builder.view(req, self.fake_resource)
+
+        self.assertEqual(set({'id', 'foo', 'fred', 'alice'}),
+                         set(actual_resource.keys()))
+
+    @ddt.data(True, False)
+    def test_versioned_method_v1_6(self, is_admin):
+        req = fakes.HTTPRequest.blank('/my_resource', version='1.6',
+                                      use_admin_context=is_admin)
+        expected_keys = set({'id', 'foo', 'fred', 'alice'})
+        if is_admin:
+            expected_keys.add('spoon')
+
+        actual_resource = self.view_builder.view(req, self.fake_resource)
+
+        self.assertEqual(expected_keys, set(actual_resource.keys()))
+
+    @ddt.unpack
+    @ddt.data({'is_admin': True, 'version': '3.14'},
+              {'is_admin': False, 'version': '3.14'},
+              {'is_admin': False, 'version': '6.2'},
+              {'is_admin': True, 'version': '6.2'})
+    def test_versioned_method_all_match(self, is_admin, version):
+        req = fakes.HTTPRequest.blank(
+            '/my_resource', version=version, use_admin_context=is_admin)
+
+        expected_keys = set({'id', 'fred', 'xyzzy', 'alice'})
+        if is_admin:
+            expected_keys.add('spoon')
+
+        actual_resource = self.view_builder.view(req, self.fake_resource)
+
+        self.assertEqual(expected_keys, set(actual_resource.keys()))
