@@ -49,6 +49,16 @@ def set_defaults(conf):
     _safe_set_of_opts(conf, 'zfs_share_helpers', 'NFS=foo.bar.Helper')
     _safe_set_of_opts(conf, 'zfs_replica_snapshot_prefix', 'foo_prefix_')
 
+    _safe_set_of_opts(conf, 'hitachi_hsp_host', '172.24.47.190')
+    _safe_set_of_opts(conf, 'hitachi_hsp_username', 'hsp_user')
+    _safe_set_of_opts(conf, 'hitachi_hsp_password', 'hsp_password')
+
+    _safe_set_of_opts(conf, 'qnap_management_url', 'http://1.2.3.4:8080')
+    _safe_set_of_opts(conf, 'qnap_share_ip', '1.2.3.4')
+    _safe_set_of_opts(conf, 'qnap_nas_login', 'admin')
+    _safe_set_of_opts(conf, 'qnap_nas_password', 'qnapadmin')
+    _safe_set_of_opts(conf, 'qnap_poolname', 'Storage Pool 1')
+
 
 def _safe_set_of_opts(conf, *args, **kwargs):
     try:
