@@ -29,9 +29,9 @@ def _create_db_row(method, default_values, custom_values):
     return method(context.get_admin_context(), default_values)
 
 
-def create_consistency_group(**kwargs):
-    """Create a consistency group object."""
-    cg = {
+def create_share_group(**kwargs):
+    """Create a share group object."""
+    share_group = {
         'share_network_id': None,
         'share_server_id': None,
         'user_id': 'fake',
@@ -39,22 +39,22 @@ def create_consistency_group(**kwargs):
         'status': constants.STATUS_CREATING,
         'host': 'fake_host'
     }
-    return _create_db_row(db.consistency_group_create, cg, kwargs)
+    return _create_db_row(db.share_group_create, share_group, kwargs)
 
 
-def create_cgsnapshot(cg_id, **kwargs):
-    """Create a cgsnapshot object."""
+def create_share_group_snapshot(share_group_id, **kwargs):
+    """Create a share group snapshot object."""
     snapshot = {
-        'consistency_group_id': cg_id,
+        'share_group_id': share_group_id,
         'user_id': 'fake',
         'project_id': 'fake',
         'status': constants.STATUS_CREATING,
     }
-    return _create_db_row(db.cgsnapshot_create, snapshot, kwargs)
+    return _create_db_row(db.share_group_snapshot_create, snapshot, kwargs)
 
 
-def create_cgsnapshot_member(cgsnapshot_id, **kwargs):
-    """Create a cgsnapshot member object."""
+def create_share_group_snapshot_member(share_group_snapshot_id, **kwargs):
+    """Create a share group snapshot member object."""
     member = {
         'share_proto': "NFS",
         'size': 0,
@@ -63,9 +63,19 @@ def create_cgsnapshot_member(cgsnapshot_id, **kwargs):
         'user_id': 'fake',
         'project_id': 'fake',
         'status': 'creating',
-        'cgsnapshot_id': cgsnapshot_id,
+        'share_group_snapshot_id': share_group_snapshot_id,
     }
-    return _create_db_row(db.cgsnapshot_member_create, member, kwargs)
+    return _create_db_row(
+        db.share_group_snapshot_member_create, member, kwargs)
+
+
+def create_share_access(**kwargs):
+    share_access = {
+        'id': 'fake_id',
+        'access_type': 'ip',
+        'access_to': 'fake_ip_address'
+    }
+    return _create_db_row(db.share_access_create, share_access, kwargs)
 
 
 def create_share(**kwargs):
@@ -86,25 +96,32 @@ def create_share(**kwargs):
     return _create_db_row(db.share_create, share, kwargs)
 
 
+def create_share_without_instance(**kwargs):
+    share = {
+        'share_proto': "NFS",
+        'size': 0,
+        'snapshot_id': None,
+        'share_network_id': None,
+        'share_server_id': None,
+        'user_id': 'fake',
+        'project_id': 'fake',
+        'metadata': {},
+        'availability_zone': 'fake_availability_zone',
+        'status': constants.STATUS_CREATING,
+        'host': 'fake_host'
+    }
+    share.update(copy.deepcopy(kwargs))
+    return db.share_create(context.get_admin_context(), share, False)
+
+
 def create_share_instance(**kwargs):
     """Create a share instance object."""
-    instance = {
-        'host': 'fake',
-        'status': constants.STATUS_CREATING,
-    }
-    instance.update(kwargs)
-
     return db.share_instance_create(context.get_admin_context(),
                                     kwargs.pop('share_id'), kwargs)
 
 
 def create_share_replica(**kwargs):
     """Create a share replica object."""
-    replica = {
-        'host': 'fake',
-        'status': constants.STATUS_CREATING,
-    }
-    replica.update(kwargs)
 
     if 'share_id' not in kwargs:
         share = create_share()
@@ -150,14 +167,44 @@ def create_snapshot_instance(snapshot_id, **kwargs):
         context.get_admin_context(), snapshot_id, snapshot_instance)
 
 
+def create_snapshot_instance_export_locations(snapshot_id, **kwargs):
+    """Create a snapshot instance export location object."""
+    export_location = {
+        'share_snapshot_instance_id': snapshot_id,
+    }
+
+    export_location.update(kwargs)
+    return db.share_snapshot_instance_export_location_create(
+        context.get_admin_context(), export_location)
+
+
 def create_access(**kwargs):
     """Create a access rule object."""
+    state = kwargs.pop('state', constants.ACCESS_STATE_QUEUED_TO_APPLY)
     access = {
         'access_type': 'fake_type',
         'access_to': 'fake_IP',
-        'share_id': None,
+        'share_id': kwargs.pop('share_id', None) or create_share()['id'],
     }
-    return _create_db_row(db.share_access_create, access, kwargs)
+    access.update(kwargs)
+    share_access_rule = _create_db_row(db.share_access_create, access, kwargs)
+
+    for mapping in share_access_rule.instance_mappings:
+        db.share_instance_access_update(
+            context.get_admin_context(), share_access_rule['id'],
+            mapping.share_instance_id, {'state': state})
+
+    return share_access_rule
+
+
+def create_snapshot_access(**kwargs):
+    """Create a snapshot access rule object."""
+    access = {
+        'access_type': 'fake_type',
+        'access_to': 'fake_IP',
+        'share_snapshot_id': None,
+    }
+    return _create_db_row(db.share_snapshot_access_create, access, kwargs)
 
 
 def create_share_server(**kwargs):
@@ -176,6 +223,17 @@ def create_share_server(**kwargs):
                                share_srv['id'])
 
 
+def create_share_type(**kwargs):
+    """Create a share type object"""
+
+    share_type = {
+        'name': 'fake_type',
+        'is_public': True,
+    }
+
+    return _create_db_row(db.share_type_create, share_type, kwargs)
+
+
 def create_share_network(**kwargs):
     """Create a share network object."""
     net = {
