@@ -175,73 +175,106 @@ class SchedulerManagerTestCase(test.TestCase):
         mock_get_pools.assert_called_once_with(self.context, 'fake_filters')
         self.assertEqual('fake_pools', result)
 
-    @mock.patch.object(db, 'consistency_group_update', mock.Mock())
-    def test_create_cg_no_valid_host_puts_cg_in_error_state(self):
-        """Test that NoValidHost is raised for create_consistency_group.
+    @mock.patch.object(db, 'share_group_update', mock.Mock())
+    def test_create_group_no_valid_host_puts_group_in_error_state(self):
+        """Test that NoValidHost is raised for create_share_group.
 
         Puts the share in 'error' state and eats the exception.
         """
 
-        fake_cg_id = 1
-        cg_id = fake_cg_id
-        request_spec = {"consistency_group_id": cg_id}
+        fake_group_id = 1
+        group_id = fake_group_id
+        request_spec = {"share_group_id": group_id}
         with mock.patch.object(
-                self.manager.driver, 'schedule_create_consistency_group',
+                self.manager.driver, 'schedule_create_share_group',
                 mock.Mock(side_effect=self.raise_no_valid_host)):
-            self.manager.create_consistency_group(self.context,
-                                                  fake_cg_id,
-                                                  request_spec=request_spec,
-                                                  filter_properties={})
-            db.consistency_group_update.assert_called_once_with(
-                self.context, fake_cg_id, {'status': 'error'})
-            (self.manager.driver.schedule_create_consistency_group.
-                assert_called_once_with(self.context, cg_id, request_spec, {}))
+            self.manager.create_share_group(self.context,
+                                            fake_group_id,
+                                            request_spec=request_spec,
+                                            filter_properties={})
+            db.share_group_update.assert_called_once_with(
+                self.context, fake_group_id, {'status': 'error'})
+            (self.manager.driver.schedule_create_share_group.
+                assert_called_once_with(self.context, group_id, request_spec,
+                                        {}))
 
-    @mock.patch.object(db, 'consistency_group_update', mock.Mock())
-    def test_create_cg_exception_puts_cg_in_error_state(self):
-        """Test that exceptions for create_consistency_group.
+    @mock.patch.object(db, 'share_group_update', mock.Mock())
+    def test_create_group_exception_puts_group_in_error_state(self):
+        """Test that exceptions for create_share_group.
 
         Puts the share in 'error' state and raises the exception.
         """
 
-        fake_cg_id = 1
-        cg_id = fake_cg_id
-        request_spec = {"consistency_group_id": cg_id}
+        fake_group_id = 1
+        group_id = fake_group_id
+        request_spec = {"share_group_id": group_id}
         with mock.patch.object(self.manager.driver,
-                               'schedule_create_consistency_group',
+                               'schedule_create_share_group',
                                mock.Mock(side_effect=exception.NotFound)):
             self.assertRaises(exception.NotFound,
-                              self.manager.create_consistency_group,
-                              self.context, fake_cg_id,
+                              self.manager.create_share_group,
+                              self.context, fake_group_id,
                               request_spec=request_spec,
                               filter_properties={})
 
     def test_migrate_share_to_host(self):
 
+        class fake_host(object):
+            host = 'fake@backend#pool'
+
         share = db_utils.create_share()
-        host = 'fake@backend#pool'
+        host = fake_host()
 
         self.mock_object(db, 'share_get', mock.Mock(return_value=share))
-        self.mock_object(share_rpcapi.ShareAPI, 'migration_start')
+        self.mock_object(share_rpcapi.ShareAPI, 'migration_start',
+                         mock.Mock(side_effect=TypeError))
         self.mock_object(base.Scheduler,
                          'host_passes_filters',
                          mock.Mock(return_value=host))
 
-        self.manager.migrate_share_to_host(self.context, share['id'], host,
-                                           False, True, {}, None)
+        self.assertRaises(
+            TypeError, self.manager.migrate_share_to_host,
+            self.context, share['id'], 'fake@backend#pool', False, True, True,
+            False, True, 'fake_net_id', 'fake_type_id', {}, None)
 
-    def test_migrate_share_to_host_no_valid_host(self):
+        db.share_get.assert_called_once_with(self.context, share['id'])
+        base.Scheduler.host_passes_filters.assert_called_once_with(
+            self.context, 'fake@backend#pool', {}, None)
+        share_rpcapi.ShareAPI.migration_start.assert_called_once_with(
+            self.context, share, host.host, False, True, True, False, True,
+            'fake_net_id', 'fake_type_id')
 
-        share = db_utils.create_share()
+    @ddt.data(exception.NoValidHost(reason='fake'), TypeError)
+    def test_migrate_share_to_host_exception(self, exc):
+
+        share = db_utils.create_share(status=constants.STATUS_MIGRATING)
         host = 'fake@backend#pool'
+        request_spec = {'share_id': share['id']}
 
+        self.mock_object(db, 'share_get', mock.Mock(return_value=share))
         self.mock_object(
             base.Scheduler, 'host_passes_filters',
-            mock.Mock(side_effect=[exception.NoValidHost('fake')]))
+            mock.Mock(side_effect=exc))
+        self.mock_object(db, 'share_update')
+        self.mock_object(db, 'share_instance_update')
+
+        capture = (exception.NoValidHost if
+                   isinstance(exc, exception.NoValidHost) else TypeError)
 
         self.assertRaises(
-            exception.NoValidHost, self.manager.migrate_share_to_host,
-            self.context, share['id'], host, False, True, {}, None)
+            capture, self.manager.migrate_share_to_host,
+            self.context, share['id'], host, False, True, True, False, True,
+            'fake_net_id', 'fake_type_id', request_spec, None)
+
+        base.Scheduler.host_passes_filters.assert_called_once_with(
+            self.context, host, request_spec, None)
+        db.share_get.assert_called_once_with(self.context, share['id'])
+        db.share_update.assert_called_once_with(
+            self.context, share['id'],
+            {'task_state': constants.TASK_STATE_MIGRATION_ERROR})
+        db.share_instance_update.assert_called_once_with(
+            self.context, share.instance['id'],
+            {'status': constants.STATUS_AVAILABLE})
 
     def test_manage_share(self):
 
@@ -258,13 +291,20 @@ class SchedulerManagerTestCase(test.TestCase):
 
         share = db_utils.create_share()
 
+        db_update = self.mock_object(db, 'share_update', mock.Mock())
         self.mock_object(
             base.Scheduler, 'host_passes_filters',
             mock.Mock(side_effect=exception.NoValidHost('fake')))
 
+        share_id = share['id']
+
         self.assertRaises(
             exception.NoValidHost, self.manager.manage_share,
-            self.context, share['id'], 'driver_options', {}, None)
+            self.context, share['id'], 'driver_options',
+            {'share_id': share_id}, None)
+        db_update.assert_called_once_with(
+            self.context, share_id,
+            {'status': constants.STATUS_MANAGE_ERROR, 'size': 1})
 
     def test_create_share_replica_exception_path(self):
         """Test 'raisable' exceptions for create_share_replica."""
