@@ -249,24 +249,26 @@ class CephFSNativeDriverTestCase(test.TestCase):
 
     def test_update_access_add_rm(self):
         alice = {
-            'id': 'accessid1',
+            'id': 'instance_mapping_id1',
+            'access_id': 'accessid1',
             'access_level': 'rw',
             'access_type': 'cephx',
             'access_to': 'alice'
         }
         bob = {
-            'id': 'accessid2',
+            'id': 'instance_mapping_id2',
+            'access_id': 'accessid2',
             'access_level': 'rw',
             'access_type': 'cephx',
             'access_to': 'bob'
         }
 
-        access_keys = self._driver.update_access(self._context, self._share,
-                                                 access_rules=[alice],
-                                                 add_rules=[alice],
-                                                 delete_rules=[bob])
+        access_updates = self._driver.update_access(
+            self._context, self._share, access_rules=[alice],
+            add_rules=[alice], delete_rules=[bob])
 
-        self.assertEqual({'accessid1': 'abc123'}, access_keys)
+        self.assertEqual(
+            {'accessid1': {'access_key': 'abc123'}}, access_updates)
         self._driver._volume_client.authorize.assert_called_once_with(
             self._driver._share_path(self._share),
             "alice",
@@ -279,19 +281,21 @@ class CephFSNativeDriverTestCase(test.TestCase):
     @ddt.data(None, 1)
     def test_update_access_all(self, volume_client_version):
         alice = {
-            'id': 'accessid1',
+            'id': 'instance_mapping_id1',
+            'access_id': 'accessid1',
             'access_level': 'rw',
             'access_type': 'cephx',
             'access_to': 'alice'
         }
         self._driver.volume_client.version = volume_client_version
 
-        access_keys = self._driver.update_access(self._context, self._share,
-                                                 access_rules=[alice],
-                                                 add_rules=[],
-                                                 delete_rules=[])
+        access_updates = self._driver.update_access(self._context, self._share,
+                                                    access_rules=[alice],
+                                                    add_rules=[],
+                                                    delete_rules=[])
 
-        self.assertEqual({'accessid1': 'abc123'}, access_keys)
+        self.assertEqual(
+            {'accessid1': {'access_key': 'abc123'}}, access_updates)
         if volume_client_version:
             (self._driver._volume_client.get_authorized_ids.
              assert_called_once_with(self._driver._share_path(self._share)))
@@ -376,32 +380,30 @@ class CephFSNativeDriverTestCase(test.TestCase):
                 self._driver._share_path(self._share),
                 "snappy1_instance1"))
 
-    def test_create_consistency_group(self):
-        self._driver.create_consistency_group(self._context, {"id": "grp1"},
-                                              None)
+    def test_create_share_group(self):
+        self._driver.create_share_group(self._context, {"id": "grp1"}, None)
 
         self._driver._volume_client.create_group.assert_called_once_with(
             "grp1")
 
-    def test_delete_consistency_group(self):
-        self._driver.delete_consistency_group(self._context, {"id": "grp1"},
-                                              None)
+    def test_delete_share_group(self):
+        self._driver.delete_share_group(self._context, {"id": "grp1"}, None)
 
         self._driver._volume_client.destroy_group.assert_called_once_with(
             "grp1")
 
-    def test_create_cg_snapshot(self):
-        self._driver.create_cgsnapshot(self._context, {
-            'consistency_group_id': 'cgid',
+    def test_create_share_snapshot(self):
+        self._driver.create_share_group_snapshot(self._context, {
+            'share_group_id': 'cgid',
             'id': 'snapid'
         })
 
         (self._driver._volume_client.create_snapshot_group.
          assert_called_once_with("cgid", "snapid"))
 
-    def test_delete_cgsnapshot(self):
-        self._driver.delete_cgsnapshot(self._context, {
-            'consistency_group_id': 'cgid',
+    def test_delete_share_group_snapshot(self):
+        self._driver.delete_share_group_snapshot(self._context, {
+            'share_group_id': 'cgid',
             'id': 'snapid'
         })
 
@@ -449,3 +451,15 @@ class CephFSNativeDriverTestCase(test.TestCase):
                           self._driver.create_share,
                           self._context,
                           self._share)
+
+    def test_check_for_setup_error(self):
+        self._driver.check_for_setup_error()
+        self._driver._volume_client.connect.assert_called_once_with(
+            premount_evict='manila')
+
+    def test_check_for_setup_error_with_connection_error(self):
+        cephfs_native.ceph_module_found = False
+        cephfs_native.ceph_volume_client = None
+
+        self.assertRaises(exception.ManilaException,
+                          self._driver.check_for_setup_error)
