@@ -12,6 +12,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from copy import deepcopy
 import sys
 
 import ddt
@@ -26,6 +27,43 @@ from manila import test
 from manila.tests.share.drivers.hpe import test_hpe_3par_constants as constants
 
 
+@ddt.ddt
+class HPE3ParDriverFPGTestCase(test.TestCase):
+
+    def setUp(self):
+        super(HPE3ParDriverFPGTestCase, self).setUp()
+
+    @ddt.data((-1, 4),
+              (0, 5),
+              (0, -1))
+    @ddt.unpack
+    def test_FPG_init_args_failure(self, min_ip, max_ip):
+        self.assertRaises(exception.HPE3ParInvalid,
+                          hpe3pardriver.FPG, min_ip, max_ip)
+
+    @ddt.data(('invalid_ip_fpg, 10.256.0.1', 0, 4),
+              (None, 0, 4),
+              (' ', 0, 4),
+              ('', 0, 4),
+              ('max_ip_fpg, 10.0.0.1, 10.0.0.2, 10.0.0.3, 10.0.0.4, 10.0.0.5',
+              0, 4),
+              ('min_1_ip_fpg', 1, 4))
+    @ddt.unpack
+    def test_FPG_type_failures(self, value, min_ip, max_ip):
+        fpg_type_obj = hpe3pardriver.FPG(min_ip=min_ip, max_ip=max_ip)
+        self.assertRaises(exception.HPE3ParInvalid, fpg_type_obj, value)
+
+    @ddt.data(('samplefpg, 10.0.0.1', {'samplefpg': ['10.0.0.1']}),
+              ('samplefpg', {'samplefpg': []}),
+              ('samplefpg, 10.0.0.1, 10.0.0.2',
+               {'samplefpg': ['10.0.0.1', '10.0.0.2']}))
+    @ddt.unpack
+    def test_FPG_type_success(self, value, expected_fpg):
+        fpg_type_obj = hpe3pardriver.FPG()
+        fpg = fpg_type_obj(value)
+        self.assertEqual(expected_fpg, fpg)
+
+
 @ddt.ddt
 class HPE3ParDriverTestCase(test.TestCase):
 
@@ -42,13 +80,11 @@ class HPE3ParDriverTestCase(test.TestCase):
         self.conf.hpe3par_san_login = constants.SAN_LOGIN
         self.conf.hpe3par_san_password = constants.SAN_PASSWORD
         self.conf.hpe3par_san_ip = constants.EXPECTED_IP_1234
-        self.conf.hpe3par_fpg = constants.EXPECTED_FPG
+        self.conf.hpe3par_fpg = constants.EXPECTED_FPG_CONF
         self.conf.hpe3par_san_ssh_port = constants.PORT
         self.conf.ssh_conn_timeout = constants.TIMEOUT
-        self.conf.hpe3par_share_ip_address = None
         self.conf.hpe3par_fstore_per_share = False
         self.conf.hpe3par_require_cifs_ip = False
-        self.conf.hpe3par_share_ip_address = constants.EXPECTED_IP_10203040
         self.conf.hpe3par_cifs_admin_access_username = constants.USERNAME,
         self.conf.hpe3par_cifs_admin_access_password = constants.PASSWORD,
         self.conf.hpe3par_cifs_admin_access_domain = (
@@ -58,6 +94,8 @@ class HPE3ParDriverTestCase(test.TestCase):
         self.conf.network_config_group = 'test_network_config_group'
         self.conf.admin_network_config_group = (
             'test_admin_network_config_group')
+        self.conf.filter_function = None
+        self.conf.goodness_function = None
 
         def safe_get(attr):
             try:
@@ -70,14 +108,20 @@ class HPE3ParDriverTestCase(test.TestCase):
         self.mock_object(hpe3parmediator, 'HPE3ParMediator')
         self.mock_mediator_constructor = hpe3parmediator.HPE3ParMediator
         self.mock_mediator = self.mock_mediator_constructor()
+        # restore needed static methods
+        self.mock_mediator.ensure_supported_protocol = (
+            self.real_hpe_3par_mediator.ensure_supported_protocol)
+        self.mock_mediator.build_export_locations = (
+            self.real_hpe_3par_mediator.build_export_locations)
 
         self.driver = hpe3pardriver.HPE3ParShareDriver(
             configuration=self.conf)
 
-    def test_driver_setup_success(self):
+    def test_driver_setup_success(self,
+                                  get_vfs_ret_val=constants.EXPECTED_GET_VFS):
         """Driver do_setup without any errors."""
 
-        self.mock_mediator.get_vfs_name.return_value = constants.EXPECTED_VFS
+        self.mock_mediator.get_vfs.return_value = get_vfs_ret_val
 
         self.driver.do_setup(None)
         conf = self.conf
@@ -92,8 +136,6 @@ class HPE3ParDriverTestCase(test.TestCase):
                       hpe3par_san_ip=conf.hpe3par_san_ip,
                       hpe3par_fstore_per_share=conf.hpe3par_fstore_per_share,
                       hpe3par_require_cifs_ip=conf.hpe3par_require_cifs_ip,
-                      hpe3par_share_ip_address=(
-                          self.conf.hpe3par_share_ip_address),
                       hpe3par_cifs_admin_access_username=(
                           conf.hpe3par_cifs_admin_access_username),
                       hpe3par_cifs_admin_access_password=(
@@ -106,27 +148,64 @@ class HPE3ParDriverTestCase(test.TestCase):
 
         self.mock_mediator.assert_has_calls([
             mock.call.do_setup(),
-            mock.call.get_vfs_name(conf.hpe3par_fpg)])
+            mock.call.get_vfs(constants.EXPECTED_FPG)])
 
-        self.assertEqual(constants.EXPECTED_VFS, self.driver.vfs)
+    def test_driver_setup_dhss_success(self):
+        """Driver do_setup without any errors with dhss=True."""
+
+        self.test_driver_setup_success()
+        self.assertEqual(constants.EXPECTED_FPG_MAP, self.driver.fpgs)
 
     def test_driver_setup_no_dhss_success(self):
         """Driver do_setup without any errors with dhss=False."""
 
         self.conf.driver_handles_share_servers = False
-        self.conf.hpe3par_share_ip_address = constants.EXPECTED_IP_10203040
+        self.test_driver_setup_success()
+        self.assertEqual(constants.EXPECTED_FPG_MAP, self.driver.fpgs)
+
+    def test_driver_setup_no_dhss_multi_getvfs_success(self):
+        """Driver do_setup when dhss=False, getvfs returns multiple IPs."""
+
+        self.conf.driver_handles_share_servers = False
+        self.test_driver_setup_success(
+            get_vfs_ret_val=constants.EXPECTED_GET_VFS_MULTIPLES)
+        self.assertEqual(constants.EXPECTED_FPG_MAP,
+                         self.driver.fpgs)
+
+    def test_driver_setup_success_no_dhss_no_conf_ss_ip(self):
+        """test driver's do_setup()
+
+        Driver do_setup with dhss=False, share server ip not set in config file
+        but discoverable at 3par array
+        """
+
+        self.conf.driver_handles_share_servers = False
+        # ss ip not provided in conf
+        original_fpg = deepcopy(self.conf.hpe3par_fpg)
+        self.conf.hpe3par_fpg[0][constants.EXPECTED_FPG] = []
 
         self.test_driver_setup_success()
 
-    def test_driver_setup_no_ss_no_ip(self):
+        self.assertEqual(constants.EXPECTED_FPG_MAP, self.driver.fpgs)
+        constants.EXPECTED_FPG_CONF = original_fpg
+
+    def test_driver_setup_failure_no_dhss_no_conf_ss_ip(self):
         """Configured IP address is required for dhss=False."""
 
         self.conf.driver_handles_share_servers = False
-        self.conf.hpe3par_share_ip_address = None
+        # ss ip not provided in conf
+        fpg_without_ss_ip = deepcopy(self.conf.hpe3par_fpg)
+        self.conf.hpe3par_fpg[0][constants.EXPECTED_FPG] = []
+        # ss ip not configured on array
+        vfs_without_ss_ip = deepcopy(constants.EXPECTED_GET_VFS)
+        vfs_without_ss_ip['vfsip']['address'] = []
+        self.mock_mediator.get_vfs.return_value = vfs_without_ss_ip
+
         self.assertRaises(exception.HPE3ParInvalid,
                           self.driver.do_setup, None)
+        constants.EXPECTED_FPG_CONF = fpg_without_ss_ip
 
-    def test_driver_with_setup_error(self):
+    def test_driver_setup_mediator_error(self):
         """Driver do_setup when the mediator setup fails."""
 
         self.mock_mediator.do_setup.side_effect = (
@@ -147,8 +226,6 @@ class HPE3ParDriverTestCase(test.TestCase):
                       hpe3par_san_ip=conf.hpe3par_san_ip,
                       hpe3par_fstore_per_share=conf.hpe3par_fstore_per_share,
                       hpe3par_require_cifs_ip=conf.hpe3par_require_cifs_ip,
-                      hpe3par_share_ip_address=(
-                          self.conf.hpe3par_share_ip_address),
                       hpe3par_cifs_admin_access_username=(
                           conf.hpe3par_cifs_admin_access_username),
                       hpe3par_cifs_admin_access_password=(
@@ -161,10 +238,10 @@ class HPE3ParDriverTestCase(test.TestCase):
 
         self.mock_mediator.assert_has_calls([mock.call.do_setup()])
 
-    def test_driver_with_vfs_error(self):
-        """Driver do_setup when the get_vfs_name fails."""
+    def test_driver_setup_with_vfs_error(self):
+        """Driver do_setup when the get_vfs fails."""
 
-        self.mock_mediator.get_vfs_name.side_effect = (
+        self.mock_mediator.get_vfs.side_effect = (
             exception.ShareBackendException('fail'))
 
         self.assertRaises(exception.ShareBackendException,
@@ -182,8 +259,6 @@ class HPE3ParDriverTestCase(test.TestCase):
                       hpe3par_san_ip=conf.hpe3par_san_ip,
                       hpe3par_fstore_per_share=conf.hpe3par_fstore_per_share,
                       hpe3par_require_cifs_ip=conf.hpe3par_require_cifs_ip,
-                      hpe3par_share_ip_address=(
-                          self.conf.hpe3par_share_ip_address),
                       hpe3par_cifs_admin_access_username=(
                           conf.hpe3par_cifs_admin_access_username),
                       hpe3par_cifs_admin_access_password=(
@@ -196,64 +271,53 @@ class HPE3ParDriverTestCase(test.TestCase):
 
         self.mock_mediator.assert_has_calls([
             mock.call.do_setup(),
-            mock.call.get_vfs_name(conf.hpe3par_fpg)])
+            mock.call.get_vfs(constants.EXPECTED_FPG)])
+
+    def test_driver_setup_conf_ips_validation_fails(self):
+        """Driver do_setup when the _validate_pool_ips fails."""
+
+        self.conf.driver_handles_share_servers = False
+        vfs_with_ss_ip = deepcopy(constants.EXPECTED_GET_VFS)
+        vfs_with_ss_ip['vfsip']['address'] = ['10.100.100.100']
+        self.mock_mediator.get_vfs.return_value = vfs_with_ss_ip
+        self.assertRaises(exception.HPE3ParInvalid,
+                          self.driver.do_setup, None)
+
+        conf = self.conf
+        self.mock_mediator_constructor.assert_has_calls([
+            mock.call(hpe3par_san_ssh_port=conf.hpe3par_san_ssh_port,
+                      hpe3par_san_password=conf.hpe3par_san_password,
+                      hpe3par_username=conf.hpe3par_username,
+                      hpe3par_san_login=conf.hpe3par_san_login,
+                      hpe3par_debug=conf.hpe3par_debug,
+                      hpe3par_api_url=conf.hpe3par_api_url,
+                      hpe3par_password=conf.hpe3par_password,
+                      hpe3par_san_ip=conf.hpe3par_san_ip,
+                      hpe3par_fstore_per_share=conf.hpe3par_fstore_per_share,
+                      hpe3par_require_cifs_ip=conf.hpe3par_require_cifs_ip,
+                      hpe3par_cifs_admin_access_username=(
+                          conf.hpe3par_cifs_admin_access_username),
+                      hpe3par_cifs_admin_access_password=(
+                          conf.hpe3par_cifs_admin_access_password),
+                      hpe3par_cifs_admin_access_domain=(
+                          conf.hpe3par_cifs_admin_access_domain),
+                      hpe3par_share_mount_path=conf.hpe3par_share_mount_path,
+                      my_ip=self.conf.my_ip,
+                      ssh_conn_timeout=conf.ssh_conn_timeout)])
+
+        self.mock_mediator.assert_has_calls([
+            mock.call.do_setup(),
+            mock.call.get_vfs(constants.EXPECTED_FPG)])
 
     def init_driver(self):
         """Simple driver setup for re-use with tests that need one."""
 
         self.driver._hpe3par = self.mock_mediator
-        self.driver.vfs = constants.EXPECTED_VFS
-        self.driver.fpg = constants.EXPECTED_FPG
+        self.driver.fpgs = constants.EXPECTED_FPG_MAP
         self.mock_object(hpe3pardriver, 'share_types')
         get_extra_specs = hpe3pardriver.share_types.get_extra_specs_from_share
         get_extra_specs.return_value = constants.EXPECTED_EXTRA_SPECS
 
-    def do_create_share(self, protocol, share_type_id, expected_project_id,
-                        expected_share_id, expected_size):
-        """Re-usable code for create share."""
-        context = None
-        share_server = {
-            'backend_details': {'ip': constants.EXPECTED_IP_10203040}}
-        share = {
-            'display_name': constants.EXPECTED_SHARE_NAME,
-            'host': constants.EXPECTED_HOST,
-            'project_id': expected_project_id,
-            'id': expected_share_id,
-            'share_proto': protocol,
-            'share_type_id': share_type_id,
-            'size': expected_size,
-        }
-        location = self.driver.create_share(context, share, share_server)
-        return location
-
-    def do_create_share_from_snapshot(self,
-                                      protocol,
-                                      share_type_id,
-                                      snapshot_instance,
-                                      expected_share_id,
-                                      expected_size):
-        """Re-usable code for create share from snapshot."""
-        context = None
-        share_server = {
-            'backend_details': {
-                'ip': constants.EXPECTED_IP_10203040,
-            },
-        }
-        share = {
-            'project_id': constants.EXPECTED_PROJECT_ID,
-            'display_name': constants.EXPECTED_SHARE_NAME,
-            'host': constants.EXPECTED_HOST,
-            'id': expected_share_id,
-            'share_proto': protocol,
-            'share_type_id': share_type_id,
-            'size': expected_size,
-        }
-        location = self.driver.create_share_from_snapshot(context,
-                                                          share,
-                                                          snapshot_instance,
-                                                          share_server)
-        return location
-
     def test_driver_check_for_setup_error_success(self):
         """check_for_setup_error when things go well."""
 
@@ -282,6 +346,98 @@ class HPE3ParDriverTestCase(test.TestCase):
         ]
         hpe3pardriver.LOG.assert_has_calls(expected_calls)
 
+    @ddt.data(([constants.SHARE_SERVER], constants.SHARE_SERVER),
+              ([], None),)
+    @ddt.unpack
+    def test_choose_share_server_compatible_with_share(self, share_servers,
+                                                       expected_share_sever):
+        context = None
+        share_server = self.driver.choose_share_server_compatible_with_share(
+            context,
+            share_servers,
+            constants.NFS_SHARE_INFO,
+            None,
+            None)
+
+        self.assertEqual(expected_share_sever, share_server)
+
+    def test_choose_share_server_compatible_with_share_with_cg(self):
+        context = None
+        cg_ref = {'id': 'dummy'}
+        self.assertRaises(
+            exception.InvalidRequest,
+            self.driver.choose_share_server_compatible_with_share,
+            context,
+            [constants.SHARE_SERVER],
+            constants.NFS_SHARE_INFO,
+            None,
+            cg_ref)
+
+    def do_create_share(self, protocol, share_type_id, expected_project_id,
+                        expected_share_id, expected_size):
+        """Re-usable code for create share."""
+        context = None
+
+        share = {
+            'display_name': constants.EXPECTED_SHARE_NAME,
+            'host': constants.EXPECTED_HOST,
+            'project_id': expected_project_id,
+            'id': expected_share_id,
+            'share_proto': protocol,
+            'share_type_id': share_type_id,
+            'size': expected_size,
+        }
+        location = self.driver.create_share(context, share,
+                                            constants.SHARE_SERVER)
+        return location
+
+    def do_create_share_from_snapshot(self,
+                                      protocol,
+                                      share_type_id,
+                                      snapshot_instance,
+                                      expected_share_id,
+                                      expected_size):
+        """Re-usable code for create share from snapshot."""
+        context = None
+        share = {
+            'project_id': constants.EXPECTED_PROJECT_ID,
+            'display_name': constants.EXPECTED_SHARE_NAME,
+            'host': constants.EXPECTED_HOST,
+            'id': expected_share_id,
+            'share_proto': protocol,
+            'share_type_id': share_type_id,
+            'size': expected_size,
+        }
+        location = self.driver.create_share_from_snapshot(
+            context,
+            share,
+            snapshot_instance,
+            constants.SHARE_SERVER)
+        return location
+
+    @ddt.data((constants.UNEXPECTED_HOST, exception.InvalidHost),
+              (constants.HOST_WITHOUT_POOL_1, exception.InvalidHost),
+              (constants.HOST_WITHOUT_POOL_2, exception.InvalidHost))
+    @ddt.unpack
+    def test_driver_create_share_fails_get_pool_location(self, host,
+                                                         expected_exception):
+        """get_pool_location fails to extract pool name from host"""
+        self.init_driver()
+        context = None
+        share_server = None
+        share = {
+            'display_name': constants.EXPECTED_SHARE_NAME,
+            'host': host,
+            'project_id': constants.EXPECTED_PROJECT_ID,
+            'id': constants.EXPECTED_SHARE_ID,
+            'share_proto': constants.CIFS,
+            'share_type_id': constants.SHARE_TYPE_ID,
+            'size': constants.EXPECTED_SIZE_2,
+        }
+        self.assertRaises(expected_exception,
+                          self.driver.create_share,
+                          context, share, share_server)
+
     def test_driver_create_cifs_share(self):
         self.init_driver()
 
@@ -291,13 +447,15 @@ class HPE3ParDriverTestCase(test.TestCase):
         self.mock_mediator.create_share.return_value = (
             constants.EXPECTED_SHARE_NAME)
 
+        hpe3parmediator.HPE3ParMediator = self.real_hpe_3par_mediator
+
         location = self.do_create_share(constants.CIFS,
                                         constants.SHARE_TYPE_ID,
                                         constants.EXPECTED_PROJECT_ID,
                                         constants.EXPECTED_SHARE_ID,
                                         constants.EXPECTED_SIZE_2)
 
-        self.assertEqual(expected_location, location)
+        self.assertIn(expected_location, location)
         expected_calls = [mock.call.create_share(
             constants.EXPECTED_PROJECT_ID,
             constants.EXPECTED_SHARE_ID,
@@ -317,6 +475,7 @@ class HPE3ParDriverTestCase(test.TestCase):
 
         self.mock_mediator.create_share.return_value = (
             constants.EXPECTED_SHARE_PATH)
+        hpe3parmediator.HPE3ParMediator = self.real_hpe_3par_mediator
 
         location = self.do_create_share(constants.NFS,
                                         constants.SHARE_TYPE_ID,
@@ -324,7 +483,7 @@ class HPE3ParDriverTestCase(test.TestCase):
                                         constants.EXPECTED_SHARE_ID,
                                         constants.EXPECTED_SIZE_1)
 
-        self.assertEqual(expected_location, location)
+        self.assertIn(expected_location, location)
         expected_calls = [
             mock.call.create_share(constants.EXPECTED_PROJECT_ID,
                                    constants.EXPECTED_SHARE_ID,
@@ -345,6 +504,7 @@ class HPE3ParDriverTestCase(test.TestCase):
 
         self.mock_mediator.create_share_from_snapshot.return_value = (
             constants.EXPECTED_SHARE_NAME)
+        hpe3parmediator.HPE3ParMediator = self.real_hpe_3par_mediator
 
         snapshot_instance = constants.SNAPSHOT_INSTANCE.copy()
         snapshot_instance['protocol'] = constants.CIFS
@@ -356,7 +516,7 @@ class HPE3ParDriverTestCase(test.TestCase):
             constants.EXPECTED_SHARE_ID,
             constants.EXPECTED_SIZE_2)
 
-        self.assertEqual(expected_location, location)
+        self.assertIn(expected_location, location)
         expected_calls = [
             mock.call.create_share_from_snapshot(
                 constants.EXPECTED_SHARE_ID,
@@ -367,7 +527,9 @@ class HPE3ParDriverTestCase(test.TestCase):
                 constants.EXPECTED_SNAP_ID,
                 constants.EXPECTED_FPG,
                 constants.EXPECTED_VFS,
-                comment=mock.ANY),
+                [constants.EXPECTED_IP_10203040],
+                comment=mock.ANY,
+                size=constants.EXPECTED_SIZE_2),
         ]
         self.mock_mediator.assert_has_calls(expected_calls)
 
@@ -379,6 +541,7 @@ class HPE3ParDriverTestCase(test.TestCase):
 
         self.mock_mediator.create_share_from_snapshot.return_value = (
             constants.EXPECTED_SHARE_PATH)
+        hpe3parmediator.HPE3ParMediator = self.real_hpe_3par_mediator
 
         location = self.do_create_share_from_snapshot(
             constants.NFS,
@@ -387,7 +550,7 @@ class HPE3ParDriverTestCase(test.TestCase):
             constants.EXPECTED_SHARE_ID,
             constants.EXPECTED_SIZE_1)
 
-        self.assertEqual(expected_location, location)
+        self.assertIn(expected_location, location)
         expected_calls = [
             mock.call.create_share_from_snapshot(
                 constants.EXPECTED_SHARE_ID,
@@ -398,7 +561,9 @@ class HPE3ParDriverTestCase(test.TestCase):
                 constants.EXPECTED_SNAP_ID,
                 constants.EXPECTED_FPG,
                 constants.EXPECTED_VFS,
-                comment=mock.ANY),
+                [constants.EXPECTED_IP_10203040],
+                comment=mock.ANY,
+                size=constants.EXPECTED_SIZE_1),
         ]
 
         self.mock_mediator.assert_has_calls(expected_calls)
@@ -413,6 +578,7 @@ class HPE3ParDriverTestCase(test.TestCase):
             'id': constants.EXPECTED_SHARE_ID,
             'share_proto': constants.CIFS,
             'size': constants.EXPECTED_SIZE_1,
+            'host': constants.EXPECTED_HOST
         }
 
         self.driver.delete_share(context, share, share_server)
@@ -420,9 +586,11 @@ class HPE3ParDriverTestCase(test.TestCase):
         expected_calls = [
             mock.call.delete_share(constants.EXPECTED_PROJECT_ID,
                                    constants.EXPECTED_SHARE_ID,
+                                   constants.EXPECTED_SIZE_1,
                                    constants.CIFS,
                                    constants.EXPECTED_FPG,
-                                   constants.EXPECTED_VFS)]
+                                   constants.EXPECTED_VFS,
+                                   constants.EXPECTED_IP_10203040)]
 
         self.mock_mediator.assert_has_calls(expected_calls)
 
@@ -463,44 +631,53 @@ class HPE3ParDriverTestCase(test.TestCase):
         ]
         self.mock_mediator.assert_has_calls(expected_calls)
 
-    def test_driver_allow_access(self):
+    def test_driver_update_access_add_rule(self):
         self.init_driver()
 
         context = None
-        self.driver.allow_access(context,
-                                 constants.NFS_SHARE_INFO,
-                                 constants.ACCESS_INFO)
+
+        self.driver.update_access(context,
+                                  constants.NFS_SHARE_INFO,
+                                  [constants.ACCESS_RULE_NFS],
+                                  [constants.ADD_RULE_IP],
+                                  [],
+                                  constants.SHARE_SERVER)
 
         expected_calls = [
-            mock.call.allow_access(constants.EXPECTED_PROJECT_ID,
-                                   constants.EXPECTED_SHARE_ID,
-                                   constants.NFS,
-                                   constants.EXPECTED_EXTRA_SPECS,
-                                   constants.IP,
-                                   constants.EXPECTED_IP_1234,
-                                   constants.ACCESS_INFO['access_level'],
-                                   constants.EXPECTED_FPG,
-                                   constants.EXPECTED_VFS)
+            mock.call.update_access(constants.EXPECTED_PROJECT_ID,
+                                    constants.EXPECTED_SHARE_ID,
+                                    constants.NFS,
+                                    constants.EXPECTED_EXTRA_SPECS,
+                                    [constants.ACCESS_RULE_NFS],
+                                    [constants.ADD_RULE_IP],
+                                    [],
+                                    constants.EXPECTED_FPG,
+                                    constants.EXPECTED_VFS)
         ]
         self.mock_mediator.assert_has_calls(expected_calls)
 
-    def test_driver_deny_access(self):
+    def test_driver_update_access_delete_rule(self):
         self.init_driver()
 
         context = None
-        self.driver.deny_access(context,
-                                constants.NFS_SHARE_INFO,
-                                constants.ACCESS_INFO)
+
+        self.driver.update_access(context,
+                                  constants.NFS_SHARE_INFO,
+                                  [constants.ACCESS_RULE_NFS],
+                                  [],
+                                  [constants.DELETE_RULE_IP],
+                                  constants.SHARE_SERVER)
 
         expected_calls = [
-            mock.call.deny_access(constants.EXPECTED_PROJECT_ID,
-                                  constants.EXPECTED_SHARE_ID,
-                                  constants.NFS,
-                                  constants.IP,
-                                  constants.EXPECTED_IP_1234,
-                                  constants.READ_WRITE,
-                                  constants.EXPECTED_FPG,
-                                  constants.EXPECTED_VFS)
+            mock.call.update_access(constants.EXPECTED_PROJECT_ID,
+                                    constants.EXPECTED_SHARE_ID,
+                                    constants.NFS,
+                                    constants.EXPECTED_EXTRA_SPECS,
+                                    [constants.ACCESS_RULE_NFS],
+                                    [],
+                                    [constants.DELETE_RULE_IP],
+                                    constants.EXPECTED_FPG,
+                                    constants.EXPECTED_VFS)
         ]
         self.mock_mediator.assert_has_calls(expected_calls)
 
@@ -510,7 +687,9 @@ class HPE3ParDriverTestCase(test.TestCase):
         old_size = constants.NFS_SHARE_INFO['size']
         new_size = old_size * 2
 
-        self.driver.extend_share(constants.NFS_SHARE_INFO, new_size)
+        share_server = None
+        self.driver.extend_share(constants.NFS_SHARE_INFO,
+                                 new_size, share_server)
 
         self.mock_mediator.resize_share.assert_called_once_with(
             constants.EXPECTED_PROJECT_ID,
@@ -526,8 +705,9 @@ class HPE3ParDriverTestCase(test.TestCase):
 
         old_size = constants.NFS_SHARE_INFO['size']
         new_size = old_size / 2
-
-        self.driver.shrink_share(constants.NFS_SHARE_INFO, new_size)
+        share_server = None
+        self.driver.shrink_share(constants.NFS_SHARE_INFO,
+                                 new_size, share_server)
 
         self.mock_mediator.resize_share.assert_called_once_with(
             constants.EXPECTED_PROJECT_ID,
@@ -553,12 +733,18 @@ class HPE3ParDriverTestCase(test.TestCase):
             'provisioned_capacity_gb': 0,
             'share_backend_name': 'HPE_3PAR',
             'snapshot_support': True,
+            'create_share_from_snapshot_support': True,
+            'revert_to_snapshot_support': False,
+            'share_group_snapshot_support': True,
+            'mount_snapshot_support': False,
             'storage_protocol': 'NFS_CIFS',
             'thin_provisioning': True,
             'total_capacity_gb': 0,
             'vendor_name': 'HPE',
             'pools': None,
             'replication_domain': None,
+            'filter_function': None,
+            'goodness_function': None,
         }
 
         result = self.driver.get_share_stats(refresh=True)
@@ -590,33 +776,48 @@ class HPE3ParDriverTestCase(test.TestCase):
         expected_version = self.driver.VERSION
 
         self.mock_mediator.get_fpg_status.return_value = {
-            'free_capacity_gb': expected_free,
+            'pool_name': constants.EXPECTED_FPG,
             'total_capacity_gb': expected_capacity,
+            'free_capacity_gb': expected_free,
             'thin_provisioning': True,
             'dedupe': False,
             'hpe3par_flash_cache': False,
             'hp3par_flash_cache': False,
+            'reserved_percentage': 0,
+            'provisioned_capacity_gb': expected_capacity
         }
 
         expected_result = {
-            'driver_handles_share_servers': True,
-            'qos': False,
+            'share_backend_name': 'HPE_3PAR',
+            'vendor_name': 'HPE',
             'driver_version': expected_version,
-            'free_capacity_gb': expected_free,
-            'max_over_subscription_ratio': None,
-            'pools': None,
+            'storage_protocol': 'NFS_CIFS',
+            'driver_handles_share_servers': True,
+            'total_capacity_gb': 0,
+            'free_capacity_gb': 0,
             'provisioned_capacity_gb': 0,
             'reserved_percentage': 0,
-            'share_backend_name': 'HPE_3PAR',
-            'storage_protocol': 'NFS_CIFS',
-            'total_capacity_gb': expected_capacity,
-            'vendor_name': 'HPE',
+            'max_over_subscription_ratio': None,
+            'qos': False,
             'thin_provisioning': True,
-            'dedupe': False,
-            'hpe3par_flash_cache': False,
-            'hp3par_flash_cache': False,
+            'pools': [{
+                'pool_name': constants.EXPECTED_FPG,
+                'total_capacity_gb': expected_capacity,
+                'free_capacity_gb': expected_free,
+                'thin_provisioning': True,
+                'dedupe': False,
+                'hpe3par_flash_cache': False,
+                'hp3par_flash_cache': False,
+                'reserved_percentage': 0,
+                'provisioned_capacity_gb': expected_capacity}],
             'snapshot_support': True,
+            'create_share_from_snapshot_support': True,
+            'revert_to_snapshot_support': False,
+            'share_group_snapshot_support': True,
+            'mount_snapshot_support': False,
             'replication_domain': None,
+            'filter_function': None,
+            'goodness_function': None,
         }
 
         result = self.driver.get_share_stats(refresh=True)
@@ -650,7 +851,13 @@ class HPE3ParDriverTestCase(test.TestCase):
             'total_capacity_gb': 0,
             'vendor_name': 'HPE',
             'snapshot_support': True,
+            'create_share_from_snapshot_support': True,
+            'revert_to_snapshot_support': False,
+            'share_group_snapshot_support': True,
+            'mount_snapshot_support': False,
             'replication_domain': None,
+            'filter_function': None,
+            'goodness_function': None,
         }
 
         result = self.driver.get_share_stats(refresh=True)
@@ -691,27 +898,6 @@ class HPE3ParDriverTestCase(test.TestCase):
     def test_get_network_allocations_number(self):
         self.assertEqual(1, self.driver.get_network_allocations_number())
 
-    def test_build_export_location_bad_protocol(self):
-        self.assertRaises(exception.InvalidInput,
-                          self.driver._build_export_location,
-                          "BOGUS",
-                          constants.EXPECTED_IP_1234,
-                          constants.EXPECTED_SHARE_PATH)
-
-    def test_build_export_location_bad_ip(self):
-        self.assertRaises(exception.InvalidInput,
-                          self.driver._build_export_location,
-                          constants.NFS,
-                          None,
-                          None)
-
-    def test_build_export_location_bad_path(self):
-        self.assertRaises(exception.InvalidInput,
-                          self.driver._build_export_location,
-                          constants.NFS,
-                          constants.EXPECTED_IP_1234,
-                          None)
-
     def test_setup_server(self):
         """Setup server by creating a new FSIP."""
 
@@ -736,8 +922,8 @@ class HPE3ParDriverTestCase(test.TestCase):
             'fpg': constants.EXPECTED_FPG,
             'vfs': constants.EXPECTED_VFS,
         }
-
-        result = self.driver._setup_server(network_info)
+        metadata = {'request_host': constants.EXPECTED_HOST}
+        result = self.driver._setup_server(network_info, metadata)
 
         expected_calls = [
             mock.call.create_fsip(constants.EXPECTED_IP_1234,
@@ -750,12 +936,60 @@ class HPE3ParDriverTestCase(test.TestCase):
 
         self.assertEqual(expected_result, result)
 
+    def test_setup_server_fails_for_unsupported_network_type(self):
+        """Setup server fails for unsupported network type"""
+
+        self.init_driver()
+
+        network_info = {
+            'network_allocations': [
+                {'ip_address': constants.EXPECTED_IP_1234}],
+            'cidr': '/'.join((constants.EXPECTED_IP_1234,
+                              constants.CIDR_PREFIX)),
+            'network_type': constants.EXPECTED_VXLAN_TYPE,
+            'segmentation_id': constants.EXPECTED_VLAN_TAG,
+            'server_id': constants.EXPECTED_SERVER_ID,
+        }
+        metadata = {'request_host': constants.EXPECTED_HOST}
+
+        self.assertRaises(exception.NetworkBadConfigurationException,
+                          self.driver._setup_server,
+                          network_info, metadata)
+
+    def test_setup_server_fails_for_exceed_pool_max_supported_ips(self):
+        """Setup server fails when the VFS has reached max supported IPs"""
+
+        self.init_driver()
+
+        network_info = {
+            'network_allocations': [
+                {'ip_address': constants.EXPECTED_IP_1234}],
+            'cidr': '/'.join((constants.EXPECTED_IP_1234,
+                              constants.CIDR_PREFIX)),
+            'network_type': constants.EXPECTED_VLAN_TYPE,
+            'segmentation_id': constants.EXPECTED_VLAN_TAG,
+            'server_id': constants.EXPECTED_SERVER_ID,
+        }
+        metadata = {'request_host': constants.EXPECTED_HOST}
+
+        expected_vfs = self.driver.fpgs[
+            constants.EXPECTED_FPG][constants.EXPECTED_VFS]
+        self.driver.fpgs[constants.EXPECTED_FPG][constants.EXPECTED_VFS] = [
+            '10.0.0.1', '10.0.0.2', '10.0.0.3', '10.0.0.4']
+
+        self.assertRaises(exception.Invalid,
+                          self.driver._setup_server,
+                          network_info, metadata)
+        self.driver.fpgs[constants.EXPECTED_FPG][constants.EXPECTED_VFS
+                                                 ] = expected_vfs
+
     def test_teardown_server(self):
+        """Test tear down server"""
 
         self.init_driver()
 
         server_details = {
-            'ip': constants.EXPECTED_IP_1234,
+            'ip': constants.EXPECTED_IP_10203040,
             'fpg': constants.EXPECTED_FPG,
             'vfs': constants.EXPECTED_VFS,
         }
@@ -763,7 +997,7 @@ class HPE3ParDriverTestCase(test.TestCase):
         self.driver._teardown_server(server_details)
 
         expected_calls = [
-            mock.call.remove_fsip(constants.EXPECTED_IP_1234,
+            mock.call.remove_fsip(constants.EXPECTED_IP_10203040,
                                   constants.EXPECTED_FPG,
                                   constants.EXPECTED_VFS)
         ]
