@@ -32,17 +32,21 @@ REMOTE_CLUSTER_NAME = 'fake_cluster_2'
 CLUSTER_ADDRESS_1 = 'fake_cluster_address'
 CLUSTER_ADDRESS_2 = 'fake_cluster_address_2'
 VERSION = 'NetApp Release 8.2.1 Cluster-Mode: Fri Mar 21 14:25:07 PDT 2014'
-NODE_NAME = 'fake_node'
+NODE_NAME = 'fake_node1'
+NODE_NAMES = ('fake_node1', 'fake_node2')
 VSERVER_NAME = 'fake_vserver'
 VSERVER_NAME_2 = 'fake_vserver_2'
 ADMIN_VSERVER_NAME = 'fake_admin_vserver'
 NODE_VSERVER_NAME = 'fake_node_vserver'
+NFS_VERSIONS = ['nfs3', 'nfs4.0']
+ROOT_AGGREGATE_NAMES = ('root_aggr1', 'root_aggr2')
 ROOT_VOLUME_AGGREGATE_NAME = 'fake_root_aggr'
 ROOT_VOLUME_NAME = 'fake_root_volume'
 SHARE_AGGREGATE_NAME = 'fake_aggr1'
 SHARE_AGGREGATE_NAMES = ('fake_aggr1', 'fake_aggr2')
 SHARE_AGGREGATE_RAID_TYPES = ('raid4', 'raid_dp')
 SHARE_AGGREGATE_DISK_TYPE = 'FCAL'
+SHARE_AGGREGATE_DISK_TYPES = ['SATA', 'SSD']
 SHARE_NAME = 'fake_share'
 SHARE_SIZE = '1000000000'
 SHARE_NAME_2 = 'fake_share_2'
@@ -82,6 +86,15 @@ SM_SOURCE_VOLUME = 'fake_source_volume'
 SM_DEST_VSERVER = 'fake_destination_vserver'
 SM_DEST_VOLUME = 'fake_destination_volume'
 
+NETWORK_INTERFACES = [{
+    'interface_name': 'fake_interface',
+    'address': IP_ADDRESS,
+    'vserver': VSERVER_NAME,
+    'netmask': NETMASK,
+    'role': 'data',
+    'home-node': NODE_NAME,
+    'home-port': VLAN_PORT
+}]
 
 IPSPACES = [{
     'uuid': 'fake_uuid',
@@ -780,34 +793,21 @@ AGGR_GET_NAMES_RESPONSE = etree.XML("""
     <attributes-list>
       <aggr-attributes>
         <aggr-raid-attributes>
-          <plexes>
-            <plex-attributes>
-              <plex-name>/%(aggr1)s/plex0</plex-name>
-              <raidgroups>
-                <raidgroup-attributes>
-                  <raidgroup-name>/%(aggr1)s/plex0/rg0</raidgroup-name>
-                </raidgroup-attributes>
-              </raidgroups>
-            </plex-attributes>
-          </plexes>
+        </aggr-raid-attributes>
+        <aggregate-name>%(root1)s</aggregate-name>
+      </aggr-attributes>
+      <aggr-attributes>
+        <aggr-raid-attributes>
+        </aggr-raid-attributes>
+        <aggregate-name>%(root2)s</aggregate-name>
+      </aggr-attributes>
+      <aggr-attributes>
+        <aggr-raid-attributes>
         </aggr-raid-attributes>
         <aggregate-name>%(aggr1)s</aggregate-name>
       </aggr-attributes>
       <aggr-attributes>
         <aggr-raid-attributes>
-          <plexes>
-            <plex-attributes>
-              <plex-name>/%(aggr2)s/plex0</plex-name>
-              <raidgroups>
-                <raidgroup-attributes>
-                  <raidgroup-name>/%(aggr2)s/plex0/rg0</raidgroup-name>
-                </raidgroup-attributes>
-                <raidgroup-attributes>
-                  <raidgroup-name>/%(aggr2)s/plex0/rg1</raidgroup-name>
-                </raidgroup-attributes>
-              </raidgroups>
-            </plex-attributes>
-          </plexes>
         </aggr-raid-attributes>
         <aggregate-name>%(aggr2)s</aggregate-name>
       </aggr-attributes>
@@ -815,6 +815,8 @@ AGGR_GET_NAMES_RESPONSE = etree.XML("""
     <num-records>2</num-records>
   </results>
 """ % {
+    'root1': ROOT_AGGREGATE_NAMES[0],
+    'root2': ROOT_AGGREGATE_NAMES[1],
     'aggr1': SHARE_AGGREGATE_NAMES[0],
     'aggr2': SHARE_AGGREGATE_NAMES[1],
 })
@@ -1136,6 +1138,193 @@ AGGR_GET_ITER_RESPONSE = etree.XML("""
     'aggr2': SHARE_AGGREGATE_NAMES[1],
 })
 
+AGGR_GET_ITER_SSC_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <aggr-attributes>
+        <aggr-64bit-upgrade-attributes>
+          <aggr-status-attributes>
+            <is-64-bit-upgrade-in-progress>false</is-64-bit-upgrade-in-progress>
+          </aggr-status-attributes>
+        </aggr-64bit-upgrade-attributes>
+        <aggr-fs-attributes>
+          <block-type>64_bit</block-type>
+          <fsid>1758646411</fsid>
+          <type>aggr</type>
+        </aggr-fs-attributes>
+        <aggr-inode-attributes>
+          <files-private-used>512</files-private-used>
+          <files-total>30384</files-total>
+          <files-used>96</files-used>
+          <inodefile-private-capacity>30384</inodefile-private-capacity>
+          <inodefile-public-capacity>30384</inodefile-public-capacity>
+          <maxfiles-available>30384</maxfiles-available>
+          <maxfiles-possible>243191</maxfiles-possible>
+          <maxfiles-used>96</maxfiles-used>
+          <percent-inode-used-capacity>0</percent-inode-used-capacity>
+        </aggr-inode-attributes>
+        <aggr-ownership-attributes>
+          <home-id>4082368507</home-id>
+          <home-name>cluster3-01</home-name>
+          <owner-id>4082368507</owner-id>
+          <owner-name>cluster3-01</owner-name>
+        </aggr-ownership-attributes>
+        <aggr-performance-attributes>
+          <free-space-realloc>off</free-space-realloc>
+          <max-write-alloc-blocks>0</max-write-alloc-blocks>
+        </aggr-performance-attributes>
+        <aggr-raid-attributes>
+          <checksum-status>active</checksum-status>
+          <checksum-style>block</checksum-style>
+          <disk-count>3</disk-count>
+          <ha-policy>cfo</ha-policy>
+          <has-local-root>true</has-local-root>
+          <has-partner-root>false</has-partner-root>
+          <is-checksum-enabled>true</is-checksum-enabled>
+          <is-hybrid>false</is-hybrid>
+          <is-hybrid-enabled>false</is-hybrid-enabled>
+          <is-inconsistent>false</is-inconsistent>
+          <mirror-status>unmirrored</mirror-status>
+          <mount-state>online</mount-state>
+          <plex-count>1</plex-count>
+          <plexes>
+            <plex-attributes>
+              <is-online>true</is-online>
+              <is-resyncing>false</is-resyncing>
+              <plex-name>/%(aggr1)s/plex0</plex-name>
+              <plex-status>normal,active</plex-status>
+              <raidgroups>
+                <raidgroup-attributes>
+                  <checksum-style>block</checksum-style>
+                  <is-cache-tier>false</is-cache-tier>
+                  <is-recomputing-parity>false</is-recomputing-parity>
+                  <is-reconstructing>false</is-reconstructing>
+                  <raidgroup-name>/%(aggr1)s/plex0/rg0</raidgroup-name>
+                  <recomputing-parity-percentage>0</recomputing-parity-percentage>
+                  <reconstruction-percentage>0</reconstruction-percentage>
+                </raidgroup-attributes>
+              </raidgroups>
+              <resyncing-percentage>0</resyncing-percentage>
+            </plex-attributes>
+          </plexes>
+          <raid-lost-write-state>on</raid-lost-write-state>
+          <raid-size>16</raid-size>
+          <raid-status>raid_dp, normal</raid-status>
+          <raid-type>raid_dp</raid-type>
+          <state>online</state>
+        </aggr-raid-attributes>
+        <aggr-snaplock-attributes>
+          <is-snaplock>false</is-snaplock>
+        </aggr-snaplock-attributes>
+        <aggr-snapshot-attributes>
+          <files-total>0</files-total>
+          <files-used>0</files-used>
+          <is-snapshot-auto-create-enabled>true</is-snapshot-auto-create-enabled>
+          <is-snapshot-auto-delete-enabled>true</is-snapshot-auto-delete-enabled>
+          <maxfiles-available>0</maxfiles-available>
+          <maxfiles-possible>0</maxfiles-possible>
+          <maxfiles-used>0</maxfiles-used>
+          <percent-inode-used-capacity>0</percent-inode-used-capacity>
+          <percent-used-capacity>0</percent-used-capacity>
+          <size-available>0</size-available>
+          <size-total>0</size-total>
+          <size-used>0</size-used>
+          <snapshot-reserve-percent>0</snapshot-reserve-percent>
+        </aggr-snapshot-attributes>
+        <aggr-space-attributes>
+          <aggregate-metadata>245760</aggregate-metadata>
+          <hybrid-cache-size-total>0</hybrid-cache-size-total>
+          <percent-used-capacity>95</percent-used-capacity>
+          <size-available>45670400</size-available>
+          <size-total>943718400</size-total>
+          <size-used>898048000</size-used>
+          <total-reserved-space>0</total-reserved-space>
+          <used-including-snapshot-reserve>898048000</used-including-snapshot-reserve>
+          <volume-footprints>897802240</volume-footprints>
+        </aggr-space-attributes>
+        <aggr-volume-count-attributes>
+          <flexvol-count>1</flexvol-count>
+          <flexvol-count-collective>0</flexvol-count-collective>
+          <flexvol-count-striped>0</flexvol-count-striped>
+        </aggr-volume-count-attributes>
+        <aggregate-name>%(aggr1)s</aggregate-name>
+        <aggregate-uuid>15863632-ea49-49a8-9c88-2bd2d57c6d7a</aggregate-uuid>
+        <nodes>
+          <node-name>cluster3-01</node-name>
+        </nodes>
+        <striping-type>unknown</striping-type>
+      </aggr-attributes>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""" % {'aggr1': SHARE_AGGREGATE_NAMES[0]})
+
+AGGR_GET_ITER_ROOT_AGGR_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <aggr-attributes>
+        <aggr-raid-attributes>
+          <has-local-root>true</has-local-root>
+          <has-partner-root>false</has-partner-root>
+        </aggr-raid-attributes>
+        <aggregate-name>%(root1)s</aggregate-name>
+      </aggr-attributes>
+      <aggr-attributes>
+        <aggr-raid-attributes>
+          <has-local-root>true</has-local-root>
+          <has-partner-root>false</has-partner-root>
+        </aggr-raid-attributes>
+        <aggregate-name>%(root2)s</aggregate-name>
+      </aggr-attributes>
+      <aggr-attributes>
+        <aggr-raid-attributes>
+          <has-local-root>false</has-local-root>
+          <has-partner-root>false</has-partner-root>
+        </aggr-raid-attributes>
+        <aggregate-name>%(aggr1)s</aggregate-name>
+      </aggr-attributes>
+      <aggr-attributes>
+        <aggr-raid-attributes>
+          <has-local-root>false</has-local-root>
+          <has-partner-root>false</has-partner-root>
+        </aggr-raid-attributes>
+        <aggregate-name>%(aggr2)s</aggregate-name>
+      </aggr-attributes>
+    </attributes-list>
+    <num-records>6</num-records>
+  </results>
+""" % {
+    'root1': ROOT_AGGREGATE_NAMES[0],
+    'root2': ROOT_AGGREGATE_NAMES[1],
+    'aggr1': SHARE_AGGREGATE_NAMES[0],
+    'aggr2': SHARE_AGGREGATE_NAMES[1],
+})
+
+AGGR_GET_ITER_NON_ROOT_AGGR_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <aggr-attributes>
+        <aggr-raid-attributes>
+          <has-local-root>false</has-local-root>
+          <has-partner-root>false</has-partner-root>
+        </aggr-raid-attributes>
+        <aggregate-name>%(aggr1)s</aggregate-name>
+      </aggr-attributes>
+      <aggr-attributes>
+        <aggr-raid-attributes>
+          <has-local-root>false</has-local-root>
+          <has-partner-root>false</has-partner-root>
+        </aggr-raid-attributes>
+        <aggregate-name>%(aggr2)s</aggregate-name>
+      </aggr-attributes>
+    </attributes-list>
+    <num-records>6</num-records>
+  </results>
+""" % {
+    'aggr1': SHARE_AGGREGATE_NAMES[0],
+    'aggr2': SHARE_AGGREGATE_NAMES[1],
+})
+
 VOLUME_GET_NAME_RESPONSE = etree.XML("""
   <results status="passed">
     <attributes-list>
@@ -1206,10 +1395,12 @@ VOLUME_MODIFY_ITER_ERROR_RESPONSE = etree.XML("""
   </results>
 """ % {'volume': SHARE_NAME, 'vserver': VSERVER_NAME})
 
+SNAPSHOT_ACCESS_TIME = '1466640058'
 SNAPSHOT_GET_ITER_NOT_BUSY_RESPONSE = etree.XML("""
   <results status="passed">
     <attributes-list>
       <snapshot-info>
+        <access-time>%(access_time)s</access-time>
         <busy>false</busy>
         <name>%(snap)s</name>
         <volume>%(volume)s</volume>
@@ -1218,12 +1409,18 @@ SNAPSHOT_GET_ITER_NOT_BUSY_RESPONSE = etree.XML("""
     </attributes-list>
     <num-records>1</num-records>
   </results>
-""" % {'snap': SNAPSHOT_NAME, 'volume': SHARE_NAME, 'vserver': VSERVER_NAME})
+""" % {
+    'access_time': SNAPSHOT_ACCESS_TIME,
+    'snap': SNAPSHOT_NAME,
+    'volume': SHARE_NAME,
+    'vserver': VSERVER_NAME,
+})
 
 SNAPSHOT_GET_ITER_BUSY_RESPONSE = etree.XML("""
   <results status="passed">
     <attributes-list>
       <snapshot-info>
+        <access-time>%(access_time)s</access-time>
         <busy>true</busy>
         <name>%(snap)s</name>
         <volume>%(volume)s</volume>
@@ -1237,7 +1434,12 @@ SNAPSHOT_GET_ITER_BUSY_RESPONSE = etree.XML("""
     </attributes-list>
     <num-records>1</num-records>
   </results>
-""" % {'snap': SNAPSHOT_NAME, 'volume': SHARE_NAME, 'vserver': VSERVER_NAME})
+""" % {
+    'access_time': SNAPSHOT_ACCESS_TIME,
+    'snap': SNAPSHOT_NAME,
+    'volume': SHARE_NAME,
+    'vserver': VSERVER_NAME,
+})
 
 SNAPSHOT_GET_ITER_NOT_UNIQUE_RESPONSE = etree.XML("""
   <results status="passed">
@@ -1324,6 +1526,24 @@ SNAPSHOT_GET_ITER_DELETED_RESPONSE = etree.XML("""
     'vserver': VSERVER_NAME,
 })
 
+SNAPSHOT_GET_ITER_SNAPMIRROR_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <snapshot-info>
+        <name>%(snap)s</name>
+        <volume>%(volume)s</volume>
+        <vserver>%(vserver)s</vserver>
+      </snapshot-info>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+
+""" % {
+    'snap': SNAPSHOT_NAME,
+    'volume': SHARE_NAME,
+    'vserver': VSERVER_NAME,
+})
+
 CIFS_SHARE_ACCESS_CONTROL_GET_ITER = etree.XML("""
   <results status="passed">
     <attributes-list>
@@ -1467,13 +1687,34 @@ STORAGE_DISK_GET_ITER_RESPONSE = etree.XML("""
       <storage-disk-info>
         <disk-name>cluster3-01:v5.19</disk-name>
         <disk-raid-info>
-          <effective-disk-type>%s</effective-disk-type>
+          <effective-disk-type>%(type0)s</effective-disk-type>
+        </disk-raid-info>
+      </storage-disk-info>
+      <storage-disk-info>
+        <disk-name>cluster3-01:v5.20</disk-name>
+        <disk-raid-info>
+          <effective-disk-type>%(type0)s</effective-disk-type>
+        </disk-raid-info>
+      </storage-disk-info>
+      <storage-disk-info>
+        <disk-name>cluster3-01:v5.20</disk-name>
+        <disk-raid-info>
+          <effective-disk-type>%(type1)s</effective-disk-type>
+        </disk-raid-info>
+      </storage-disk-info>
+      <storage-disk-info>
+        <disk-name>cluster3-01:v5.20</disk-name>
+        <disk-raid-info>
+          <effective-disk-type>%(type1)s</effective-disk-type>
         </disk-raid-info>
       </storage-disk-info>
     </attributes-list>
-    <num-records>1</num-records>
+    <num-records>4</num-records>
   </results>
-""" % SHARE_AGGREGATE_DISK_TYPE)
+""" % {
+    'type0': SHARE_AGGREGATE_DISK_TYPES[0],
+    'type1': SHARE_AGGREGATE_DISK_TYPES[1],
+})
 
 STORAGE_DISK_GET_ITER_RESPONSE_PAGE_1 = etree.XML("""
   <results status="passed">
@@ -1683,6 +1924,27 @@ LUN_GET_ITER_RESPONSE = etree.XML("""
     'volume': SHARE_NAME,
 })
 
+VOLUME_GET_ITER_NOT_UNIQUE_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <volume-attributes>
+        <volume-id-attributes>
+          <name>%(volume1)s</name>
+        </volume-id-attributes>
+      </volume-attributes>
+      <volume-attributes>
+        <volume-id-attributes>
+          <name>%(volume2)s</name>
+        </volume-id-attributes>
+      </volume-attributes>
+    </attributes-list>
+    <num-records>2</num-records>
+  </results>
+""" % {
+    'volume1': SHARE_NAME,
+    'volume2': SHARE_NAME_2,
+})
+
 VOLUME_GET_ITER_JUNCTIONED_VOLUMES_RESPONSE = etree.XML("""
   <results status="passed">
     <attributes-list>
@@ -1884,6 +2146,152 @@ SNAPMIRROR_INITIALIZE_RESULT = etree.XML("""
   </results>
 """)
 
+VOLUME_MOVE_GET_ITER_RESULT = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <volume-move-info>
+        <cutover-action>retry_on_failure</cutover-action>
+        <details>Cutover Completed::Volume move job finishing move</details>
+        <estimated-completion-time>1481919246</estimated-completion-time>
+        <percent-complete>82</percent-complete>
+        <phase>finishing</phase>
+        <state>healthy</state>
+        <volume>%(volume)s</volume>
+        <vserver>%(vserver)s</vserver>
+      </volume-move-info>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""" % {
+    'volume': SHARE_NAME,
+    'vserver': VSERVER_NAME,
+})
+
+PERF_OBJECT_COUNTER_TOTAL_CP_MSECS_LABELS = [
+    'SETUP', 'PRE_P0', 'P0_SNAP_DEL', 'P1_CLEAN', 'P1_QUOTA', 'IPU_DISK_ADD',
+    'P2V_INOFILE', 'P2V_INO_PUB', 'P2V_INO_PRI', 'P2V_FSINFO', 'P2V_DLOG1',
+    'P2V_DLOG2', 'P2V_REFCOUNT', 'P2V_TOPAA', 'P2V_DF_SCORES_SUB', 'P2V_BM',
+    'P2V_SNAP', 'P2V_DF_SCORES', 'P2V_VOLINFO', 'P2V_CONT', 'P2A_INOFILE',
+    'P2A_INO', 'P2A_DLOG1', 'P2A_HYA', 'P2A_DLOG2', 'P2A_FSINFO',
+    'P2A_IPU_BITMAP_GROW', 'P2A_REFCOUNT', 'P2A_TOPAA', 'P2A_HYABC', 'P2A_BM',
+    'P2A_SNAP', 'P2A_VOLINFO', 'P2_FLUSH', 'P2_FINISH', 'P3_WAIT',
+    'P3V_VOLINFO', 'P3A_VOLINFO', 'P3_FINISH', 'P4_FINISH', 'P5_FINISH',
+]
+
+PERF_OBJECT_COUNTER_LIST_INFO_WAFL_RESPONSE = etree.XML("""
+  <results status="passed">
+    <counters>
+      <counter-info>
+        <desc>No. of times 8.3 names are accessed per second.</desc>
+        <name>access_8_3_names</name>
+        <privilege-level>diag</privilege-level>
+        <properties>rate</properties>
+        <unit>per_sec</unit>
+      </counter-info>
+      <counter-info>
+        <desc>Array of counts of different types of CPs</desc>
+        <labels>
+          <label-info>wafl_timer generated CP</label-info>
+          <label-info>snapshot generated CP</label-info>
+          <label-info>wafl_avail_bufs generated CP</label-info>
+          <label-info>dirty_blk_cnt generated CP</label-info>
+          <label-info>full NV-log generated CP,back-to-back CP</label-info>
+          <label-info>flush generated CP,sync generated CP</label-info>
+          <label-info>deferred back-to-back CP</label-info>
+          <label-info>low mbufs generated CP</label-info>
+          <label-info>low datavecs generated CP</label-info>
+          <label-info>nvlog replay takeover time limit CP</label-info>
+        </labels>
+        <name>cp_count</name>
+        <privilege-level>diag</privilege-level>
+        <properties>delta</properties>
+        <type>array</type>
+        <unit>none</unit>
+      </counter-info>
+      <counter-info>
+        <base-counter>total_cp_msecs</base-counter>
+        <desc>Array of percentage time spent in different phases of CP</desc>
+        <labels>
+          <label-info>%(labels)s</label-info>
+        </labels>
+        <name>cp_phase_times</name>
+        <privilege-level>diag</privilege-level>
+        <properties>percent</properties>
+        <type>array</type>
+        <unit>percent</unit>
+      </counter-info>
+    </counters>
+  </results>
+""" % {'labels': ','.join(PERF_OBJECT_COUNTER_TOTAL_CP_MSECS_LABELS)})
+
+PERF_OBJECT_GET_INSTANCES_SYSTEM_RESPONSE_CMODE = etree.XML("""
+  <results status="passed">
+    <instances>
+      <instance-data>
+        <counters>
+          <counter-data>
+            <name>avg_processor_busy</name>
+            <value>5674745133134</value>
+          </counter-data>
+        </counters>
+        <name>system</name>
+        <uuid>%(node1)s:kernel:system</uuid>
+      </instance-data>
+      <instance-data>
+        <counters>
+          <counter-data>
+            <name>avg_processor_busy</name>
+            <value>4077649009234</value>
+          </counter-data>
+        </counters>
+        <name>system</name>
+        <uuid>%(node2)s:kernel:system</uuid>
+      </instance-data>
+    </instances>
+    <timestamp>1453412013</timestamp>
+  </results>
+""" % {'node1': NODE_NAMES[0], 'node2': NODE_NAMES[1]})
+
+PERF_OBJECT_GET_INSTANCES_SYSTEM_RESPONSE_7MODE = etree.XML("""
+  <results status="passed">
+    <timestamp>1454146292</timestamp>
+    <instances>
+      <instance-data>
+        <name>system</name>
+        <counters>
+          <counter-data>
+            <name>avg_processor_busy</name>
+            <value>13215732322</value>
+          </counter-data>
+        </counters>
+      </instance-data>
+    </instances>
+  </results>""")
+
+PERF_OBJECT_INSTANCE_LIST_INFO_ITER_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <instance-info>
+        <name>system</name>
+        <uuid>%(node)s:kernel:system</uuid>
+      </instance-info>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""" % {'node': NODE_NAME})
+
+PERF_OBJECT_INSTANCE_LIST_INFO_RESPONSE = etree.XML("""
+  <results status="passed">
+    <instances>
+      <instance-info>
+        <name>processor0</name>
+      </instance-info>
+      <instance-info>
+        <name>processor1</name>
+      </instance-info>
+    </instances>
+  </results>""")
+
 FAKE_VOL_XML = """<volume-info xmlns='http://www.netapp.com/filer/admin'>
     <name>open123</name>
     <state>online</state>
@@ -1955,3 +2363,12 @@ FAKE_RESULT_SUCCESS = api.NaElement('result')
 FAKE_RESULT_SUCCESS.add_attr('status', 'passed')
 
 FAKE_HTTP_OPENER = urllib.request.build_opener()
+
+FAKE_MANAGE_VOLUME = {
+    'aggregate': SHARE_AGGREGATE_NAME,
+    'name': SHARE_NAME,
+    'owning-vserver-name': VSERVER_NAME,
+    'junction_path': VOLUME_JUNCTION_PATH,
+    'style': 'fake_style',
+    'size': SHARE_SIZE,
+}
