@@ -28,6 +28,7 @@ from manila.share.drivers.netapp.dataontap.cluster_mode import lib_base
 from manila.share.drivers.netapp.dataontap.cluster_mode import lib_multi_svm
 from manila.share.drivers.netapp import utils as na_utils
 from manila import test
+from manila.tests.share.drivers.netapp.dataontap.client import fakes as c_fake
 from manila.tests.share.drivers.netapp.dataontap import fakes as fake
 
 
@@ -170,6 +171,22 @@ class NetAppFileStorageLibraryTestCase(test.TestCase):
 
         self.assertTupleEqual((fake.VSERVER1, 'fake_client'), result)
 
+    def test_get_ems_pool_info(self):
+
+        self.mock_object(self.library,
+                         '_find_matching_aggregates',
+                         mock.Mock(return_value=['aggr1', 'aggr2']))
+
+        result = self.library._get_ems_pool_info()
+
+        expected = {
+            'pools': {
+                'vserver': None,
+                'aggregates': ['aggr1', 'aggr2'],
+            },
+        }
+        self.assertEqual(expected, result)
+
     def test_handle_housekeeping_tasks(self):
 
         self.mock_object(self.client, 'prune_deleted_nfs_export_policies')
@@ -185,14 +202,16 @@ class NetAppFileStorageLibraryTestCase(test.TestCase):
 
     def test_find_matching_aggregates(self):
 
-        self.mock_object(self.client,
-                         'list_aggregates',
-                         mock.Mock(return_value=fake.AGGREGATES))
-
+        mock_list_non_root_aggregates = self.mock_object(
+            self.client, 'list_non_root_aggregates',
+            mock.Mock(return_value=fake.AGGREGATES))
         self.library.configuration.netapp_aggregate_name_search_pattern = (
             '.*_aggr_1')
+
         result = self.library._find_matching_aggregates()
+
         self.assertListEqual([fake.AGGREGATES[0]], result)
+        mock_list_non_root_aggregates.assert_called_once_with()
 
     def test_setup_server(self):
 
@@ -272,6 +291,8 @@ class NetAppFileStorageLibraryTestCase(test.TestCase):
 
     def test_create_vserver(self):
 
+        versions = ['fake_v1', 'fake_v2']
+        self.library.configuration.netapp_enabled_share_protocols = versions
         vserver_id = fake.NETWORK_INFO['server_id']
         vserver_name = fake.VSERVER_NAME_TEMPLATE % vserver_id
         vserver_client = mock.Mock()
@@ -305,7 +326,7 @@ class NetAppFileStorageLibraryTestCase(test.TestCase):
             vserver_name, vserver_client, fake.NETWORK_INFO, fake.IPSPACE)
         self.library._create_vserver_admin_lif.assert_called_with(
             vserver_name, vserver_client, fake.NETWORK_INFO, fake.IPSPACE)
-        self.assertTrue(vserver_client.enable_nfs.called)
+        vserver_client.enable_nfs.assert_called_once_with(versions)
         self.library._client.setup_security_services.assert_called_with(
             fake.NETWORK_INFO['security_services'], vserver_client,
             vserver_name)
@@ -399,17 +420,6 @@ class NetAppFileStorageLibraryTestCase(test.TestCase):
 
         self.assertEqual('Default', result)
 
-    def test_create_ipspace_not_neutron(self):
-
-        self.library._client.features.IPSPACES = True
-        network_info = copy.deepcopy(fake.NETWORK_INFO)
-        network_info['neutron_subnet_id'] = None
-        network_info['nova_net_id'] = 'fake_nova_net_id'
-
-        result = self.library._create_ipspace(network_info)
-
-        self.assertEqual('Default', result)
-
     def test_create_ipspace_already_present(self):
 
         self.library._client.features.IPSPACES = True
@@ -537,7 +547,19 @@ class NetAppFileStorageLibraryTestCase(test.TestCase):
 
         self.assertEqual('os_132dbb10-9a36-46f2-8d89-3d909830c356', result)
 
-    def test_create_lif(self):
+    @ddt.data(fake.MTU, None, 'not-present')
+    def test_create_lif(self, mtu):
+        """Tests cases where MTU is a valid value, None or not present."""
+
+        expected_mtu = (mtu if mtu not in (None, 'not-present') else
+                        fake.DEFAULT_MTU)
+
+        network_allocations = copy.deepcopy(
+            fake.NETWORK_INFO['network_allocations'][0])
+        network_allocations['mtu'] = mtu
+
+        if mtu == 'not-present':
+            network_allocations.pop('mtu')
 
         vserver_client = mock.Mock()
         vserver_client.network_interface_exists = mock.Mock(
@@ -551,12 +573,12 @@ class NetAppFileStorageLibraryTestCase(test.TestCase):
                                  'fake_ipspace',
                                  'fake_node',
                                  'fake_lif',
-                                 fake.NETWORK_INFO['network_allocations'][0])
+                                 network_allocations)
 
         self.library._client.create_network_interface.assert_has_calls([
             mock.call('10.10.10.10', '255.255.255.0', '1000', 'fake_node',
                       'fake_port', 'fake_vserver', 'fake_lif',
-                      'fake_ipspace')])
+                      'fake_ipspace', expected_mtu)])
 
     def test_create_lif_if_nonexistent_already_present(self):
 
@@ -649,6 +671,11 @@ class NetAppFileStorageLibraryTestCase(test.TestCase):
         self.mock_object(self.library,
                          '_get_api_client',
                          mock.Mock(return_value=vserver_client))
+        mock_delete_vserver_vlan = self.mock_object(self.library,
+                                                    '_delete_vserver_vlan')
+        self.mock_object(vserver_client,
+                         'get_network_interfaces',
+                         mock.Mock(return_value=c_fake.NETWORK_INTERFACES))
         security_services = fake.NETWORK_INFO['security_services']
 
         self.library._delete_vserver(fake.VSERVER1,
@@ -659,6 +686,8 @@ class NetAppFileStorageLibraryTestCase(test.TestCase):
         self.library._client.delete_vserver.assert_called_once_with(
             fake.VSERVER1, vserver_client, security_services=security_services)
         self.assertFalse(self.library._client.delete_ipspace.called)
+        mock_delete_vserver_vlan.assert_called_once_with(
+            c_fake.NETWORK_INTERFACES)
 
     def test_delete_vserver_ipspace_has_data_vservers(self):
 
@@ -672,6 +701,11 @@ class NetAppFileStorageLibraryTestCase(test.TestCase):
         self.mock_object(self.library._client,
                          'ipspace_has_data_vservers',
                          mock.Mock(return_value=True))
+        mock_delete_vserver_vlan = self.mock_object(self.library,
+                                                    '_delete_vserver_vlan')
+        self.mock_object(vserver_client,
+                         'get_network_interfaces',
+                         mock.Mock(return_value=c_fake.NETWORK_INTERFACES))
         security_services = fake.NETWORK_INFO['security_services']
 
         self.library._delete_vserver(fake.VSERVER1,
@@ -682,8 +716,11 @@ class NetAppFileStorageLibraryTestCase(test.TestCase):
         self.library._client.delete_vserver.assert_called_once_with(
             fake.VSERVER1, vserver_client, security_services=security_services)
         self.assertFalse(self.library._client.delete_ipspace.called)
+        mock_delete_vserver_vlan.assert_called_once_with(
+            c_fake.NETWORK_INTERFACES)
 
-    def test_delete_vserver_with_ipspace(self):
+    @ddt.data([], c_fake.NETWORK_INTERFACES)
+    def test_delete_vserver_with_ipspace(self, interfaces):
 
         self.mock_object(self.library._client,
                          'get_vserver_ipspace',
@@ -695,6 +732,12 @@ class NetAppFileStorageLibraryTestCase(test.TestCase):
         self.mock_object(self.library._client,
                          'ipspace_has_data_vservers',
                          mock.Mock(return_value=False))
+        mock_delete_vserver_vlan = self.mock_object(self.library,
+                                                    '_delete_vserver_vlan')
+        self.mock_object(vserver_client,
+                         'get_network_interfaces',
+                         mock.Mock(return_value=interfaces))
+
         security_services = fake.NETWORK_INFO['security_services']
 
         self.library._delete_vserver(fake.VSERVER1,
@@ -706,3 +749,31 @@ class NetAppFileStorageLibraryTestCase(test.TestCase):
             fake.VSERVER1, vserver_client, security_services=security_services)
         self.library._client.delete_ipspace.assert_called_once_with(
             fake.IPSPACE)
+        mock_delete_vserver_vlan.assert_called_once_with(interfaces)
+
+    def test_delete_vserver_vlan(self):
+
+        self.library._delete_vserver_vlan(c_fake.NETWORK_INTERFACES)
+        for interface in c_fake.NETWORK_INTERFACES:
+            home_port = interface['home-port']
+            port, vlan = home_port.split('-')
+            node = interface['home-node']
+            self.library._client.delete_vlan.assert_called_once_with(
+                node, port, vlan)
+
+    def test_delete_vserver_vlan_client_error(self):
+
+        mock_exception_log = self.mock_object(lib_multi_svm.LOG, 'exception')
+        self.mock_object(
+            self.library._client,
+            'delete_vlan',
+            mock.Mock(side_effect=exception.NetAppException("fake error")))
+
+        self.library._delete_vserver_vlan(c_fake.NETWORK_INTERFACES)
+        for interface in c_fake.NETWORK_INTERFACES:
+            home_port = interface['home-port']
+            port, vlan = home_port.split('-')
+            node = interface['home-node']
+            self.library._client.delete_vlan.assert_called_once_with(
+                node, port, vlan)
+            self.assertEqual(1, mock_exception_log.call_count)
