@@ -54,7 +54,12 @@ def fake_snapshot(**kwargs):
         'name': 'fakesnapshotname',
         'share_proto': 'NFS',
         'export_location': '127.0.0.1:/mnt/nfs/volume-00002',
-        'share': {'size': 1},
+        'share': {
+            'id': 'fakeid',
+            'name': 'fakename',
+            'size': 1,
+            'share_proto': 'NFS',
+        },
     }
     snapshot.update(kwargs)
     return db_fakes.FakeModel(snapshot)
@@ -166,11 +171,12 @@ class LVMShareDriverTestCase(test.TestCase):
         self.assertEqual('/dev/mapper/fake--vg-fake--sharename', ret)
 
     def test_create_share(self):
-        self._helper_nfs.create_export.return_value = 'fakelocation'
+        CONF.set_default('lvm_share_mirrors', 0)
         self._driver._mount_device = mock.Mock()
+
         ret = self._driver.create_share(self._context, self.share,
                                         self.share_server)
-        CONF.set_default('lvm_share_mirrors', 0)
+
         self._driver._mount_device.assert_called_with(
             self.share, '/dev/mapper/fakevg-fakename')
         expected_exec = [
@@ -178,7 +184,7 @@ class LVMShareDriverTestCase(test.TestCase):
             'mkfs.ext4 /dev/mapper/fakevg-fakename',
         ]
         self.assertEqual(expected_exec, fake_utils.fake_execute_get_log())
-        self.assertEqual('fakelocation', ret)
+        self.assertEqual(self._helper_nfs.create_exports.return_value, ret)
 
     def test_create_share_from_snapshot(self):
         CONF.set_default('lvm_share_mirrors', 0)
@@ -200,6 +206,7 @@ class LVMShareDriverTestCase(test.TestCase):
         expected_exec = [
             'lvcreate -L 1G -n fakename fakevg',
             'mkfs.ext4 /dev/mapper/fakevg-fakename',
+            'tune2fs -U random %s' % mount_share,
             ("dd count=0 if=%s of=%s iflag=direct oflag=direct" %
              (mount_snapshot, mount_share)),
             ("dd if=%s of=%s count=1024 bs=1M iflag=direct oflag=direct" %
@@ -208,13 +215,13 @@ class LVMShareDriverTestCase(test.TestCase):
         self.assertEqual(expected_exec, fake_utils.fake_execute_get_log())
 
     def test_create_share_mirrors(self):
-
         share = fake_share(size='2048')
         CONF.set_default('lvm_share_mirrors', 2)
-        self._helper_nfs.create_export.return_value = 'fakelocation'
         self._driver._mount_device = mock.Mock()
+
         ret = self._driver.create_share(self._context, share,
                                         self.share_server)
+
         self._driver._mount_device.assert_called_with(
             share, '/dev/mapper/fakevg-fakename')
         expected_exec = [
@@ -222,7 +229,7 @@ class LVMShareDriverTestCase(test.TestCase):
             'mkfs.ext4 /dev/mapper/fakevg-fakename',
         ]
         self.assertEqual(expected_exec, fake_utils.fake_execute_get_log())
-        self.assertEqual('fakelocation', ret)
+        self.assertEqual(self._helper_nfs.create_exports.return_value, ret)
 
     def test_deallocate_container(self):
         expected_exec = ['lvremove -f fakevg/fakename']
@@ -318,9 +325,14 @@ class LVMShareDriverTestCase(test.TestCase):
     def test_create_snapshot(self):
         self._driver.create_snapshot(self._context, self.snapshot,
                                      self.share_server)
+        mount_path = self._get_mount_path(self.snapshot)
         expected_exec = [
             ("lvcreate -L 1G --name fakesnapshotname --snapshot "
              "%s/fakename" % (CONF.lvm_share_volume_group,)),
+            "tune2fs -U random /dev/mapper/fakevg-%s" % self.snapshot['name'],
+            "mkdir -p " + mount_path,
+            "mount /dev/mapper/fakevg-fakesnapshotname " + mount_path,
+            "chmod 777 " + mount_path,
         ]
         self.assertEqual(expected_exec, fake_utils.fake_execute_get_log())
 
@@ -333,7 +345,7 @@ class LVMShareDriverTestCase(test.TestCase):
                                       self.share_server)
             self._driver._mount_device.assert_called_with(self.share,
                                                           device_name)
-            self._helper_nfs.create_export.assert_called_once_with(
+            self._helper_nfs.create_exports.assert_called_once_with(
                 self.server, self.share['name'], recreate=True)
 
     def test_delete_share(self):
@@ -342,7 +354,10 @@ class LVMShareDriverTestCase(test.TestCase):
         self._driver._delete_share(self._context, self.share)
 
     def test_delete_snapshot(self):
-        expected_exec = ['lvremove -f fakevg/fakesnapshotname']
+        expected_exec = [
+            'umount -f ' + self._get_mount_path(self.snapshot),
+            'lvremove -f fakevg/fakesnapshotname',
+        ]
         self._driver.delete_snapshot(self._context, self.snapshot,
                                      self.share_server)
         self.assertEqual(expected_exec, fake_utils.fake_execute_get_log())
@@ -359,7 +374,7 @@ class LVMShareDriverTestCase(test.TestCase):
             mock.Mock(side_effect=exception.ProcessExecutionError))
 
         self._driver._delete_share(self._context, self.share)
-        self._helper_nfs.remove_export.assert_called_once_with(
+        self._helper_nfs.remove_exports.assert_called_once_with(
             self.server,
             self.share['name'])
 
@@ -513,7 +528,66 @@ class LVMShareDriverTestCase(test.TestCase):
         self.assertEqual('LVM', self._driver._stats['share_backend_name'])
         self.assertEqual('NFS_CIFS', self._driver._stats['storage_protocol'])
         self.assertEqual(50, self._driver._stats['reserved_percentage'])
-        self.assertIsNone(self._driver._stats['consistency_group_support'])
-        self.assertEqual(True, self._driver._stats['snapshot_support'])
+        self.assertTrue(self._driver._stats['snapshot_support'])
         self.assertEqual('LVMShareDriver', self._driver._stats['driver_name'])
         self.assertEqual('test-pool', self._driver._stats['pools'])
+
+    def test_revert_to_snapshot(self):
+        mock_update_access = self.mock_object(self._helper_nfs,
+                                              'update_access')
+        self._driver.revert_to_snapshot(self._context, self.snapshot,
+                                        [], self.share_server)
+        snap_lv = "%s/fakesnapshotname" % (CONF.lvm_share_volume_group)
+        share_lv = "%s/fakename" % (CONF.lvm_share_volume_group)
+        share_mount_path = self._get_mount_path(self.snapshot['share'])
+        snapshot_mount_path = self._get_mount_path(self.snapshot)
+        expected_exec = [
+            ('umount -f %s' % snapshot_mount_path),
+            ("lvconvert --merge %s" % snap_lv),
+            ("umount %s" % share_mount_path),
+            ("rmdir %s" % share_mount_path),
+            ("lvchange -an %s" % share_lv),
+            ("lvchange -ay %s" % share_lv),
+            ("lvcreate -L 1G --name fakesnapshotname --snapshot %s" %
+                share_lv),
+            ('tune2fs -U random /dev/mapper/%s-fakesnapshotname' %
+                CONF.lvm_share_volume_group),
+            ("mkdir -p %s" % share_mount_path),
+            ("mount /dev/mapper/%s-fakename %s" %
+                (CONF.lvm_share_volume_group, share_mount_path)),
+            ("chmod 777 %s" % share_mount_path),
+            ("mkdir -p %s" % snapshot_mount_path),
+            ("mount /dev/mapper/fakevg-fakesnapshotname "
+             "%s" % snapshot_mount_path),
+            ("chmod 777 %s" % snapshot_mount_path),
+        ]
+        self.assertEqual(expected_exec, fake_utils.fake_execute_get_log())
+        self.assertEqual(2, mock_update_access.call_count)
+
+    def test_snapshot_update_access(self):
+        access_rules = [{
+            'access_type': 'ip',
+            'access_to': '1.1.1.1',
+            'access_level': 'ro',
+        }]
+
+        add_rules = [{
+            'access_type': 'ip',
+            'access_to': '2.2.2.2',
+            'access_level': 'ro',
+        }]
+
+        delete_rules = [{
+            'access_type': 'ip',
+            'access_to': '3.3.3.3',
+            'access_level': 'ro',
+        }]
+
+        self._driver.snapshot_update_access(self._context, self.snapshot,
+                                            access_rules, add_rules,
+                                            delete_rules)
+
+        (self._driver._helpers[self.snapshot['share']['share_proto']].
+            update_access.assert_called_once_with(
+            self.server, self.snapshot['name'],
+            access_rules, add_rules=add_rules, delete_rules=delete_rules))
