@@ -125,6 +125,16 @@ class ManilaExceptionTestCase(test.TestCase):
         self.assertEqual(500, e.code)
         self.assertIn(reason, e.msg)
 
+    def test_snapshot_access_already_exists(self):
+        # Verify response code for exception.ShareSnapshotAccessExists
+        access_type = "fake_type"
+        access = "fake_access"
+        e = exception.ShareSnapshotAccessExists(access_type=access_type,
+                                                access=access)
+        self.assertEqual(400, e.code)
+        self.assertIn(access_type, e.msg)
+        self.assertIn(access, e.msg)
+
 
 class ManilaExceptionResponseCode400(test.TestCase):
 
@@ -241,6 +251,13 @@ class ManilaExceptionResponseCode400(test.TestCase):
         self.assertEqual(400, e.code)
         self.assertIn(reason, e.msg)
 
+    def test_invalid_share_snapshot_instance(self):
+        # Verify response code for exception.InvalidShareSnapshotInstance
+        reason = "fake_reason"
+        e = exception.InvalidShareSnapshotInstance(reason=reason)
+        self.assertEqual(400, e.code)
+        self.assertIn(reason, e.msg)
+
 
 class ManilaExceptionResponseCode403(test.TestCase):
 
@@ -452,6 +469,13 @@ class ManilaExceptionResponseCode404(test.TestCase):
         self.assertEqual(404, e.code)
         self.assertIn(share_type_name, e.msg)
 
+    def test_share_type_does_not_exist(self):
+        # verify response code for exception.ShareTypeDoesNotExist
+        share_type = "fake_share_type_1234"
+        e = exception.ShareTypeDoesNotExist(share_type=share_type)
+        self.assertEqual(404, e.code)
+        self.assertIn(share_type, e.msg)
+
     def test_share_type_extra_specs_not_found(self):
         # verify response code for exception.ShareTypeExtraSpecsNotFound
         share_type_id = "fake_share_type_id"
