@@ -16,19 +16,16 @@
 
 import datetime
 import errno
-import os
-import os.path
 import socket
-import tempfile
 import time
-import uuid
 
 import ddt
 import mock
 from oslo_config import cfg
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 import paramiko
-from six.moves import builtins
+from webob import exc
 
 import manila
 from manila.common import constants
@@ -41,221 +38,8 @@ from manila import utils
 CONF = cfg.CONF
 
 
-class GetFromPathTestCase(test.TestCase):
-    def test_tolerates_nones(self):
-        f = utils.get_from_path
-
-        input = []
-        self.assertEqual([], f(input, "a"))
-        self.assertEqual([], f(input, "a/b"))
-        self.assertEqual([], f(input, "a/b/c"))
-
-        input = [None]
-        self.assertEqual([], f(input, "a"))
-        self.assertEqual([], f(input, "a/b"))
-        self.assertEqual([], f(input, "a/b/c"))
-
-        input = [{'a': None}]
-        self.assertEqual([], f(input, "a"))
-        self.assertEqual([], f(input, "a/b"))
-        self.assertEqual([], f(input, "a/b/c"))
-
-        input = [{'a': {'b': None}}]
-        self.assertEqual([{'b': None}], f(input, "a"))
-        self.assertEqual([], f(input, "a/b"))
-        self.assertEqual([], f(input, "a/b/c"))
-
-        input = [{'a': {'b': {'c': None}}}]
-        self.assertEqual([{'b': {'c': None}}], f(input, "a"))
-        self.assertEqual([{'c': None}], f(input, "a/b"))
-        self.assertEqual([], f(input, "a/b/c"))
-
-        input = [{'a': {'b': {'c': None}}}, {'a': None}]
-        self.assertEqual([{'b': {'c': None}}], f(input, "a"))
-        self.assertEqual([{'c': None}], f(input, "a/b"))
-        self.assertEqual([], f(input, "a/b/c"))
-
-        input = [{'a': {'b': {'c': None}}}, {'a': {'b': None}}]
-        self.assertEqual([{'b': {'c': None}}, {'b': None}], f(input, "a"))
-        self.assertEqual([{'c': None}], f(input, "a/b"))
-        self.assertEqual([], f(input, "a/b/c"))
-
-    def test_does_select(self):
-        f = utils.get_from_path
-
-        input = [{'a': 'a_1'}]
-        self.assertEqual(['a_1'], f(input, "a"))
-        self.assertEqual([], f(input, "a/b"))
-        self.assertEqual([], f(input, "a/b/c"))
-
-        input = [{'a': {'b': 'b_1'}}]
-        self.assertEqual([{'b': 'b_1'}], f(input, "a"))
-        self.assertEqual(['b_1'], f(input, "a/b"))
-        self.assertEqual([], f(input, "a/b/c"))
-
-        input = [{'a': {'b': {'c': 'c_1'}}}]
-        self.assertEqual([{'b': {'c': 'c_1'}}], f(input, "a"))
-        self.assertEqual([{'c': 'c_1'}], f(input, "a/b"))
-        self.assertEqual(['c_1'], f(input, "a/b/c"))
-
-        input = [{'a': {'b': {'c': 'c_1'}}}, {'a': None}]
-        self.assertEqual([{'b': {'c': 'c_1'}}], f(input, "a"))
-        self.assertEqual([{'c': 'c_1'}], f(input, "a/b"))
-        self.assertEqual(['c_1'], f(input, "a/b/c"))
-
-        input = [{'a': {'b': {'c': 'c_1'}}},
-                 {'a': {'b': None}}]
-        self.assertEqual([{'b': {'c': 'c_1'}}, {'b': None}], f(input, "a"))
-        self.assertEqual([{'c': 'c_1'}], f(input, "a/b"))
-        self.assertEqual(['c_1'], f(input, "a/b/c"))
-
-        input = [{'a': {'b': {'c': 'c_1'}}},
-                 {'a': {'b': {'c': 'c_2'}}}]
-        self.assertEqual([{'b': {'c': 'c_1'}}, {'b': {'c': 'c_2'}}],
-                         f(input, "a"))
-        self.assertEqual([{'c': 'c_1'}, {'c': 'c_2'}], f(input, "a/b"))
-        self.assertEqual(['c_1', 'c_2'], f(input, "a/b/c"))
-
-        self.assertEqual([], f(input, "a/b/c/d"))
-        self.assertEqual([], f(input, "c/a/b/d"))
-        self.assertEqual([], f(input, "i/r/t"))
-
-    def test_flattens_lists(self):
-        f = utils.get_from_path
-
-        input = [{'a': [1, 2, 3]}]
-        self.assertEqual([1, 2, 3], f(input, "a"))
-        self.assertEqual([], f(input, "a/b"))
-        self.assertEqual([], f(input, "a/b/c"))
-
-        input = [{'a': {'b': [1, 2, 3]}}]
-        self.assertEqual([{'b': [1, 2, 3]}], f(input, "a"))
-        self.assertEqual([1, 2, 3], f(input, "a/b"))
-        self.assertEqual([], f(input, "a/b/c"))
-
-        input = [{'a': {'b': [1, 2, 3]}}, {'a': {'b': [4, 5, 6]}}]
-        self.assertEqual([1, 2, 3, 4, 5, 6], f(input, "a/b"))
-        self.assertEqual([], f(input, "a/b/c"))
-
-        input = [{'a': [{'b': [1, 2, 3]}, {'b': [4, 5, 6]}]}]
-        self.assertEqual([1, 2, 3, 4, 5, 6], f(input, "a/b"))
-        self.assertEqual([], f(input, "a/b/c"))
-
-        input = [{'a': [1, 2, {'b': 'b_1'}]}]
-        self.assertEqual([1, 2, {'b': 'b_1'}], f(input, "a"))
-        self.assertEqual(['b_1'], f(input, "a/b"))
-
-    def test_bad_xpath(self):
-        f = utils.get_from_path
-
-        self.assertRaises(exception.Error, f, [], None)
-        self.assertRaises(exception.Error, f, [], "")
-        self.assertRaises(exception.Error, f, [], "/")
-        self.assertRaises(exception.Error, f, [], "/a")
-        self.assertRaises(exception.Error, f, [], "/a/")
-        self.assertRaises(exception.Error, f, [], "//")
-        self.assertRaises(exception.Error, f, [], "//a")
-        self.assertRaises(exception.Error, f, [], "a//a")
-        self.assertRaises(exception.Error, f, [], "a//a/")
-        self.assertRaises(exception.Error, f, [], "a/a/")
-
-    def test_real_failure1(self):
-        # Real world failure case...
-        #  We weren't coping when the input was a Dictionary instead of a List
-        # This led to test_accepts_dictionaries
-        f = utils.get_from_path
-
-        inst = {'fixed_ip': {'floating_ips': [{'address': '1.2.3.4'}],
-                             'address': '192.168.0.3'},
-                'hostname': ''}
-
-        private_ips = f(inst, 'fixed_ip/address')
-        public_ips = f(inst, 'fixed_ip/floating_ips/address')
-        self.assertEqual(['192.168.0.3'], private_ips)
-        self.assertEqual(['1.2.3.4'], public_ips)
-
-    def test_accepts_dictionaries(self):
-        f = utils.get_from_path
-
-        input = {'a': [1, 2, 3]}
-        self.assertEqual([1, 2, 3], f(input, "a"))
-        self.assertEqual([], f(input, "a/b"))
-        self.assertEqual([], f(input, "a/b/c"))
-
-        input = {'a': {'b': [1, 2, 3]}}
-        self.assertEqual([{'b': [1, 2, 3]}], f(input, "a"))
-        self.assertEqual([1, 2, 3], f(input, "a/b"))
-        self.assertEqual([], f(input, "a/b/c"))
-
-        input = {'a': [{'b': [1, 2, 3]}, {'b': [4, 5, 6]}]}
-        self.assertEqual([1, 2, 3, 4, 5, 6], f(input, "a/b"))
-        self.assertEqual([], f(input, "a/b/c"))
-
-        input = {'a': [1, 2, {'b': 'b_1'}]}
-        self.assertEqual([1, 2, {'b': 'b_1'}], f(input, "a"))
-        self.assertEqual(['b_1'], f(input, "a/b"))
-
-
 @ddt.ddt
 class GenericUtilsTestCase(test.TestCase):
-    def test_read_cached_file(self):
-        cache_data = {"data": 1123, "mtime": 1}
-        with mock.patch.object(os.path, "getmtime", mock.Mock(return_value=1)):
-            data = utils.read_cached_file("/this/is/a/fake", cache_data)
-            self.assertEqual(cache_data["data"], data)
-            os.path.getmtime.assert_called_once_with("/this/is/a/fake")
-
-    def test_read_modified_cached_file(self):
-        with mock.patch.object(os.path, "getmtime", mock.Mock(return_value=2)):
-            fake_contents = "lorem ipsum"
-            fake_file = mock.Mock()
-            fake_file.read = mock.Mock(return_value=fake_contents)
-            fake_context_manager = mock.Mock()
-            fake_context_manager.__enter__ = mock.Mock(return_value=fake_file)
-            fake_context_manager.__exit__ = mock.Mock()
-            with mock.patch.object(
-                    builtins, 'open',
-                    mock.Mock(return_value=fake_context_manager)):
-                cache_data = {"data": 1123, "mtime": 1}
-                self.reload_called = False
-
-                def test_reload(reloaded_data):
-                    self.assertEqual(fake_contents, reloaded_data)
-                    self.reload_called = True
-
-                data = utils.read_cached_file("/this/is/a/fake",
-                                              cache_data,
-                                              reload_func=test_reload)
-                self.assertEqual(fake_contents, data)
-                self.assertTrue(self.reload_called)
-                fake_file.read.assert_called_once_with()
-                fake_context_manager.__enter__.assert_any_call()
-                builtins.open.assert_called_once_with("/this/is/a/fake")
-                os.path.getmtime.assert_called_once_with("/this/is/a/fake")
-
-    def test_read_file_as_root(self):
-        def fake_execute(*args, **kwargs):
-            if args[1] == 'bad':
-                raise exception.ProcessExecutionError
-            return 'fakecontents', None
-
-        self.mock_object(utils, 'execute', fake_execute)
-        contents = utils.read_file_as_root('good')
-        self.assertEqual('fakecontents', contents)
-        self.assertRaises(exception.FileNotFound,
-                          utils.read_file_as_root, 'bad')
-
-    def test_temporary_chown(self):
-        def fake_execute(*args, **kwargs):
-            if args[0] == 'chown':
-                fake_execute.uid = args[1]
-        self.mock_object(utils, 'execute', fake_execute)
-
-        with tempfile.NamedTemporaryFile() as f:
-            with utils.temporary_chown(f.name, owner_uid=2):
-                self.assertEqual(2, fake_execute.uid)
-            self.assertEqual(fake_execute.uid, os.getuid())
-
     def test_service_is_up(self):
         fts_func = datetime.datetime.fromtimestamp
         fake_now = 1000
@@ -289,35 +73,6 @@ class GenericUtilsTestCase(test.TestCase):
             self.assertFalse(result)
             timeutils.utcnow.assert_called_once_with()
 
-    def test_is_ipv6_configured0(self):
-        fake_fd = mock.Mock()
-        fake_fd.read.return_value = 'test'
-        with mock.patch('six.moves.builtins.open',
-                        mock.Mock(return_value=fake_fd)) as open:
-            self.assertTrue(utils.is_ipv6_configured())
-
-            open.assert_called_once_with('/proc/net/if_inet6')
-            fake_fd.read.assert_called_once_with(32)
-
-    def test_is_ipv6_configured1(self):
-        fake_fd = mock.Mock()
-        fake_fd.read.return_value = ''
-        with mock.patch(
-                'six.moves.builtins.open', mock.Mock(return_value=fake_fd)):
-            self.assertFalse(utils.is_ipv6_configured())
-
-    def test_is_ipv6_configured2(self):
-        with mock.patch('six.moves.builtins.open',
-                        mock.Mock(side_effect=IOError(
-                            errno.ENOENT, 'Fake no such file error.'))):
-            self.assertFalse(utils.is_ipv6_configured())
-
-    def test_is_ipv6_configured3(self):
-        with mock.patch('six.moves.builtins.open',
-                        mock.Mock(side_effect=IOError(
-                            errno.EPERM, 'Fake no such file error.'))):
-            self.assertRaises(IOError, utils.is_ipv6_configured)
-
     def test_is_eventlet_bug105(self):
         fake_dns = mock.Mock()
         fake_dns.getaddrinfo.side_effect = socket.gaierror(errno.EBADF)
@@ -428,26 +183,26 @@ class MonkeyPatchTestCase(test.TestCase):
 
         self.assertEqual(8, ret_b)
         package_a = self.example_package + 'example_a.'
-        self.assertTrue(package_a + 'example_function_a'
-                        in manila.tests.monkey_patch_example.CALLED_FUNCTION)
+        self.assertIn(package_a + 'example_function_a',
+                      manila.tests.monkey_patch_example.CALLED_FUNCTION)
 
-        self.assertTrue(package_a + 'ExampleClassA.example_method'
-                        in manila.tests.monkey_patch_example.CALLED_FUNCTION)
-        self.assertTrue(package_a + 'ExampleClassA.example_method_add'
-                        in manila.tests.monkey_patch_example.CALLED_FUNCTION)
+        self.assertIn(package_a + 'ExampleClassA.example_method',
+                      manila.tests.monkey_patch_example.CALLED_FUNCTION)
+        self.assertIn(package_a + 'ExampleClassA.example_method_add',
+                      manila.tests.monkey_patch_example.CALLED_FUNCTION)
         package_b = self.example_package + 'example_b.'
-        self.assertFalse(package_b + 'example_function_b'
-                         in manila.tests.monkey_patch_example.CALLED_FUNCTION)
-        self.assertFalse(package_b + 'ExampleClassB.example_method'
-                         in manila.tests.monkey_patch_example.CALLED_FUNCTION)
-        self.assertFalse(package_b + 'ExampleClassB.example_method_add'
-                         in manila.tests.monkey_patch_example.CALLED_FUNCTION)
+        self.assertNotIn(package_b + 'example_function_b',
+                         manila.tests.monkey_patch_example.CALLED_FUNCTION)
+        self.assertNotIn(package_b + 'ExampleClassB.example_method',
+                         manila.tests.monkey_patch_example.CALLED_FUNCTION)
+        self.assertNotIn(package_b + 'ExampleClassB.example_method_add',
+                         manila.tests.monkey_patch_example.CALLED_FUNCTION)
 
 
 class FakeSSHClient(object):
 
     def __init__(self):
-        self.id = uuid.uuid4()
+        self.id = uuidutils.generate_uuid()
         self.transport = FakeTransport()
 
     def set_missing_host_key_policy(self, policy):
@@ -624,6 +379,56 @@ class CidrToNetmaskTestCase(test.TestCase):
         self.assertRaises(exception.InvalidInput, utils.cidr_to_netmask, cidr)
 
 
+@ddt.ddt
+class ParseBoolValueTestCase(test.TestCase):
+
+    @ddt.data(
+        ('t', True),
+        ('on', True),
+        ('1', True),
+        ('false', False),
+        ('n', False),
+        ('no', False),
+        ('0', False),)
+    @ddt.unpack
+    def test_bool_with_valid_string(self, string, value):
+        fake_dict = {'fake_key': string}
+        result = utils.get_bool_from_api_params('fake_key', fake_dict)
+        self.assertEqual(value, result)
+
+    @ddt.data('None', 'invalid', 'falses')
+    def test_bool_with_invalid_string(self, string):
+        fake_dict = {'fake_key': string}
+        self.assertRaises(exc.HTTPBadRequest,
+                          utils.get_bool_from_api_params,
+                          'fake_key', fake_dict)
+
+    @ddt.data('undefined', None)
+    def test_bool_with_key_not_found_raise_error(self, def_val):
+        fake_dict = {'fake_key1': 'value1'}
+        self.assertRaises(exc.HTTPBadRequest,
+                          utils.get_bool_from_api_params,
+                          'fake_key2',
+                          fake_dict,
+                          def_val)
+
+    @ddt.data((False, False, False),
+              (True, True, False),
+              ('true', True, False),
+              ('false', False, False),
+              ('undefined', 'undefined', False),
+              (False, False, True),
+              ('true', True, True))
+    @ddt.unpack
+    def test_bool_with_key_not_found(self, def_val, expected, strict):
+        fake_dict = {'fake_key1': 'value1'}
+        invalid_default = utils.get_bool_from_api_params('fake_key2',
+                                                         fake_dict,
+                                                         def_val,
+                                                         strict)
+        self.assertEqual(expected, invalid_default)
+
+
 @ddt.ddt
 class IsValidIPVersion(test.TestCase):
     """Test suite for function 'is_valid_ip_address'."""
@@ -886,8 +691,14 @@ class ShareMigrationHelperTestCase(test.TestCase):
     def test_wait_for_access_update(self):
         sid = 1
         fake_share_instances = [
-            {'id': sid, 'access_rules_status': constants.STATUS_OUT_OF_SYNC},
-            {'id': sid, 'access_rules_status': constants.STATUS_ACTIVE},
+            {
+                'id': sid,
+                'access_rules_status': constants.SHARE_INSTANCE_RULES_SYNCING,
+            },
+            {
+                'id': sid,
+                'access_rules_status': constants.STATUS_ACTIVE,
+            },
         ]
 
         self.mock_object(time, 'sleep')
@@ -904,11 +715,17 @@ class ShareMigrationHelperTestCase(test.TestCase):
 
     @ddt.data(
         (
-            {'id': '1', 'access_rules_status': constants.STATUS_ERROR},
+            {
+                'id': '1',
+                'access_rules_status': constants.SHARE_INSTANCE_RULES_ERROR,
+            },
             exception.ShareMigrationFailed
         ),
         (
-            {'id': '1', 'access_rules_status': constants.STATUS_OUT_OF_SYNC},
+            {
+                'id': '1',
+                'access_rules_status': constants.SHARE_INSTANCE_RULES_SYNCING,
+            },
             exception.ShareMigrationFailed
         ),
     )
