@@ -17,16 +17,10 @@ from __future__ import print_function
 
 from oslo_config import cfg
 
-from tempest import config  # noqa
-
-service_available_group = cfg.OptGroup(name="service_available",
-                                       title="Available OpenStack Services")
-
-ServiceAvailableGroup = [
-    cfg.BoolOpt("manila",
-                default=True,
-                help="Whether or not manila is expected to be available"),
-]
+service_option = cfg.BoolOpt("manila",
+                             default=True,
+                             help="Whether or not manila is expected to be "
+                                  "available")
 
 share_group = cfg.OptGroup(name="share", title="Share Service Options")
 
@@ -36,7 +30,7 @@ ShareGroup = [
                help="The minimum api microversion is configured to be the "
                     "value of the minimum microversion supported by Manila."),
     cfg.StrOpt("max_api_microversion",
-               default="2.15",
+               default="2.32",
                help="The maximum api microversion is configured to be the "
                     "value of the latest microversion supported by Manila."),
     cfg.StrOpt("region",
@@ -58,6 +52,15 @@ ShareGroup = [
                 help="This option used to determine backend driver type, "
                      "multitenant driver uses share-networks, but "
                      "single-tenant doesn't."),
+    cfg.BoolOpt("create_networks_when_multitenancy_enabled",
+                default=True,
+                help="This option is used only when other "
+                     "'multitenancy_enabled' option is set to 'True'. "
+                     "If this one is set to True, then tempest will create "
+                     "neutron networks for each new manila share-network "
+                     "it creates. Else it will use manila share-networks with "
+                     "empty values (case of StandAloneNetworkPlugin and "
+                     "NeutronSingleNetworkPlugin)."),
     cfg.ListOpt("enable_protocols",
                 default=["nfs", "cifs"],
                 help="First value of list is protocol by default, "
@@ -93,7 +96,18 @@ ShareGroup = [
                      "capability called 'snapshot_support' and will be used "
                      "for setting up custom share type. Defaults to value of "
                      "other config option 'run_snapshot_tests'."),
-
+    cfg.BoolOpt("capability_create_share_from_snapshot_support",
+                help="Defines extra spec that satisfies specific back end "
+                     "capability called 'create_share_from_snapshot_support' "
+                     "and will be used for setting up a custom share type. "
+                     "Defaults to the value of run_snapshot_tests. Set it to "
+                     "False if the driver being tested does not support "
+                     "creating shares from snapshots."),
+    cfg.BoolOpt("capability_revert_to_snapshot_support",
+                help="Defines extra spec that satisfies specific back end "
+                     "capability called 'revert_to_snapshot_support' "
+                     "and will be used for setting up custom share type. "
+                     "Defaults to the value of run_revert_to_snapshot_tests."),
     cfg.StrOpt("share_network_id",
                default="",
                help="Some backend drivers requires share network "
@@ -151,19 +165,37 @@ ShareGroup = [
                 help="Defines whether to run tests that use share snapshots "
                      "or not. Disable this feature if used driver doesn't "
                      "support it."),
-    cfg.BoolOpt("run_consistency_group_tests",
+    cfg.BoolOpt("run_revert_to_snapshot_tests",
+                default=False,
+                help="Defines whether to run tests that revert shares "
+                     "to snapshots or not. Enable this feature if used "
+                     "driver supports it."),
+    cfg.BoolOpt("run_share_group_tests",
                 default=True,
-                help="Defines whether to run consistency group tests or not. "
-                     "Disable this feature if used driver doesn't support "
-                     "it."),
+                deprecated_name="run_consistency_group_tests",
+                help="Defines whether to run share group tests or not."),
     cfg.BoolOpt("run_replication_tests",
                 default=False,
                 help="Defines whether to run replication tests or not. "
                      "Enable this feature if the driver is configured "
                      "for replication."),
-    cfg.BoolOpt("run_migration_tests",
+    cfg.BoolOpt("run_multiple_share_replicas_tests",
+                default=True,
+                help="Defines whether to run multiple replicas creation test "
+                     "or not. Enable this if the driver can create more than "
+                     "one replica for a share."),
+    cfg.BoolOpt("run_host_assisted_migration_tests",
+                deprecated_name="run_migration_tests",
                 default=False,
-                help="Enable or disable migration tests."),
+                help="Enable or disable host-assisted migration tests."),
+    cfg.BoolOpt("run_driver_assisted_migration_tests",
+                deprecated_name="run_migration_tests",
+                default=False,
+                help="Enable or disable driver-assisted migration tests."),
+    cfg.BoolOpt("run_migration_with_preserve_snapshots_tests",
+                default=False,
+                help="Enable or disable migration with "
+                     "preserve_snapshots tests set to True."),
     cfg.BoolOpt("run_manage_unmanage_tests",
                 default=False,
                 help="Defines whether to run manage/unmanage tests or not. "
@@ -174,9 +206,12 @@ ShareGroup = [
                 help="Defines whether to run manage/unmanage snapshot tests "
                      "or not. These tests may leave orphaned resources, so be "
                      "careful enabling this opt."),
+    cfg.BoolOpt("run_mount_snapshot_tests",
+                default=False,
+                help="Enable or disable mountable snapshot tests."),
 
     cfg.StrOpt("image_with_share_tools",
-               default="manila-service-image",
+               default="manila-service-image-master",
                help="Image name for vm booting with nfs/smb clients tool."),
     cfg.StrOpt("image_username",
                default="manila",
@@ -197,4 +232,7 @@ ShareGroup = [
                default='none',
                choices=['none', 'writable', 'readable', 'dr'],
                help="Specify the replication type supported by the backend."),
+    cfg.IntOpt("share_size",
+               default=1,
+               help="Default size in GB for shares created by share tests."),
 ]
