@@ -13,40 +13,42 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from tempest import config  # noqa
-from tempest import test  # noqa
-from tempest_lib import exceptions as lib_exc  # noqa
-import testtools  # noqa
+import ddt
+from tempest import config
+from tempest.lib import exceptions as lib_exc
+from testtools import testcase as tc
 
-from manila_tempest_tests import clients_share as clients
 from manila_tempest_tests.tests.api import base
 
 CONF = config.CONF
 
 
+@ddt.ddt
 class SharesAdminQuotasNegativeTest(base.BaseSharesAdminTest):
 
     force_tenant_isolation = True
 
     @classmethod
     def resource_setup(cls):
-        cls.os = clients.AdminManager()
+        if not CONF.share.run_quota_tests:
+            msg = "Quota tests are disabled."
+            raise cls.skipException(msg)
         super(SharesAdminQuotasNegativeTest, cls).resource_setup()
         cls.user_id = cls.shares_client.user_id
         cls.tenant_id = cls.shares_client.tenant_id
 
-    @test.attr(type=["gate", "smoke", "negative"])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
     def test_get_quotas_with_empty_tenant_id(self):
         self.assertRaises(lib_exc.NotFound,
                           self.shares_client.show_quotas, "")
 
-    @test.attr(type=["gate", "smoke", "negative"])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
     def test_reset_quotas_with_empty_tenant_id(self):
         client = self.get_client_with_isolated_creds()
         self.assertRaises(lib_exc.NotFound,
                           client.reset_quotas, "")
 
-    @test.attr(type=["gate", "smoke", "negative"])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
     def test_update_shares_quota_with_wrong_data(self):
         # -1 is acceptable value as unlimited
         client = self.get_client_with_isolated_creds()
@@ -55,7 +57,7 @@ class SharesAdminQuotasNegativeTest(base.BaseSharesAdminTest):
                           client.tenant_id,
                           shares=-2)
 
-    @test.attr(type=["gate", "smoke", "negative"])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
     def test_update_snapshots_quota_with_wrong_data(self):
         # -1 is acceptable value as unlimited
         client = self.get_client_with_isolated_creds()
@@ -64,7 +66,7 @@ class SharesAdminQuotasNegativeTest(base.BaseSharesAdminTest):
                           client.tenant_id,
                           snapshots=-2)
 
-    @test.attr(type=["gate", "smoke", "negative"])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
     def test_update_gigabytes_quota_with_wrong_data(self):
         # -1 is acceptable value as unlimited
         client = self.get_client_with_isolated_creds()
@@ -73,7 +75,7 @@ class SharesAdminQuotasNegativeTest(base.BaseSharesAdminTest):
                           client.tenant_id,
                           gigabytes=-2)
 
-    @test.attr(type=["gate", "smoke", "negative"])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
     def test_update_snapshot_gigabytes_quota_with_wrong_data(self):
         # -1 is acceptable value as unlimited
         client = self.get_client_with_isolated_creds()
@@ -82,7 +84,7 @@ class SharesAdminQuotasNegativeTest(base.BaseSharesAdminTest):
                           client.tenant_id,
                           snapshot_gigabytes=-2)
 
-    @test.attr(type=["gate", "smoke", "negative"])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
     def test_update_share_networks_quota_with_wrong_data(self):
         # -1 is acceptable value as unlimited
         client = self.get_client_with_isolated_creds()
@@ -91,7 +93,7 @@ class SharesAdminQuotasNegativeTest(base.BaseSharesAdminTest):
                           client.tenant_id,
                           share_networks=-2)
 
-    @test.attr(type=["gate", "smoke", "negative"])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
     def test_create_share_with_size_bigger_than_quota(self):
         quotas = self.shares_client.show_quotas(
             self.shares_client.tenant_id)
@@ -99,10 +101,10 @@ class SharesAdminQuotasNegativeTest(base.BaseSharesAdminTest):
 
         # try schedule share with size, bigger than gigabytes quota
         self.assertRaises(lib_exc.OverLimit,
-                          self.shares_client.create_share,
+                          self.create_share,
                           size=overquota)
 
-    @test.attr(type=["gate", "smoke", "negative"])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
     def test_try_set_user_quota_shares_bigger_than_tenant_quota(self):
         client = self.get_client_with_isolated_creds()
 
@@ -117,7 +119,7 @@ class SharesAdminQuotasNegativeTest(base.BaseSharesAdminTest):
                           client.user_id,
                           shares=bigger_value)
 
-    @test.attr(type=["gate", "smoke", "negative"])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
     def test_try_set_user_quota_snaps_bigger_than_tenant_quota(self):
         client = self.get_client_with_isolated_creds()
 
@@ -132,7 +134,7 @@ class SharesAdminQuotasNegativeTest(base.BaseSharesAdminTest):
                           client.user_id,
                           snapshots=bigger_value)
 
-    @test.attr(type=["gate", "smoke", "negative"])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
     def test_try_set_user_quota_gigabytes_bigger_than_tenant_quota(self):
         client = self.get_client_with_isolated_creds()
 
@@ -147,7 +149,7 @@ class SharesAdminQuotasNegativeTest(base.BaseSharesAdminTest):
                           client.user_id,
                           gigabytes=bigger_value)
 
-    @test.attr(type=["gate", "smoke", "negative"])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
     def test_try_set_user_quota_snap_gigabytes_bigger_than_tenant_quota(self):
         client = self.get_client_with_isolated_creds()
 
@@ -162,7 +164,7 @@ class SharesAdminQuotasNegativeTest(base.BaseSharesAdminTest):
                           client.user_id,
                           snapshot_gigabytes=bigger_value)
 
-    @test.attr(type=["gate", "smoke", "negative"])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
     def test_try_set_user_quota_share_networks_bigger_than_tenant_quota(self):
         client = self.get_client_with_isolated_creds()
 
@@ -176,3 +178,40 @@ class SharesAdminQuotasNegativeTest(base.BaseSharesAdminTest):
                           client.tenant_id,
                           client.user_id,
                           share_networks=bigger_value)
+
+    @ddt.data(
+        ('quota-sets', '2.0', 'show_quotas'),
+        ('quota-sets', '2.0', 'default_quotas'),
+        ('quota-sets', '2.0', 'reset_quotas'),
+        ('quota-sets', '2.0', 'update_quotas'),
+        ('quota-sets', '2.6', 'show_quotas'),
+        ('quota-sets', '2.6', 'default_quotas'),
+        ('quota-sets', '2.6', 'reset_quotas'),
+        ('quota-sets', '2.6', 'update_quotas'),
+        ('os-quota-sets', '2.7', 'show_quotas'),
+        ('os-quota-sets', '2.7', 'default_quotas'),
+        ('os-quota-sets', '2.7', 'reset_quotas'),
+        ('os-quota-sets', '2.7', 'update_quotas'),
+    )
+    @ddt.unpack
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    @base.skip_if_microversion_not_supported("2.7")
+    def test_show_quotas_with_wrong_versions(self, url, version, method_name):
+        self.assertRaises(
+            lib_exc.NotFound,
+            getattr(self.shares_v2_client, method_name),
+            self.shares_v2_client.tenant_id,
+            version=version, url=url,
+        )
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    def test_show_quota_detail_with_wrong_versions(self):
+        version = '2.24'
+        url = 'quota-sets'
+
+        self.assertRaises(
+            lib_exc.NotFound,
+            self.shares_v2_client.detail_quotas,
+            self.shares_v2_client.tenant_id,
+            version=version, url=url,
+        )
