@@ -18,7 +18,7 @@ import re
 import six
 from tempest import config
 from tempest.lib import exceptions as lib_exc
-from tempest import test
+from testtools import testcase as tc
 
 from manila_tempest_tests.tests.api import base
 
@@ -50,10 +50,10 @@ class ShareServersAdminTest(base.BaseSharesAdminTest):
         cls.date_re = re.compile("^([0-9]{4}-[0-9]{2}-[0-9]{2}[A-Z]{1}"
                                  "[0-9]{2}:[0-9]{2}:[0-9]{2}).*$")
 
-    @test.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
+    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
     def test_list_share_servers_without_filters(self):
         servers = self.shares_client.list_share_servers()
-        self.assertTrue(len(servers) > 0)
+        self.assertGreater(len(servers), 0)
         keys = [
             "id",
             "host",
@@ -70,18 +70,18 @@ class ShareServersAdminTest(base.BaseSharesAdminTest):
             if server["updated_at"]:
                 self.assertTrue(self.date_re.match(server["updated_at"]))
             # Host is not empty
-            self.assertTrue(len(server["host"]) > 0)
+            self.assertGreater(len(server["host"]), 0)
             # Id is not empty
-            self.assertTrue(len(server["id"]) > 0)
+            self.assertGreater(len(server["id"]), 0)
             # Project id is not empty
-            self.assertTrue(len(server["project_id"]) > 0)
+            self.assertGreater(len(server["project_id"]), 0)
 
         # Do not verify statuses because we get all share servers from whole
         # cluster and here can be servers with any state.
         # Server we used is present.
         any(s["share_network_name"] in self.sn_name_and_id for s in servers)
 
-    @test.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
+    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
     def test_list_share_servers_with_host_filter(self):
         # Get list of share servers and remember 'host' name
         servers = self.shares_client.list_share_servers()
@@ -103,11 +103,11 @@ class ShareServersAdminTest(base.BaseSharesAdminTest):
             raise lib_exc.NotFound(message=msg)
         search_opts = {"host": host}
         servers = self.shares_client.list_share_servers(search_opts)
-        self.assertTrue(len(servers) > 0)
+        self.assertGreater(len(servers), 0)
         for server in servers:
             self.assertEqual(server["host"], host)
 
-    @test.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
+    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
     def test_list_share_servers_with_status_filter(self):
         # Get list of share servers
         servers = self.shares_client.list_share_servers()
@@ -129,41 +129,41 @@ class ShareServersAdminTest(base.BaseSharesAdminTest):
             raise lib_exc.NotFound(message=msg)
         search_opts = {"status": status}
         servers = self.shares_client.list_share_servers(search_opts)
-        self.assertTrue(len(servers) > 0)
+        self.assertGreater(len(servers), 0)
         for server in servers:
             self.assertEqual(server["status"], status)
 
-    @test.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
+    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
     def test_list_share_servers_with_project_id_filter(self):
         search_opts = {"project_id": self.share_network["project_id"]}
         servers = self.shares_client.list_share_servers(search_opts)
         # Should exist, at least, one share server, used by this test suite.
-        self.assertTrue(len(servers) > 0)
+        self.assertGreater(len(servers), 0)
         for server in servers:
             self.assertEqual(server["project_id"],
                              self.share_network["project_id"])
 
-    @test.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
+    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
     def test_list_share_servers_with_share_network_name_filter(self):
         search_opts = {"share_network": self.share_network["name"]}
         servers = self.shares_client.list_share_servers(search_opts)
         # Should exist, at least, one share server, used by this test suite.
-        self.assertTrue(len(servers) > 0)
+        self.assertGreater(len(servers), 0)
         for server in servers:
             self.assertEqual(server["share_network_name"],
                              self.share_network["name"])
 
-    @test.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
+    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
     def test_list_share_servers_with_share_network_id_filter(self):
         search_opts = {"share_network": self.share_network["id"]}
         servers = self.shares_client.list_share_servers(search_opts)
         # Should exist, at least, one share server, used by this test suite.
-        self.assertTrue(len(servers) > 0)
+        self.assertGreater(len(servers), 0)
         for server in servers:
             self.assertIn(server["share_network_name"],
                           self.sn_name_and_id)
 
-    @test.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
+    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
     def test_show_share_server(self):
         share = self.shares_client.get_share(self.share["id"])
         server = self.shares_client.show_share_server(share["share_server_id"])
@@ -186,19 +186,19 @@ class ShareServersAdminTest(base.BaseSharesAdminTest):
         if server["updated_at"]:
             self.assertTrue(self.date_re.match(server["updated_at"]))
         # Host is not empty
-        self.assertTrue(len(server["host"]) > 0)
+        self.assertGreater(len(server["host"]), 0)
         # Id is not empty
-        self.assertTrue(len(server["id"]) > 0)
+        self.assertGreater(len(server["id"]), 0)
         # Project id is not empty
-        self.assertTrue(len(server["project_id"]) > 0)
+        self.assertGreater(len(server["project_id"]), 0)
         # Status is not empty
-        self.assertTrue(len(server["status"]) > 0)
+        self.assertGreater(len(server["status"]), 0)
         # share_network_name is not empty
-        self.assertTrue(len(server["share_network_name"]) > 0)
+        self.assertGreater(len(server["share_network_name"]), 0)
         # backend_details should be a dict
         self.assertIsInstance(server["backend_details"], dict)
 
-    @test.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
+    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
     def test_show_share_server_details(self):
         servers = self.shares_client.list_share_servers()
         details = self.shares_client.show_share_server_details(
@@ -265,10 +265,10 @@ class ShareServersAdminTest(base.BaseSharesAdminTest):
                 self.shares_client.wait_for_resource_deletion(
                     sn_id=new_sn["id"])
 
-    @test.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
+    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
     def test_delete_share_server(self):
         self._delete_share_server(False)
 
-    @test.attr(type=[base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND])
+    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
     def test_delete_share_server_by_deletion_of_share_network(self):
         self._delete_share_server(True)
