@@ -13,10 +13,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from tempest import config  # noqa
-from tempest.lib import exceptions as lib_exc  # noqa
-from tempest import test  # noqa
-import testtools  # noqa
+from tempest import config
+from tempest.lib import exceptions as lib_exc
+import testtools
+from testtools import testcase as tc
 
 from manila_tempest_tests import share_exceptions
 from manila_tempest_tests.tests.api import base
@@ -31,95 +31,17 @@ class SharesNegativeTest(base.BaseSharesTest):
         cls.share = cls.create_share(
             name='public_share',
             description='public_share_desc',
-            size=1,
             is_public=True,
             metadata={'key': 'value'}
         )
 
-    @test.attr(type=["negative", "smoke", "gate", ])
-    def test_create_share_with_invalid_protocol(self):
-        self.assertRaises(lib_exc.BadRequest,
-                          self.shares_client.create_share,
-                          share_protocol="nonexistent_protocol")
-
-    @test.attr(type=["negative", "smoke", "gate", ])
-    def test_create_share_with_wrong_public_value(self):
-        self.assertRaises(lib_exc.BadRequest,
-                          self.shares_client.create_share, is_public='truebar')
-
-    @test.attr(type=["negative", "smoke", "gate", ])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
     def test_update_share_with_wrong_public_value(self):
         self.assertRaises(lib_exc.BadRequest,
                           self.shares_client.update_share, self.share["id"],
                           is_public="truebar")
 
-    @test.attr(type=["negative", "smoke", "gate", ])
-    def test_get_share_with_wrong_id(self):
-        self.assertRaises(lib_exc.NotFound, self.shares_client.get_share,
-                          "wrong_share_id")
-
-    @test.attr(type=["negative", "smoke", "gate", ])
-    def test_get_share_without_passing_share_id(self):
-        # Should not be able to get share when empty ID is passed
-        self.assertRaises(lib_exc.NotFound,
-                          self.shares_client.get_share, '')
-
-    @test.attr(type=["negative", "smoke", "gate", ])
-    def test_list_shares_nonadmin_with_nonexistent_share_server_filter(self):
-        # filtering by share server allowed only for admins by default
-        self.assertRaises(lib_exc.Forbidden,
-                          self.shares_client.list_shares_with_detail,
-                          {'share_server_id': 'fake_share_server_id'})
-
-    @test.attr(type=["negative", "smoke", "gate", ])
-    def test_delete_share_with_wrong_id(self):
-        self.assertRaises(lib_exc.NotFound, self.shares_client.delete_share,
-                          "wrong_share_id")
-
-    @test.attr(type=["negative", "smoke", "gate", ])
-    def test_delete_share_without_passing_share_id(self):
-        # Should not be able to delete share when empty ID is passed
-        self.assertRaises(lib_exc.NotFound,
-                          self.shares_client.delete_share, '')
-
-    @test.attr(type=["negative", "smoke", "gate", ])
-    @testtools.skipUnless(CONF.share.run_snapshot_tests,
-                          "Snapshot tests are disabled.")
-    def test_create_snapshot_with_wrong_id(self):
-        self.assertRaises(lib_exc.NotFound,
-                          self.shares_client.create_snapshot,
-                          "wrong_share_id")
-
-    @test.attr(type=["negative", "smoke", "gate", ])
-    @testtools.skipUnless(CONF.share.run_snapshot_tests,
-                          "Snapshot tests are disabled.")
-    def test_delete_snapshot_with_wrong_id(self):
-        self.assertRaises(lib_exc.NotFound,
-                          self.shares_client.delete_snapshot,
-                          "wrong_share_id")
-
-    @test.attr(type=["negative", "smoke", "gate", ])
-    def test_create_share_with_invalid_size(self):
-        self.assertRaises(lib_exc.BadRequest,
-                          self.shares_client.create_share, size="#$%")
-
-    @test.attr(type=["negative", "smoke", "gate", ])
-    def test_create_share_with_out_passing_size(self):
-        self.assertRaises(lib_exc.BadRequest,
-                          self.shares_client.create_share, size="")
-
-    @test.attr(type=["negative", "smoke", "gate", ])
-    def test_create_share_with_zero_size(self):
-        self.assertRaises(lib_exc.BadRequest,
-                          self.shares_client.create_share, size=0)
-
-    @test.attr(type=["negative", "gate", ])
-    def test_create_share_non_existent_az(self):
-        self.assertRaises(lib_exc.NotFound,
-                          self.shares_v2_client.create_share,
-                          availability_zone='fake_az')
-
-    @test.attr(type=["negative", "gate", ])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
     @testtools.skipUnless(CONF.share.run_snapshot_tests,
                           "Snapshot tests are disabled.")
     def test_try_delete_share_with_existing_snapshot(self):
@@ -135,16 +57,20 @@ class SharesNegativeTest(base.BaseSharesTest):
         self.assertRaises(lib_exc.Forbidden,
                           self.shares_client.delete_share, share["id"])
 
-    @test.attr(type=["negative", "gate", ])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
     @testtools.skipUnless(CONF.share.run_snapshot_tests,
                           "Snapshot tests are disabled.")
+    @testtools.skipUnless(
+        CONF.share.capability_create_share_from_snapshot_support,
+        "Create share from snapshot tests are disabled.")
     def test_create_share_from_snap_with_less_size(self):
         # requires minimum 5Gb available space
 
         skip_msg = "Check disc space for this test"
 
         try:  # create share
-            share = self.create_share(size=2, cleanup_in_class=False)
+            size = CONF.share.share_size + 1
+            share = self.create_share(size=size, cleanup_in_class=False)
         except share_exceptions.ShareBuildErrorException:
             self.skip(skip_msg)
 
@@ -157,10 +83,10 @@ class SharesNegativeTest(base.BaseSharesTest):
         # try create share from snapshot with less size
         self.assertRaises(lib_exc.BadRequest,
                           self.create_share,
-                          size=1, snapshot_id=snap["id"],
+                          snapshot_id=snap["id"],
                           cleanup_in_class=False)
 
-    @test.attr(type=["negative", "smoke", "gate", ])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
     @testtools.skipIf(not CONF.share.multitenancy_enabled,
                       "Only for multitenancy.")
     def test_create_share_with_nonexistant_share_network(self):
@@ -168,12 +94,18 @@ class SharesNegativeTest(base.BaseSharesTest):
                           self.shares_client.create_share,
                           share_network_id="wrong_sn_id")
 
-    @test.attr(type=["negative", "smoke", "gate", ])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
     @testtools.skipIf(not CONF.share.multitenancy_enabled,
                       "Only for multitenancy.")
     @testtools.skipUnless(CONF.share.run_snapshot_tests,
                           "Snapshot tests are disabled.")
+    @testtools.skipUnless(
+        CONF.share.capability_create_share_from_snapshot_support,
+        "Create share from snapshot tests are disabled.")
     def test_create_share_from_snap_with_different_share_network(self):
+        # We can't create a share from a snapshot whose base share does not
+        # have 'create_share_from_snapshot_support'.
+
         # create share
         share = self.create_share(cleanup_in_class=False)
 
@@ -203,14 +135,14 @@ class SharesNegativeTest(base.BaseSharesTest):
             snapshot_id=snap["id"],
         )
 
-    @test.attr(type=["gate", "smoke", "negative", ])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
     def test_update_other_tenants_public_share(self):
         isolated_client = self.get_client_with_isolated_creds(
             type_of_creds='alt')
         self.assertRaises(lib_exc.Forbidden, isolated_client.update_share,
                           self.share["id"], name="new_name")
 
-    @test.attr(type=["gate", "smoke", "negative", ])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
     def test_delete_other_tenants_public_share(self):
         isolated_client = self.get_client_with_isolated_creds(
             type_of_creds='alt')
@@ -218,7 +150,7 @@ class SharesNegativeTest(base.BaseSharesTest):
                           isolated_client.delete_share,
                           self.share['id'])
 
-    @test.attr(type=["gate", "smoke", "negative", ])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
     def test_set_metadata_of_other_tenants_public_share(self):
         isolated_client = self.get_client_with_isolated_creds(
             type_of_creds='alt')
@@ -227,7 +159,7 @@ class SharesNegativeTest(base.BaseSharesTest):
                           self.share['id'],
                           {'key': 'value'})
 
-    @test.attr(type=["gate", "smoke", "negative", ])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
     def test_update_metadata_of_other_tenants_public_share(self):
         isolated_client = self.get_client_with_isolated_creds(
             type_of_creds='alt')
@@ -236,7 +168,7 @@ class SharesNegativeTest(base.BaseSharesTest):
                           self.share['id'],
                           {'key': 'value'})
 
-    @test.attr(type=["gate", "smoke", "negative", ])
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
     def test_delete_metadata_of_other_tenants_public_share(self):
         isolated_client = self.get_client_with_isolated_creds(
             type_of_creds='alt')
@@ -245,21 +177,107 @@ class SharesNegativeTest(base.BaseSharesTest):
                           self.share['id'],
                           'key')
 
-    @test.attr(type=["gate", "smoke", "negative", ])
-    def test_list_by_share_server_by_user(self):
-        self.assertRaises(lib_exc.Forbidden,
-                          self.shares_client.list_shares,
-                          params={'share_server_id': 12345})
 
-    @test.attr(type=["gate", "smoke", "negative", ])
+class SharesAPIOnlyNegativeTest(base.BaseSharesTest):
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    def test_unmanage_share_by_user(self):
+        self.assertRaises(lib_exc.Forbidden,
+                          self.shares_client.unmanage_share,
+                          'fake-id')
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
     def test_manage_share_by_user(self):
         self.assertRaises(lib_exc.Forbidden,
                           self.shares_client.manage_share,
                           'fake-host', 'nfs', '/export/path',
                           'fake-type')
 
-    @test.attr(type=["gate", "smoke", "negative", ])
-    def test_unmanage_share_by_user(self):
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    def test_list_by_user_with_host_filter(self):
         self.assertRaises(lib_exc.Forbidden,
-                          self.shares_client.unmanage_share,
-                          'fake-id')
+                          self.shares_v2_client.list_shares,
+                          params={'host': 'fake_host'})
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    def test_list_by_share_server_by_user(self):
+        self.assertRaises(lib_exc.Forbidden,
+                          self.shares_client.list_shares,
+                          params={'share_server_id': 12345})
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    def test_create_share_non_existent_az(self):
+        self.assertRaises(lib_exc.NotFound,
+                          self.shares_v2_client.create_share,
+                          availability_zone='fake_az')
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    def test_create_share_with_zero_size(self):
+        self.assertRaises(lib_exc.BadRequest,
+                          self.shares_client.create_share, size=0)
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    def test_create_share_with_invalid_size(self):
+        self.assertRaises(lib_exc.BadRequest,
+                          self.shares_client.create_share, size="#$%")
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    def test_create_share_with_out_passing_size(self):
+        self.assertRaises(lib_exc.BadRequest,
+                          self.shares_client.create_share, size="")
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    @testtools.skipUnless(CONF.share.run_snapshot_tests,
+                          "Snapshot tests are disabled.")
+    def test_delete_snapshot_with_wrong_id(self):
+        self.assertRaises(lib_exc.NotFound,
+                          self.shares_client.delete_snapshot,
+                          "wrong_share_id")
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    @testtools.skipUnless(CONF.share.run_snapshot_tests,
+                          "Snapshot tests are disabled.")
+    def test_create_snapshot_with_wrong_id(self):
+        self.assertRaises(lib_exc.NotFound,
+                          self.shares_client.create_snapshot,
+                          "wrong_share_id")
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    def test_create_share_with_invalid_protocol(self):
+        self.assertRaises(lib_exc.BadRequest,
+                          self.shares_client.create_share,
+                          share_protocol="nonexistent_protocol")
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    def test_create_share_with_wrong_public_value(self):
+        self.assertRaises(lib_exc.BadRequest,
+                          self.shares_client.create_share, is_public='truebar')
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    def test_get_share_with_wrong_id(self):
+        self.assertRaises(lib_exc.NotFound, self.shares_client.get_share,
+                          "wrong_share_id")
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    def test_get_share_without_passing_share_id(self):
+        # Should not be able to get share when empty ID is passed
+        self.assertRaises(lib_exc.NotFound,
+                          self.shares_client.get_share, '')
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    def test_list_shares_nonadmin_with_nonexistent_share_server_filter(self):
+        # filtering by share server allowed only for admins by default
+        self.assertRaises(lib_exc.Forbidden,
+                          self.shares_client.list_shares_with_detail,
+                          {'share_server_id': 'fake_share_server_id'})
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    def test_delete_share_with_wrong_id(self):
+        self.assertRaises(lib_exc.NotFound, self.shares_client.delete_share,
+                          "wrong_share_id")
+
+    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
+    def test_delete_share_without_passing_share_id(self):
+        # Should not be able to delete share when empty ID is passed
+        self.assertRaises(lib_exc.NotFound,
+                          self.shares_client.delete_share, '')
