@@ -16,34 +16,47 @@
 from oslo_log import log
 import six
 
-from tempest.common.utils.linux import remote_client  # noqa
-from tempest import config  # noqa
+from tempest import config
 from tempest.lib.common.utils import data_utils
-from tempest.scenario import manager  # noqa
+from tempest.scenario import manager
 
-from manila_tempest_tests import clients_share
+from manila_tempest_tests.common import constants
+from manila_tempest_tests.common import remote_client
+from manila_tempest_tests.services.share.json import shares_client
+from manila_tempest_tests.services.share.v2.json import (
+    shares_client as shares_v2_client)
 
 CONF = config.CONF
-
 LOG = log.getLogger(__name__)
 
 
 class ShareScenarioTest(manager.NetworkScenarioTest):
     """Provide harness to do Manila scenario tests."""
 
+    credentials = ('admin', 'primary')
+
     @classmethod
     def resource_setup(cls):
         cls.set_network_resources()
         super(ShareScenarioTest, cls).resource_setup()
 
         # Manila clients
-        cls.shares_client = clients_share.Manager().shares_client
-        cls.shares_v2_client = clients_share.Manager().shares_v2_client
-        cls.shares_admin_client = clients_share.AdminManager().shares_client
-        cls.shares_admin_v2_client = (
-            clients_share.AdminManager().shares_v2_client)
+        cls.shares_client = shares_client.SharesClient(
+            cls.os_primary.auth_provider)
+        cls.shares_v2_client = shares_v2_client.SharesV2Client(
+            cls.os_primary.auth_provider)
+        cls.shares_admin_client = shares_client.SharesClient(
+            cls.os_admin.auth_provider)
+        cls.shares_admin_v2_client = shares_v2_client.SharesV2Client(
+            cls.os_admin.auth_provider)
 
-    def _create_share(self, share_protocol=None, size=1, name=None,
+    @classmethod
+    def skip_checks(cls):
+        super(ShareScenarioTest, cls).skip_checks()
+        if not CONF.service_available.manila:
+            raise cls.skipException("Manila support is required")
+
+    def _create_share(self, share_protocol=None, size=None, name=None,
                       snapshot_id=None, description=None, metadata=None,
                       share_network_id=None, share_type_id=None,
                       client=None, cleanup_in_class=True):
@@ -72,7 +85,7 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
         metadata = metadata or {}
         kwargs = {
             'share_protocol': share_protocol,
-            'size': size,
+            'size': size or CONF.share.share_size,
             'name': name,
             'snapshot_id': snapshot_id,
             'description': description,
@@ -90,6 +103,15 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
         client.wait_for_share_status(share['id'], 'available')
         return share
 
+    def _create_snapshot(self, share_id, client=None, **kwargs):
+        client = client or self.shares_v2_client
+        snapshot = client.create_snapshot(share_id, **kwargs)
+        self.addCleanup(
+            client.wait_for_resource_deletion, snapshot_id=snapshot['id'])
+        self.addCleanup(client.delete_snapshot, snapshot['id'])
+        client.wait_for_snapshot_status(snapshot["id"], "available")
+        return snapshot
+
     def _wait_for_share_server_deletion(self, sn_id, client=None):
         """Wait for a share server to be deleted
 
@@ -101,6 +123,7 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
             search_opts={"share_network": sn_id})
         for server in servers:
             client.delete_share_server(server['id'])
+        for server in servers:
             client.wait_for_resource_deletion(server_id=server['id'])
 
     def _create_share_network(self, client=None, **kwargs):
@@ -143,18 +166,35 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
             self.addCleanup(client.delete_access_rule, share_id, access['id'])
         return access
 
-    def _create_router_interface(self, subnet_id, client=None,
-                                 tenant_id=None, router_id=None):
+    def _allow_access_snapshot(self, snapshot_id, access_type="ip",
+                               access_to="0.0.0.0/0", cleanup=True):
+        """Allow snapshot access
+
+        :param snapshot_id: id of the snapshot
+        :param access_type: "ip", "user" or "cert"
+        :param access_to
+        :returns: access object
+        """
+        access = self.shares_v2_client.create_snapshot_access_rule(
+            snapshot_id, access_type, access_to)
+
+        if cleanup:
+            self.addCleanup(self.shares_v2_client.delete_snapshot_access_rule,
+                            snapshot_id, access['id'])
+
+        self.shares_v2_client.wait_for_snapshot_access_rule_status(
+            snapshot_id, access['id'])
+
+        return access
+
+    def _create_router_interface(self, subnet_id, client=None, router_id=None):
         """Create a router interface
 
         :param subnet_id: id of the subnet
         :param client: client object
-        :param tenant_id
         """
         if not client:
-            client = self.network_client
-        if not tenant_id:
-            tenant_id = client.tenant_id
+            client = self.routers_client
         if not router_id:
             router_id = self._get_router()['id']
         client.add_router_interface(router_id, subnet_id=subnet_id)
@@ -193,11 +233,22 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
 
         return linux_client
 
-    def _migrate_share(self, share_id, dest_host, client=None):
+    def _migrate_share(self, share_id, dest_host, status, force_host_assisted,
+                       client=None):
         client = client or self.shares_admin_v2_client
-        client.migrate_share(share_id, dest_host, True)
-        share = client.wait_for_migration_status(share_id, dest_host,
-                                                 'migration_success')
+        client.migrate_share(
+            share_id, dest_host, writable=False, preserve_metadata=False,
+            nondisruptive=False, preserve_snapshots=False,
+            force_host_assisted_migration=force_host_assisted)
+        share = client.wait_for_migration_status(share_id, dest_host, status)
+        return share
+
+    def _migration_complete(self, share_id, dest_host, client=None, **kwargs):
+        client = client or self.shares_admin_v2_client
+        client.migration_complete(share_id, **kwargs)
+        share = client.wait_for_migration_status(
+            share_id, dest_host, constants.TASK_STATE_MIGRATION_SUCCESS,
+            **kwargs)
         return share
 
     def _create_share_type(self, name, is_public=True, **kwargs):
